@@ -19,6 +19,7 @@
 #include <cstdlib> // getenv
 #include <cassert>
 
+#include <boost/config.hpp>
 #ifdef BOOST_NO_STDC_NAMESPACE
 namespace std {
     using ::getenv;
@@ -28,7 +29,7 @@ namespace std {
 namespace boost {
 namespace archive {
 
-char * tmpdir(){
+inline char * tmpdir(){
     char *dirname;
     dirname = std::getenv("TMP");
     if(NULL == dirname)
