@@ -4,9 +4,9 @@
 /* Copyright (c) 2002,2003 CrystalClear Software, Inc.
  * Use, modification and distribution is subject to the 
  * Boost Software License, Version 1.0. (See accompanying
- * file LICENSE-1.0 or http://www.boost.org/LICENSE-1.0)
+ * file LICENSE_1_0.txt or http://www.boost.org/LICENSE_1_0.txt)
  * Author: Jeff Garland, Bart Garst
- * $Date: 2004/07/18 23:37:24 $
+ * $Date: 2008-02-27 15:00:24 -0500 (Wed, 27 Feb 2008) $
  */
 
 #include "boost/date_time/gregorian/gregorian_types.hpp"
@@ -31,6 +31,7 @@ namespace gregorian {
     typedef boost::date_time::weekdays weekday_enum;
   };
 
+#if defined(USE_DATE_TIME_PRE_1_33_FACET_IO)
   //! Create the base facet type for gregorian::date
   typedef boost::date_time::date_names_put<greg_facet_config> greg_base_facet;
 
@@ -137,7 +138,8 @@ namespace gregorian {
   std::basic_ostream<charT, traits>&
   operator<<(std::basic_ostream<charT, traits>& os, const partial_date& pd)
   {
-    os << pd.day() << ' ' << pd.month().as_short_string() ; 
+    os << std::setw(2) << std::setfill('0') << pd.day() << ' ' 
+       << pd.month().as_short_string() ; 
     return os;
   }
 
@@ -199,7 +201,7 @@ namespace gregorian {
     os << fkb.day_of_week() << " before"; 
     return os;
   }
-
+#endif // USE_DATE_TIME_PRE_1_33_FACET_IO
   /**************** Input Streaming ******************/
   
 #if !defined(BOOST_NO_STD_ITERATOR_TRAITS)
@@ -279,7 +281,7 @@ namespace gregorian {
       const facet_def& f = std::use_facet<facet_def>(is.getloc());
       num = date_time::find_match(f.get_short_month_names(), 
                                   f.get_long_month_names(), 
-                                  greg_month::max(), s); 
+                                  (greg_month::max)(), s); 
     }
     /* bad_cast will be thrown if the desired facet is not accessible
      * so we can generate the facet. This has the drawback of using english
@@ -290,11 +292,12 @@ namespace gregorian {
       const facet_def* f = create_facet_def(a);
       num = date_time::find_match(f->get_short_month_names(), 
                                   f->get_long_month_names(), 
-                                  greg_month::max(), s); 
+                                  (greg_month::max)(), s); 
       delete(f);
     }
-   
-    m = greg_month(num +1); // months numbered 1-12
+    
+    num += 1; // months numbered 1-12
+    m = greg_month(num); 
 
     return is;
   }
@@ -320,7 +323,7 @@ namespace gregorian {
       const facet_def& f = std::use_facet<facet_def>(is.getloc());
       num = date_time::find_match(f.get_short_weekday_names(), 
                                   f.get_long_weekday_names(), 
-                                  greg_weekday::max(), s); 
+                                  (greg_weekday::max)(), s); 
     }
     /* bad_cast will be thrown if the desired facet is not accessible
      * so we can generate the facet. This has the drawback of using english
@@ -331,7 +334,7 @@ namespace gregorian {
       const facet_def* f = create_facet_def(a);
       num = date_time::find_match(f->get_short_weekday_names(), 
                                   f->get_long_weekday_names(), 
-                                  greg_weekday::max(), s); 
+                                  (greg_weekday::max)(), s); 
       delete(f);
     }
    
