@@ -2,25 +2,9 @@
 // Copyright 1997, 1998, 1999, 2000 University of Notre Dame.
 // Authors: Andrew Lumsdaine, Lie-Quan Lee, Jeremy G. Siek
 //
-// This file is part of the Boost Graph Library
-//
-// You should have received a copy of the License Agreement for the
-// Boost Graph Library along with the software; see the file LICENSE.
-// If not, contact Office of Research, University of Notre Dame, Notre
-// Dame, IN 46556.
-//
-// Permission to modify the code and to distribute modified code is
-// granted, provided the text of this NOTICE is retained, a notice that
-// the code was modified is included with the above COPYRIGHT NOTICE and
-// with the COPYRIGHT NOTICE in the LICENSE file, and that the LICENSE
-// file is distributed with the modified code.
-//
-// LICENSOR MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED.
-// By way of example, but not limitation, Licensor MAKES NO
-// REPRESENTATIONS OR WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY
-// PARTICULAR PURPOSE OR THAT THE USE OF THE LICENSED SOFTWARE COMPONENTS
-// OR DOCUMENTATION WILL NOT INFRINGE ANY PATENTS, COPYRIGHTS, TRADEMARKS
-// OR OTHER RIGHTS.
+// Distributed under the Boost Software License, Version 1.0. (See
+// accompanying file LICENSE_1_0.txt or copy at
+// http://www.boost.org/LICENSE_1_0.txt)
 //=======================================================================
 
 #ifndef BOOST_GRAPH_ADJACENCY_LIST_HPP
@@ -34,17 +18,28 @@
 #include <set>
 
 #if !defined BOOST_NO_HASH
-#include <hash_set>
+#  ifdef BOOST_HASH_SET_HEADER
+#    include BOOST_HASH_SET_HEADER
+#  else
+#    include <hash_set>
+#  endif
 #endif
 
 #if !defined BOOST_NO_SLIST
-#include <slist>
+#  ifdef BOOST_SLIST_HEADER
+#    include BOOST_SLIST_HEADER
+#  else
+#    include <slist>
+#  endif
 #endif
 
 #include <boost/graph/graph_traits.hpp>
 #include <boost/graph/graph_selectors.hpp>
 #include <boost/property_map.hpp>
-#include <boost/pending/ct_if.hpp>
+#include <boost/mpl/if.hpp>
+#include <boost/mpl/and.hpp>
+#include <boost/mpl/not.hpp>
+#include <boost/mpl/bool.hpp>
 #include <boost/graph/detail/edge.hpp>
 #include <boost/type_traits/is_same.hpp>
 #include <boost/detail/workaround.hpp>
@@ -127,7 +122,7 @@ namespace boost {
 #if !defined BOOST_NO_SLIST
   struct slistS {
     template <class T>
-    struct bind_ { typedef std::slist<T> type; };
+    struct bind_ { typedef BOOST_STD_EXTENSION_NAMESPACE::slist<T> type; };
   };
 #endif
 
@@ -246,12 +241,12 @@ namespace boost {
   namespace detail {
     template <class Directed> struct is_random_access { 
       enum { value = false};
-      typedef false_type type;
+      typedef mpl::false_ type;
     };
     template <>
     struct is_random_access<vecS> { 
       enum { value = true }; 
-      typedef true_type type;
+      typedef mpl::true_ type;
     };
 
   } // namespace detail
@@ -267,7 +262,8 @@ namespace boost {
 
   template <class OutEdgeListS = vecS,
             class VertexListS = vecS,
-            class DirectedS = directedS>
+            class DirectedS = directedS,
+            class EdgeListS = listS>
   struct adjacency_list_traits
   {
     typedef typename detail::is_random_access<VertexListS>::type
@@ -275,9 +271,9 @@ namespace boost {
     typedef typename DirectedS::is_bidir_t is_bidir;
     typedef typename DirectedS::is_directed_t is_directed;
 
-    typedef typename boost::ct_if_t<is_bidir,
+    typedef typename mpl::if_<is_bidir,
       bidirectional_tag,
-      typename boost::ct_if_t<is_directed,
+      typename mpl::if_<is_directed,
         directed_tag, undirected_tag
       >::type
     >::type directed_category;
@@ -286,10 +282,26 @@ namespace boost {
       edge_parallel_category;
 
     typedef void* vertex_ptr;
-    typedef typename boost::ct_if_t<is_rand_access,
+    typedef typename mpl::if_<is_rand_access,
       std::size_t, vertex_ptr>::type vertex_descriptor;
     typedef detail::edge_desc_impl<directed_category, vertex_descriptor>
       edge_descriptor;
+
+    typedef std::size_t vertices_size_type;
+
+  private:
+    // Logic to figure out the edges_size_type
+    struct dummy {};
+    typedef typename container_gen<EdgeListS, dummy>::type EdgeContainer;
+    typedef typename DirectedS::is_bidir_t BidirectionalT;
+    typedef typename DirectedS::is_directed_t DirectedT;
+    typedef typename mpl::and_<DirectedT, 
+      typename mpl::not_<BidirectionalT>::type >::type on_edge_storage;
+  public:
+    typedef typename mpl::if_<on_edge_storage,
+       std::size_t, typename EdgeContainer::size_type
+    >::type edges_size_type;
+
   };
 
 } // namespace boost
@@ -333,9 +345,6 @@ namespace boost {
       maybe_edge_bundled;
 #endif
 
-     struct no_vertex_bundle {};
-     struct no_edge_bundle {};
-
   public:
 #if !defined(BOOST_GRAPH_NO_BUNDLED_PROPERTIES)
     typedef typename detail::retag_property_list<vertex_bundle_t,
@@ -346,10 +355,10 @@ namespace boost {
       edge_property_type;
 
     // The types that are actually bundled
-    typedef typename ct_if<(is_same<maybe_vertex_bundled, no_property>::value),
+    typedef typename mpl::if_c<(is_same<maybe_vertex_bundled, no_property>::value),
                            no_vertex_bundle,
                            maybe_vertex_bundled>::type vertex_bundled;
-    typedef typename ct_if<(is_same<maybe_edge_bundled, no_property>::value),
+    typedef typename mpl::if_c<(is_same<maybe_edge_bundled, no_property>::value),
                            no_edge_bundle,
                            maybe_edge_bundled>::type edge_bundled;
 #else
@@ -373,6 +382,10 @@ namespace boost {
     typedef typename Base::degree_size_type degree_size_type;
     typedef typename Base::vertex_descriptor vertex_descriptor;
     typedef typename Base::edge_descriptor edge_descriptor;
+    typedef OutEdgeListS out_edge_list_selector;
+    typedef VertexListS vertex_list_selector;
+    typedef DirectedS directed_selector;
+    typedef EdgeListS edge_list_selector;
 
     typedef GraphProperty graph_property_type;
 
@@ -401,7 +414,7 @@ namespace boost {
     template <class EdgeIterator>
     inline adjacency_list(EdgeIterator first, EdgeIterator last,
                           vertices_size_type n,
-                          edges_size_type m = 0,
+                          edges_size_type = 0,
                           const GraphProperty& p = GraphProperty())
       : Base(n, first, last), m_property(p) { }
 
@@ -409,7 +422,7 @@ namespace boost {
     inline adjacency_list(EdgeIterator first, EdgeIterator last,
                           EdgePropertyIterator ep_iter,
                           vertices_size_type n,
-                          edges_size_type m = 0,
+                          edges_size_type = 0,
                           const GraphProperty& p = GraphProperty())
       : Base(n, first, last, ep_iter), m_property(p) { }
 #endif
@@ -440,28 +453,28 @@ namespace boost {
     GraphProperty m_property;
   };
 
-  template <class OEL, class VL, class DS, class VP,class EP, class GP,
+  template <class OEL, class VL, class DirS, class VP,class EP, class GP,
             class EL, class Tag, class Value>
   inline void
-  set_property(adjacency_list<OEL,VL,DS,VP,EP,GP,EL>& g, Tag,
+  set_property(adjacency_list<OEL,VL,DirS,VP,EP,GP,EL>& g, Tag,
                const Value& value) {
     get_property_value(g.m_property, Tag()) = value;;
   }
 
-  template <class OEL, class VL, class DS, class VP, class EP, class GP,
+  template <class OEL, class VL, class DirS, class VP, class EP, class GP,
             class Tag, class EL>
   inline
-  typename graph_property<adjacency_list<OEL,VL,DS,VP,EP,GP,EL>, Tag>::type&
-  get_property(adjacency_list<OEL,VL,DS,VP,EP,GP,EL>& g, Tag) {
+  typename graph_property<adjacency_list<OEL,VL,DirS,VP,EP,GP,EL>, Tag>::type&
+  get_property(adjacency_list<OEL,VL,DirS,VP,EP,GP,EL>& g, Tag) {
     return get_property_value(g.m_property, Tag());
   }
 
-  template <class OEL, class VL, class DS, class VP, class EP, class GP,
+  template <class OEL, class VL, class DirS, class VP, class EP, class GP,
             class Tag, class EL>
   inline
   const
-  typename graph_property<adjacency_list<OEL,VL,DS,VP,EP,GP,EL>, Tag>::type&
-  get_property(const adjacency_list<OEL,VL,DS,VP,EP,GP,EL>& g, Tag) {
+  typename graph_property<adjacency_list<OEL,VL,DirS,VP,EP,GP,EL>, Tag>::type&
+  get_property(const adjacency_list<OEL,VL,DirS,VP,EP,GP,EL>& g, Tag) {
     return get_property_value(g.m_property, Tag());
   }
 
