@@ -1,9 +1,9 @@
 //  boost sinc.hpp header file
 
-//  (C) Copyright Hubert Holin 2001. Permission to copy, use, modify, sell and
-//  distribute this software is granted provided this copyright notice appears
-//  in all copies. This software is provided "as is" without express or implied
-//  warranty, and with no claim as to its suitability for any purpose.
+//  (C) Copyright Hubert Holin 2001.
+//  Distributed under the Boost Software License, Version 1.0. (See
+//  accompanying file LICENSE_1_0.txt or copy at
+//  http://www.boost.org/LICENSE_1_0.txt)
 
 // See http://www.boost.org for updates, documentation, and revision history.
 
@@ -11,6 +11,10 @@
 #define BOOST_SINC_HPP
 
 
+#include <boost/math/tools/config.hpp>
+#include <boost/math/tools/precision.hpp>
+#include <boost/math/policies/policy.hpp>
+#include <boost/math/special_functions/math_fwd.hpp>
 #include <cmath>
 #include <boost/limits.hpp>
 #include <string>
@@ -26,6 +30,8 @@ namespace boost
 {
     namespace math
     {
+       namespace detail
+       {
 #if        defined(__GNUC__) && (__GNUC__ < 3)
         // gcc 2.x ignores function scope using declarations,
         // put them in the scope of the enclosing namespace instead:
@@ -40,9 +46,9 @@ namespace boost
         // This is the "Sinus Cardinal" of index Pi.
 
         template<typename T>
-        inline T    sinc_pi(const T x)
+        inline T    sinc_pi_imp(const T x)
         {
-#ifdef    BOOST_NO_STDC_NAMESPACE
+#if defined(BOOST_NO_STDC_NAMESPACE) && !defined(__SUNPRO_CC)
             using    ::abs;
             using    ::sin;
             using    ::sqrt;
@@ -52,9 +58,8 @@ namespace boost
             using    ::std::sqrt;
 #endif    /* BOOST_NO_STDC_NAMESPACE */
 
-            using    ::std::numeric_limits;
-
-            static T const    taylor_0_bound = numeric_limits<T>::epsilon();
+            // Note: this code is *not* thread safe!
+            static T const    taylor_0_bound = tools::epsilon<T>();
             static T const    taylor_2_bound = sqrt(taylor_0_bound);
             static T const    taylor_n_bound = sqrt(taylor_2_bound);
 
@@ -85,6 +90,21 @@ namespace boost
             }
         }
 
+       } // namespace detail
+
+       template <class T>
+       inline typename tools::promote_args<T>::type sinc_pi(T x)
+       {
+          typedef typename tools::promote_args<T>::type result_type;
+          return detail::sinc_pi_imp(static_cast<result_type>(x));
+       }
+
+       template <class T, class Policy>
+       inline typename tools::promote_args<T>::type sinc_pi(T x, const Policy&)
+       {
+          typedef typename tools::promote_args<T>::type result_type;
+          return detail::sinc_pi_imp(static_cast<result_type>(x));
+       }
 
 #ifdef    BOOST_NO_TEMPLATE_TEMPLATES
 #else    /* BOOST_NO_TEMPLATE_TEMPLATES */
@@ -105,7 +125,7 @@ namespace boost
 
             using    ::std::numeric_limits;
 
-            static T const    taylor_0_bound = numeric_limits<T>::epsilon();
+            static T const    taylor_0_bound = tools::epsilon<T>();
             static T const    taylor_2_bound = sqrt(taylor_0_bound);
             static T const    taylor_n_bound = sqrt(taylor_2_bound);
 
@@ -139,6 +159,12 @@ namespace boost
                 return(result);
             }
         }
+
+        template<typename T, template<typename> class U, class Policy>
+        inline U<T>    sinc_pi(const U<T> x, const Policy&)
+        {
+           return sinc_pi(x);
+        }
 #endif    /* BOOST_NO_TEMPLATE_TEMPLATES */
     }
 }
