@@ -1,4 +1,4 @@
-/* Copyright 2003-2004 Joaqun M Lpez Muoz.
+/* Copyright 2003-2006 Joaqun M Lpez Muoz.
  * Distributed under the Boost Software License, Version 1.0.
  * (See accompanying file LICENSE_1_0.txt or copy at
  * http://www.boost.org/LICENSE_1_0.txt)
@@ -9,6 +9,10 @@
 #ifndef BOOST_MULTI_INDEX_SAFE_MODE_ERRORS_HPP
 #define BOOST_MULTI_INDEX_SAFE_MODE_ERRORS_HPP
 
+#if defined(_MSC_VER)&&(_MSC_VER>=1200)
+#pragma once
+#endif
+
 namespace boost{
 
 namespace multi_index{
@@ -31,6 +35,7 @@ enum error_code
   not_same_owner,
   invalid_range,
   inside_range,
+  out_of_bounds,
   same_container
 };
 
