@@ -1,4 +1,4 @@
-/* Copyright 2003-2004 Joaqun M Lpez Muoz.
+/* Copyright 2003-2007 Joaqun M Lpez Muoz.
  * Distributed under the Boost Software License, Version 1.0.
  * (See accompanying file LICENSE_1_0.txt or copy at
  * http://www.boost.org/LICENSE_1_0.txt)
@@ -9,24 +9,38 @@
 #ifndef BOOST_MULTI_INDEX_SEQUENCED_INDEX_HPP
 #define BOOST_MULTI_INDEX_SEQUENCED_INDEX_HPP
 
+#if defined(_MSC_VER)&&(_MSC_VER>=1200)
+#pragma once
+#endif
+
 #include <boost/config.hpp> /* keep it first to prevent nasty warns in MSVC */
 #include <boost/call_traits.hpp>
+#include <boost/detail/allocator_utilities.hpp>
 #include <boost/detail/no_exceptions_support.hpp>
 #include <boost/detail/workaround.hpp>
 #include <boost/iterator/reverse_iterator.hpp>
+#include <boost/mpl/bool.hpp>
+#include <boost/mpl/not.hpp>
 #include <boost/mpl/push_front.hpp>
 #include <boost/multi_index/detail/access_specifier.hpp>
-#include <boost/multi_index/detail/index_iterator.hpp>
-#include <boost/multi_index/detail/seq_index_node.hpp>
-#include <boost/multi_index/detail/seq_index_ops.hpp>
+#include <boost/multi_index/detail/bidir_node_iterator.hpp>
+#include <boost/multi_index/detail/index_node_base.hpp>
+#include <boost/multi_index/detail/safe_ctr_proxy.hpp>
 #include <boost/multi_index/detail/safe_mode.hpp>
 #include <boost/multi_index/detail/scope_guard.hpp>
+#include <boost/multi_index/detail/seq_index_node.hpp>
+#include <boost/multi_index/detail/seq_index_ops.hpp>
 #include <boost/multi_index/sequenced_index_fwd.hpp>
 #include <boost/tuple/tuple.hpp>
+#include <boost/type_traits/is_integral.hpp>
 #include <cstddef>
 #include <functional>
 #include <utility>
 
+#if !defined(BOOST_MULTI_INDEX_DISABLE_SERIALIZATION)
+#include <boost/bind.hpp>
+#endif
+
 #if defined(BOOST_MULTI_INDEX_ENABLE_INVARIANT_CHECKING)
 #define BOOST_MULTI_INDEX_SEQ_INDEX_CHECK_INVARIANT                          \
   detail::scope_guard BOOST_JOIN(check_invariant_,__LINE__)=                 \
@@ -44,21 +58,20 @@ namespace detail{
 
 /* sequenced_index adds a layer of sequenced indexing to a given Super */
 
-template<typename Super,typename TagList>
+template<typename SuperMeta,typename TagList>
+class sequenced_index:
+  BOOST_MULTI_INDEX_PROTECTED_IF_MEMBER_TEMPLATE_FRIENDS SuperMeta::type
 
 #if defined(BOOST_MULTI_INDEX_ENABLE_SAFE_MODE)
 #if BOOST_WORKAROUND(BOOST_MSVC,<1300)
-class sequenced_index:
-  BOOST_MULTI_INDEX_PROTECTED_IF_MEMBER_TEMPLATE_FRIENDS Super,
-  public index_proxy<sequenced_index_node<typename Super::node_type> >
+  ,public safe_ctr_proxy_impl<
+    bidir_node_iterator<
+      sequenced_index_node<typename SuperMeta::type::node_type> >,
+    sequenced_index<SuperMeta,TagList> >
 #else
-class sequenced_index:
-  BOOST_MULTI_INDEX_PROTECTED_IF_MEMBER_TEMPLATE_FRIENDS Super,
-  public safe_container<sequenced_index<Super,TagList> >
+  ,public safe_mode::safe_container<
+    sequenced_index<SuperMeta,TagList> >
 #endif
-#else
-class sequenced_index:
-  BOOST_MULTI_INDEX_PROTECTED_IF_MEMBER_TEMPLATE_FRIENDS Super
 #endif
 
 { 
@@ -72,64 +85,81 @@ class sequenced_index:
 #pragma parse_mfunc_templ off
 #endif
 
-protected:
-  typedef sequenced_index_node<typename Super::node_type> node_type;
+  typedef typename SuperMeta::type                    super;
 
+protected:
+  typedef sequenced_index_node<
+    typename super::node_type>                        node_type;
+
+private:
+  typedef typename node_type::impl_type               node_impl_type;
+ 
 public:
   /* types */
 
-  typedef typename node_type::value_type             value_type;
-  typedef tuples::null_type                          ctor_args;
-  typedef typename Super::final_allocator_type       allocator_type;
-  typedef typename allocator_type::reference         reference;
-  typedef typename allocator_type::const_reference   const_reference;
+  typedef typename node_type::value_type              value_type;
+  typedef tuples::null_type                           ctor_args;
+  typedef typename super::final_allocator_type        allocator_type;
+  typedef typename allocator_type::reference          reference;
+  typedef typename allocator_type::const_reference    const_reference;
 
 #if defined(BOOST_MULTI_INDEX_ENABLE_SAFE_MODE)
 #if BOOST_WORKAROUND(BOOST_MSVC,<1300)
-  typedef index_iterator<node_type>                  iterator;
-  typedef index_iterator<node_type>                  const_iterator;
+  typedef safe_mode::safe_iterator<
+    bidir_node_iterator<node_type>,
+    safe_ctr_proxy<
+      bidir_node_iterator<node_type> > >              iterator;
 #else
-  typedef index_iterator<node_type,sequenced_index>  iterator;
-  typedef index_iterator<node_type,sequenced_index>  const_iterator;
+  typedef safe_mode::safe_iterator<
+    bidir_node_iterator<node_type>,
+    sequenced_index>                                  iterator;
 #endif
 #else
-  typedef index_iterator<node_type>                  iterator;
-  typedef index_iterator<node_type>                  const_iterator;
+  typedef bidir_node_iterator<node_type>              iterator;
 #endif
 
-  typedef std::size_t                                size_type;      
-  typedef std::ptrdiff_t                             difference_type;
-  typedef typename allocator_type::pointer           pointer;
-  typedef typename allocator_type::const_pointer     const_pointer;
+  typedef iterator                                    const_iterator;
+
+  typedef std::size_t                                 size_type;      
+  typedef std::ptrdiff_t                              difference_type;
+  typedef typename allocator_type::pointer            pointer;
+  typedef typename allocator_type::const_pointer      const_pointer;
   typedef typename
-    boost::reverse_iterator<iterator>                reverse_iterator;
+    boost::reverse_iterator<iterator>                 reverse_iterator;
   typedef typename
-    boost::reverse_iterator<const_iterator>          const_reverse_iterator;
-  typedef typename TagList::type                     tag_list;
+    boost::reverse_iterator<const_iterator>           const_reverse_iterator;
+  typedef TagList                                     tag_list;
 
 protected:
-  typedef typename Super::final_node_type     final_node_type;
+  typedef typename super::final_node_type     final_node_type;
   typedef tuples::cons<
     ctor_args, 
-    typename Super::ctor_args_list>           ctor_args_list;
+    typename super::ctor_args_list>           ctor_args_list;
   typedef typename mpl::push_front<
-    typename Super::index_type_list,
+    typename super::index_type_list,
     sequenced_index>::type                    index_type_list;
   typedef typename mpl::push_front<
-    typename Super::iterator_type_list,
+    typename super::iterator_type_list,
     iterator>::type                           iterator_type_list;
   typedef typename mpl::push_front<
-    typename Super::const_iterator_type_list,
+    typename super::const_iterator_type_list,
     const_iterator>::type                     const_iterator_type_list;
-  typedef typename Super::copy_map_type       copy_map_type;
+  typedef typename super::copy_map_type       copy_map_type;
+
+#if !defined(BOOST_MULTI_INDEX_DISABLE_SERIALIZATION)
+  typedef typename super::index_saver_type    index_saver_type;
+  typedef typename super::index_loader_type   index_loader_type;
+#endif
 
 private:
 #if defined(BOOST_MULTI_INDEX_ENABLE_SAFE_MODE)
 #if BOOST_WORKAROUND(BOOST_MSVC,<1300)
-  typedef index_proxy<sequenced_index_node<
-      typename Super::node_type> >           safe_super;
+  typedef safe_ctr_proxy_impl<
+    bidir_node_iterator<node_type>,
+    sequenced_index>                          safe_super;
 #else
-  typedef safe_container<sequenced_index>    safe_super;
+  typedef safe_mode::safe_container<
+    sequenced_index>                          safe_super;
 #endif
 #endif
 
@@ -142,8 +172,8 @@ public:
    * not supposed to be created on their own. No range ctor either.
    */
 
-  sequenced_index<Super,TagList>& operator=(
-    const sequenced_index<Super,TagList>& x)
+  sequenced_index<SuperMeta,TagList>& operator=(
+    const sequenced_index<SuperMeta,TagList>& x)
   {
     this->final()=x.final();
     return *this;
@@ -152,9 +182,7 @@ public:
   template <class InputIterator>
   void assign(InputIterator first,InputIterator last)
   {
-    BOOST_MULTI_INDEX_SEQ_INDEX_CHECK_INVARIANT;
-    clear();
-    for(;first!=last;++first)push_back(*first);
+    assign_iter(first,last,mpl::not_<is_integral<InputIterator> >());
   }
 
   void assign(size_type n,value_param_type value)
@@ -181,6 +209,20 @@ public:
   const_reverse_iterator rbegin()const{return make_reverse_iterator(end());}
   reverse_iterator       rend(){return make_reverse_iterator(begin());}
   const_reverse_iterator rend()const{return make_reverse_iterator(begin());}
+  const_iterator         cbegin()const{return begin();}
+  const_iterator         cend()const{return end();}
+  const_reverse_iterator crbegin()const{return rbegin();}
+  const_reverse_iterator crend()const{return rend();}
+
+  iterator iterator_to(const value_type& x)
+  {
+    return make_iterator(node_from_value<node_type>(&x));
+  }
+
+  const_iterator iterator_to(const value_type& x)const
+  {
+    return make_iterator(node_from_value<node_type>(&x));
+  }
 
   /* capacity */
 
@@ -193,8 +235,15 @@ public:
     BOOST_MULTI_INDEX_SEQ_INDEX_CHECK_INVARIANT;
     if(n>size())insert(end(),n-size(),x);
     else if(n<size()){
-      iterator it=begin();
-      std::advance(it,n);
+      iterator it;
+      if(n<=size()/2){
+        it=begin();
+        std::advance(it,n);
+      }
+      else{
+        it=end();
+        for(size_type m=size()-n;m--;--it){}
+      }
       erase(it,end());
     }   
   }
@@ -221,7 +270,9 @@ public:
     BOOST_MULTI_INDEX_CHECK_IS_OWNER(position,*this);
     BOOST_MULTI_INDEX_SEQ_INDEX_CHECK_INVARIANT;
     std::pair<final_node_type*,bool> p=this->final_insert_(x);
-    if(p.second)relink(position.get_node(),p.first);
+    if(p.second&&position.get_node()!=header()){
+      relink(position.get_node(),p.first);
+    }
     return std::pair<iterator,bool>(make_iterator(p.first),p.second);
   }
 
@@ -236,8 +287,7 @@ public:
   template<typename InputIterator>
   void insert(iterator position,InputIterator first,InputIterator last)
   {
-    BOOST_MULTI_INDEX_SEQ_INDEX_CHECK_INVARIANT;
-    for(;first!=last;++first)insert(position,*first);
+    insert_iter(position,first,last,mpl::not_<is_integral<InputIterator> >());
   }
 
   iterator erase(iterator position)
@@ -295,7 +345,28 @@ public:
       mod,static_cast<final_node_type*>(position.get_node()));
   }
 
-  void swap(sequenced_index<Super,TagList>& x)
+  template<typename Modifier,typename Rollback>
+  bool modify(iterator position,Modifier mod,Rollback back)
+  {
+    BOOST_MULTI_INDEX_CHECK_VALID_ITERATOR(position);
+    BOOST_MULTI_INDEX_CHECK_DEREFERENCEABLE_ITERATOR(position);
+    BOOST_MULTI_INDEX_CHECK_IS_OWNER(position,*this);
+    BOOST_MULTI_INDEX_SEQ_INDEX_CHECK_INVARIANT;
+
+#if defined(BOOST_MULTI_INDEX_ENABLE_SAFE_MODE)
+    /* MSVC++ 6.0 optimizer on safe mode code chokes if this
+     * this is not added. Left it for all compilers as it does no
+     * harm.
+     */
+
+    position.detach();
+#endif
+
+    return this->final_modify_(
+      mod,back,static_cast<final_node_type*>(position.get_node()));
+  }
+
+  void swap(sequenced_index<SuperMeta,TagList>& x)
   {
     BOOST_MULTI_INDEX_SEQ_INDEX_CHECK_INVARIANT;
     this->final_swap_(x.final());
@@ -304,12 +375,12 @@ public:
   void clear()
   {
     BOOST_MULTI_INDEX_SEQ_INDEX_CHECK_INVARIANT;
-    erase(begin(),end());
+    this->final_clear_();
   }
 
   /* list operations */
 
-  void splice(iterator position,sequenced_index<Super,TagList>& x)
+  void splice(iterator position,sequenced_index<SuperMeta,TagList>& x)
   {
     BOOST_MULTI_INDEX_CHECK_VALID_ITERATOR(position);
     BOOST_MULTI_INDEX_CHECK_IS_OWNER(position,*this);
@@ -317,12 +388,12 @@ public:
     BOOST_MULTI_INDEX_SEQ_INDEX_CHECK_INVARIANT;
     iterator first=x.begin(),last=x.end();
     while(first!=last){
-      if(insert(position,*first).second)x.erase(first++);
+      if(insert(position,*first).second)first=x.erase(first);
       else ++first;
     }
   }
 
-  void splice(iterator position,sequenced_index<Super,TagList>& x,iterator i)
+  void splice(iterator position,sequenced_index<SuperMeta,TagList>& x,iterator i)
   {
     BOOST_MULTI_INDEX_CHECK_VALID_ITERATOR(position);
     BOOST_MULTI_INDEX_CHECK_IS_OWNER(position,*this);
@@ -330,7 +401,7 @@ public:
     BOOST_MULTI_INDEX_CHECK_DEREFERENCEABLE_ITERATOR(i);
     BOOST_MULTI_INDEX_CHECK_IS_OWNER(i,x);
     BOOST_MULTI_INDEX_SEQ_INDEX_CHECK_INVARIANT;
-    if(x==*this){
+    if(&x==this){
       if(position!=i)relink(position.get_node(),i.get_node());
     }
     else{
@@ -352,7 +423,7 @@ public:
   }
 
   void splice(
-    iterator position,sequenced_index<Super,TagList>& x,
+    iterator position,sequenced_index<SuperMeta,TagList>& x,
     iterator first,iterator last)
   {
     BOOST_MULTI_INDEX_CHECK_VALID_ITERATOR(position);
@@ -363,14 +434,14 @@ public:
     BOOST_MULTI_INDEX_CHECK_IS_OWNER(last,x);
     BOOST_MULTI_INDEX_CHECK_VALID_RANGE(first,last);
     BOOST_MULTI_INDEX_SEQ_INDEX_CHECK_INVARIANT;
-    if(x==*this){
+    if(&x==this){
       BOOST_MULTI_INDEX_CHECK_OUTSIDE_RANGE(position,first,last);
       if(position!=last)relink(
         position.get_node(),first.get_node(),last.get_node());
     }
     else{
       while(first!=last){
-        if(insert(position,*first).second)x.erase(first++);
+        if(insert(position,*first).second)first=x.erase(first);
         else ++first;
       }
     }
@@ -399,13 +470,13 @@ public:
     sequenced_index_unique(*this,binary_pred);
   }
 
-  void merge(sequenced_index<Super,TagList>& x)
+  void merge(sequenced_index<SuperMeta,TagList>& x)
   {
     sequenced_index_merge(*this,x,std::less<value_type>());
   }
 
   template <typename Compare>
-  void merge(sequenced_index<Super,TagList>& x,Compare comp)
+  void merge(sequenced_index<SuperMeta,TagList>& x,Compare comp)
   {
     sequenced_index_merge(*this,x,comp);
   }
@@ -426,10 +497,10 @@ public:
   void reverse()
   {
     BOOST_MULTI_INDEX_SEQ_INDEX_CHECK_INVARIANT;
-    sequenced_index_node_impl::reverse(header()->impl());
+    node_impl_type::reverse(header()->impl());
   }
 
-  /* relocate operations */
+  /* rearrange operations */
 
   void relocate(iterator position,iterator i)
   {
@@ -457,25 +528,29 @@ public:
       position.get_node(),first.get_node(),last.get_node());
   }
     
-BOOST_MULTI_INDEX_PROTECTED_IF_MEMBER_TEMPLATE_FRIENDS:
-  sequenced_index(const ctor_args_list& args_list,const allocator_type& al):
-    Super(args_list.get_tail(),al)
-
-#if defined(BOOST_MULTI_INDEX_ENABLE_SAFE_MODE)&&\
-    BOOST_WORKAROUND(BOOST_MSVC,<1300)
-    ,safe_super(final_header())
-#endif
-
+  template<typename InputIterator>
+  void rearrange(InputIterator first)
   {
-    header()->prior()=header()->next()=header()->impl();
+    BOOST_MULTI_INDEX_SEQ_INDEX_CHECK_INVARIANT;
+    node_type* pos=header();
+    for(size_type s=size();s--;){
+      const value_type& v=*first++;
+      relink(pos,node_from_value<node_type>(&v));
+    }
   }
 
-  sequenced_index(const sequenced_index<Super,TagList>& x):
-    Super(x)
+BOOST_MULTI_INDEX_PROTECTED_IF_MEMBER_TEMPLATE_FRIENDS:
+  sequenced_index(const ctor_args_list& args_list,const allocator_type& al):
+    super(args_list.get_tail(),al)
+  {
+    empty_initialize();
+  }
 
-#if defined(BOOST_MULTI_INDEX_ENABLE_SAFE_MODE)&&\
-    BOOST_WORKAROUND(BOOST_MSVC,<1300)
-    ,safe_super(final_header())
+  sequenced_index(const sequenced_index<SuperMeta,TagList>& x):
+    super(x)
+
+#if defined(BOOST_MULTI_INDEX_ENABLE_SAFE_MODE)
+    ,safe_super()
 #endif
 
   {
@@ -498,7 +573,8 @@ BOOST_MULTI_INDEX_PROTECTED_IF_MEMBER_TEMPLATE_FRIENDS:
                    {return const_iterator(node);}
 #endif
 
-  void copy_(const sequenced_index<Super,TagList>& x,const copy_map_type& map)
+  void copy_(
+    const sequenced_index<SuperMeta,TagList>& x,const copy_map_type& map)
   {
     node_type* org=x.header();
     node_type* cpy=header();
@@ -511,19 +587,19 @@ BOOST_MULTI_INDEX_PROTECTED_IF_MEMBER_TEMPLATE_FRIENDS:
       cpy=next_cpy;
     }while(org!=x.header());
 
-    Super::copy_(x,map);
+    super::copy_(x,map);
   }
 
   node_type* insert_(value_param_type v,node_type* x)
   {
-    node_type* res=static_cast<node_type*>(Super::insert_(v,x));
+    node_type* res=static_cast<node_type*>(super::insert_(v,x));
     if(res==x)link(x);
     return res;
   }
 
   node_type* insert_(value_param_type v,node_type* position,node_type* x)
   {
-    node_type* res=static_cast<node_type*>(Super::insert_(v,position,x));
+    node_type* res=static_cast<node_type*>(super::insert_(v,position,x));
     if(res==x)link(x);
     return res;
   }
@@ -531,31 +607,50 @@ BOOST_MULTI_INDEX_PROTECTED_IF_MEMBER_TEMPLATE_FRIENDS:
   void erase_(node_type* x)
   {
     unlink(x);
-    Super::erase_(x);
+    super::erase_(x);
 
 #if defined(BOOST_MULTI_INDEX_ENABLE_SAFE_MODE)
     detach_iterators(x);
 #endif
   }
 
-  void swap_(sequenced_index<Super,TagList>& x)
+  void delete_all_nodes_()
+  {
+    for(node_type* x=node_type::from_impl(header()->next());x!=header();){
+      node_type* y=node_type::from_impl(x->next());
+      this->final_delete_node_(static_cast<final_node_type*>(x));
+      x=y;
+    }
+  }
+
+  void clear_()
+  {
+    super::clear_();
+    empty_initialize();
+
+#if defined(BOOST_MULTI_INDEX_ENABLE_SAFE_MODE)
+    safe_super::detach_dereferenceable_iterators();
+#endif
+  }
+
+  void swap_(sequenced_index<SuperMeta,TagList>& x)
   {
 #if defined(BOOST_MULTI_INDEX_ENABLE_SAFE_MODE)
     safe_super::swap(x);
 #endif
 
-    Super::swap_(x);
+    super::swap_(x);
   }
 
   bool replace_(value_param_type v,node_type* x)
   {
-    return Super::replace_(v,x);
+    return super::replace_(v,x);
   }
 
   bool modify_(node_type* x)
   {
     BOOST_TRY{
-      if(!Super::modify_(x)){
+      if(!super::modify_(x)){
         unlink(x);
 
 #if defined(BOOST_MULTI_INDEX_ENABLE_SAFE_MODE)
@@ -578,6 +673,33 @@ BOOST_MULTI_INDEX_PROTECTED_IF_MEMBER_TEMPLATE_FRIENDS:
     BOOST_CATCH_END
   }
 
+  bool modify_rollback_(node_type* x)
+  {
+    return super::modify_rollback_(x);
+  }
+
+#if !defined(BOOST_MULTI_INDEX_DISABLE_SERIALIZATION)
+  /* serialization */
+
+  template<typename Archive>
+  void save_(
+    Archive& ar,const unsigned int version,const index_saver_type& sm)const
+  {
+    sm.save(begin(),end(),ar,version);
+    super::save_(ar,version,sm);
+  }
+
+  template<typename Archive>
+  void load_(
+    Archive& ar,const unsigned int version,const index_loader_type& lm)
+  {
+    lm.load(
+      ::boost::bind(&sequenced_index::rearranger,this,_1,_2),
+      ar,version);
+    super::load_(ar,version,lm);
+  }
+#endif
+
 #if defined(BOOST_MULTI_INDEX_ENABLE_INVARIANT_CHECKING)
   /* invariant stuff */
 
@@ -597,7 +719,7 @@ BOOST_MULTI_INDEX_PROTECTED_IF_MEMBER_TEMPLATE_FRIENDS:
       if(s!=size())return false;
     }
 
-    return Super::invariant_();
+    return super::invariant_();
   }
 
   /* This forwarding function eases things for the boost::mem_fn construct
@@ -610,27 +732,41 @@ BOOST_MULTI_INDEX_PROTECTED_IF_MEMBER_TEMPLATE_FRIENDS:
 private:
   node_type* header()const{return this->final_header();}
 
+  void empty_initialize()
+  {
+    header()->prior()=header()->next()=header()->impl();
+  }
+
   void link(node_type* x)
   {
-    sequenced_index_node_impl::link(x->impl(),header()->impl());
+    node_impl_type::link(x->impl(),header()->impl());
   };
 
   static void unlink(node_type* x)
   {
-    sequenced_index_node_impl::unlink(x->impl());
+    node_impl_type::unlink(x->impl());
   }
 
   static void relink(node_type* position,node_type* x)
   {
-    sequenced_index_node_impl::relink(position->impl(),x->impl());
+    node_impl_type::relink(position->impl(),x->impl());
   }
 
   static void relink(node_type* position,node_type* first,node_type* last)
   {
-    sequenced_index_node_impl::relink(
+    node_impl_type::relink(
       position->impl(),first->impl(),last->impl());
   }
 
+#if !defined(BOOST_MULTI_INDEX_DISABLE_SERIALIZATION)
+  void rearranger(node_type* position,node_type *x)
+  {
+    if(!position)position=header();
+    node_type::increment(position);
+    if(position!=x)relink(position,x);
+  }
+#endif
+
 #if defined(BOOST_MULTI_INDEX_ENABLE_SAFE_MODE)
   void detach_iterators(node_type* x)
   {
@@ -639,6 +775,38 @@ private:
   }
 #endif
 
+  template <class InputIterator>
+  void assign_iter(InputIterator first,InputIterator last,mpl::true_)
+  {
+    BOOST_MULTI_INDEX_SEQ_INDEX_CHECK_INVARIANT;
+    clear();
+    for(;first!=last;++first)push_back(*first);
+  }
+
+  void assign_iter(size_type n,value_param_type value,mpl::false_)
+  {
+    BOOST_MULTI_INDEX_SEQ_INDEX_CHECK_INVARIANT;
+    clear();
+    for(size_type i=0;i<n;++i)push_back(value);
+  }
+
+  template<typename InputIterator>
+  void insert_iter(
+    iterator position,InputIterator first,InputIterator last,mpl::true_)
+  {
+    BOOST_MULTI_INDEX_SEQ_INDEX_CHECK_INVARIANT;
+    for(;first!=last;++first)insert(position,*first);
+  }
+
+  void insert_iter(
+    iterator position,size_type n,value_param_type x,mpl::false_)
+  {
+    BOOST_MULTI_INDEX_CHECK_VALID_ITERATOR(position);
+    BOOST_MULTI_INDEX_CHECK_IS_OWNER(position,*this);
+    BOOST_MULTI_INDEX_SEQ_INDEX_CHECK_INVARIANT;
+    for(size_type i=0;i<n;++i)insert(position,x);
+  }
+ 
 #if defined(BOOST_MULTI_INDEX_ENABLE_INVARIANT_CHECKING)&&\
     BOOST_WORKAROUND(__MWERKS__,<=0x3003)
 #pragma parse_mfunc_templ reset
@@ -648,77 +816,77 @@ private:
 /* comparison */
 
 template<
-  typename Super1,typename TagList1,
-  typename Super2,typename TagList2
+  typename SuperMeta1,typename TagList1,
+  typename SuperMeta2,typename TagList2
 >
 bool operator==(
-  const sequenced_index<Super1,TagList1>& x,
-  const sequenced_index<Super2,TagList2>& y)
+  const sequenced_index<SuperMeta1,TagList1>& x,
+  const sequenced_index<SuperMeta2,TagList2>& y)
 {
   return x.size()==y.size()&&std::equal(x.begin(),x.end(),y.begin());
 }
 
 template<
-  typename Super1,typename TagList1,
-  typename Super2,typename TagList2
+  typename SuperMeta1,typename TagList1,
+  typename SuperMeta2,typename TagList2
 >
 bool operator<(
-  const sequenced_index<Super1,TagList1>& x,
-  const sequenced_index<Super2,TagList2>& y)
+  const sequenced_index<SuperMeta1,TagList1>& x,
+  const sequenced_index<SuperMeta2,TagList2>& y)
 {
   return std::lexicographical_compare(x.begin(),x.end(),y.begin(),y.end());
 }
 
 template<
-  typename Super1,typename TagList1,
-  typename Super2,typename TagList2
+  typename SuperMeta1,typename TagList1,
+  typename SuperMeta2,typename TagList2
 >
 bool operator!=(
-  const sequenced_index<Super1,TagList1>& x,
-  const sequenced_index<Super2,TagList2>& y)
+  const sequenced_index<SuperMeta1,TagList1>& x,
+  const sequenced_index<SuperMeta2,TagList2>& y)
 {
   return !(x==y);
 }
 
 template<
-  typename Super1,typename TagList1,
-  typename Super2,typename TagList2
+  typename SuperMeta1,typename TagList1,
+  typename SuperMeta2,typename TagList2
 >
 bool operator>(
-  const sequenced_index<Super1,TagList1>& x,
-  const sequenced_index<Super2,TagList2>& y)
+  const sequenced_index<SuperMeta1,TagList1>& x,
+  const sequenced_index<SuperMeta2,TagList2>& y)
 {
   return y<x;
 }
 
 template<
-  typename Super1,typename TagList1,
-  typename Super2,typename TagList2
+  typename SuperMeta1,typename TagList1,
+  typename SuperMeta2,typename TagList2
 >
 bool operator>=(
-  const sequenced_index<Super1,TagList1>& x,
-  const sequenced_index<Super2,TagList2>& y)
+  const sequenced_index<SuperMeta1,TagList1>& x,
+  const sequenced_index<SuperMeta2,TagList2>& y)
 {
   return !(x<y);
 }
 
 template<
-  typename Super1,typename TagList1,
-  typename Super2,typename TagList2
+  typename SuperMeta1,typename TagList1,
+  typename SuperMeta2,typename TagList2
 >
 bool operator<=(
-  const sequenced_index<Super1,TagList1>& x,
-  const sequenced_index<Super2,TagList2>& y)
+  const sequenced_index<SuperMeta1,TagList1>& x,
+  const sequenced_index<SuperMeta2,TagList2>& y)
 {
   return !(x>y);
 }
 
 /*  specialized algorithms */
 
-template<typename Super,typename TagList>
+template<typename SuperMeta,typename TagList>
 void swap(
-  sequenced_index<Super,TagList>& x,
-  sequenced_index<Super,TagList>& y)
+  sequenced_index<SuperMeta,TagList>& x,
+  sequenced_index<SuperMeta,TagList>& y)
 {
   x.swap(y);
 }
@@ -738,10 +906,10 @@ struct sequenced
     typedef detail::sequenced_index_node<Super> type;
   };
 
-  template<typename Super>
+  template<typename SuperMeta>
   struct index_class
   {
-    typedef detail::sequenced_index<Super,TagList> type;
+    typedef detail::sequenced_index<SuperMeta,typename TagList::type> type;
   };
 };
 
