@@ -1,6 +1,7 @@
 /* Boost interval/hw_rounding.hpp template implementation file
  *
  * Copyright 2002 Herv Brnnimann, Guillaume Melquiond, Sylvain Pion
+ * Copyright 2005 Guillaume Melquiond
  *
  * Distributed under the Boost Software License, Version 1.0.
  * (See accompanying file LICENSE_1_0.txt or
@@ -13,16 +14,29 @@
 #include <boost/numeric/interval/rounding.hpp>
 #include <boost/numeric/interval/rounded_arith.hpp>
 
+#define BOOST_NUMERIC_INTERVAL_NO_HARDWARE
+
 // define appropriate specialization of rounding_control for built-in types
-#if defined(__i386__) || defined(_M_IX86) || defined(__BORLANDC__)
+#if defined(__x86_64__) && defined(__USE_ISOC99)
+#  include <boost/numeric/interval/detail/c99_rounding_control.hpp>
+#elif defined(__i386__) || defined(_M_IX86) || defined(__BORLANDC__) || defined(_M_X64)
 #  include <boost/numeric/interval/detail/x86_rounding_control.hpp>
 #elif defined(powerpc) || defined(__powerpc__) || defined(__ppc__)
 #  include <boost/numeric/interval/detail/ppc_rounding_control.hpp>
 #elif defined(sparc) || defined(__sparc__)
 #  include <boost/numeric/interval/detail/sparc_rounding_control.hpp>
-#elif defined(__USE_ISOC99)
+#elif defined(alpha) || defined(__alpha__)
+#  include <boost/numeric/interval/detail/alpha_rounding_control.hpp>
+#elif defined(ia64) || defined(__ia64) || defined(__ia64__)
+#  include <boost/numeric/interval/detail/ia64_rounding_control.hpp>
+#endif
+
+#if defined(BOOST_NUMERIC_INTERVAL_NO_HARDWARE) && (defined(__USE_ISOC99) || defined(__MSL__))
 #  include <boost/numeric/interval/detail/c99_rounding_control.hpp>
-#else
+#endif
+
+#if defined(BOOST_NUMERIC_INTERVAL_NO_HARDWARE)
+#  undef BOOST_NUMERIC_INTERVAL_NO_HARDWARE
 #  error Boost.Numeric.Interval: Please specify rounding control mechanism.
 #endif
 
