@@ -1,5 +1,7 @@
 /*=============================================================================
     Copyright (c) 2001-2003 Daniel Nuffer
+    Copyright (c) 2001-2007 Hartmut Kaiser
+    Revised 2007, Copyright (c) Tobias Schwinger
     http://spirit.sourceforge.net/
 
     Use, modification and distribution is subject to the Boost Software
@@ -9,26 +11,33 @@
 #ifndef BOOST_SPIRIT_TREE_COMMON_HPP
 #define BOOST_SPIRIT_TREE_COMMON_HPP
 
+#if !defined(BOOST_SPIRIT_USE_LIST_FOR_TREES)
 #include <vector>
+#else
+#include <list>
+#endif
+
+#if defined(BOOST_SPIRIT_USE_BOOST_ALLOCATOR_FOR_TREES)
+#include <boost/pool/pool_alloc.hpp>
+#endif
+
+#include <algorithm>
+
 #include <boost/ref.hpp>
 #include <boost/call_traits.hpp>
 #include <boost/spirit/core.hpp>
 #include <boost/detail/iterator.hpp> // for boost::detail::iterator_traits
 
 #if defined(BOOST_SPIRIT_DEBUG) && \
-    (BOOST_SPIRIT_DEBUG_FLAGS_NODES & BOOST_SPIRIT_DEBUG_FLAGS_TREES)
+    (BOOST_SPIRIT_DEBUG_FLAGS & BOOST_SPIRIT_DEBUG_FLAGS_NODES)
 #include <iostream>
 #include <boost/spirit/debug/debug_node.hpp>
 #endif
 
+#include <boost/spirit/tree/common_fwd.hpp>
+
 namespace boost { namespace spirit {
 
-template <typename T>
-struct tree_node;
-
-template <typename IteratorT = char const*, typename ValueT = nil_t>
-struct node_iter_data;
-
 template <typename T>
 void swap(tree_node<T>& a, tree_node<T>& b);
 
@@ -44,7 +53,21 @@ template <typename T>
 struct tree_node
 {
     typedef T parse_node_t;
-    typedef std::vector<tree_node<T> > children_t;
+    
+#if !defined(BOOST_SPIRIT_USE_BOOST_ALLOCATOR_FOR_TREES)
+    typedef std::allocator<tree_node<T> > allocator_type;
+#elif !defined(BOOST_SPIRIT_USE_LIST_FOR_TREES)
+    typedef boost::pool_allocator<tree_node<T> > allocator_type;
+#else
+    typedef boost::fast_pool_allocator<tree_node<T> > allocator_type;
+#endif
+
+#if !defined(BOOST_SPIRIT_USE_LIST_FOR_TREES)
+    typedef std::vector<tree_node<T>, allocator_type> children_t;
+#else
+    typedef std::list<tree_node<T>, allocator_type> children_t;
+#endif  // BOOST_SPIRIT_USE_LIST_FOR_TREES
+
     typedef typename children_t::iterator tree_iterator;
     typedef typename children_t::const_iterator const_tree_iterator;
 
@@ -81,7 +104,7 @@ struct tree_node
 };
 
 #if defined(BOOST_SPIRIT_DEBUG) && \
-    (BOOST_SPIRIT_DEBUG_FLAGS_NODES & BOOST_SPIRIT_DEBUG_FLAGS_TREES)
+    (BOOST_SPIRIT_DEBUG_FLAGS & BOOST_SPIRIT_DEBUG_FLAGS_NODES)
 template <typename T>
 inline std::ostream&
 operator<<(std::ostream& o, tree_node<T> const& n)
@@ -188,7 +211,7 @@ public:
 };
 
 #if defined(BOOST_SPIRIT_DEBUG) && \
-    (BOOST_SPIRIT_DEBUG_FLAGS_NODES & BOOST_SPIRIT_DEBUG_FLAGS_TREES)
+    (BOOST_SPIRIT_DEBUG_FLAGS & BOOST_SPIRIT_DEBUG_FLAGS_NODES)
 // value is default nil_t, so provide an operator<< for nil_t
 inline std::ostream&
 operator<<(std::ostream& o, nil_t const&)
@@ -206,7 +229,7 @@ operator<<(std::ostream& o, node_iter_data<IteratorT, ValueT> const& n)
     for (iterator_t it = n.begin(); it != n.end(); ++it)
         impl::token_printer(o, *it);
     o << "\" is_root = " << n.is_root()
-        << " value = " << n.value() << ")";
+        << /*" value = " << n.value() << */")";
     return o;
 }
 #endif
@@ -218,7 +241,21 @@ struct node_val_data
     typedef
         typename boost::detail::iterator_traits<IteratorT>::value_type
         value_type;
-    typedef std::vector<value_type> container_t;
+
+#if !defined(BOOST_SPIRIT_USE_BOOST_ALLOCATOR_FOR_TREES)
+    typedef std::allocator<value_type> allocator_type;
+#elif !defined(BOOST_SPIRIT_USE_LIST_FOR_TREES)
+    typedef boost::pool_allocator<value_type> allocator_type;
+#else
+    typedef boost::fast_pool_allocator<value_type> allocator_type;
+#endif
+
+#if !defined(BOOST_SPIRIT_USE_LIST_FOR_TREES)
+    typedef std::vector<value_type, allocator_type> container_t;
+#else
+    typedef std::list<value_type, allocator_type> container_t;
+#endif
+
     typedef typename container_t::iterator iterator_t;
     typedef typename container_t::const_iterator const_iterator_t;
 
@@ -233,7 +270,7 @@ struct node_val_data
             std::copy(_first, _last, std::inserter(text, text.end()));
         }
 
-    // This constructor is for building text out of vector iterators
+    // This constructor is for building text out of iterators
     template <typename IteratorT2>
     node_val_data(IteratorT2 const& _first, IteratorT2 const& _last)
         : text(), is_root_(false), parser_id_(), value_()
@@ -245,7 +282,7 @@ struct node_val_data
         : text(_first, _last), is_root_(false), parser_id_(), value_()
         {}
 
-    // This constructor is for building text out of vector iterators
+    // This constructor is for building text out of iterators
     template <typename IteratorT2>
     node_val_data(IteratorT2 const& _first, IteratorT2 const& _last)
         : text(_first, _last), is_root_(false), parser_id_(), value_()
@@ -318,7 +355,7 @@ private:
 };
 
 #if defined(BOOST_SPIRIT_DEBUG) && \
-    (BOOST_SPIRIT_DEBUG_FLAGS_NODES & BOOST_SPIRIT_DEBUG_FLAGS_TREES)
+    (BOOST_SPIRIT_DEBUG_FLAGS & BOOST_SPIRIT_DEBUG_FLAGS_NODES)
 template <typename IteratorT, typename ValueT>
 inline std::ostream&
 operator<<(std::ostream& o, node_val_data<IteratorT, ValueT> const& n)
@@ -348,17 +385,12 @@ swap(node_iter_data<T, V>& a, node_iter_data<T, V>& b)
     a.swap(b);
 }
 
-
-//////////////////////////////////
-template <typename ValueT = nil_t>
-class node_iter_data_factory;
-
 //////////////////////////////////
 template <typename ValueT>
 class node_iter_data_factory
 {
 public:
-    // This inner class is so that node_iter_data_factory can simluate
+    // This inner class is so that node_iter_data_factory can simulate
     // a template template parameter
     template <typename IteratorT>
     class factory
@@ -389,13 +421,12 @@ public:
     };
 };
 
-
 //////////////////////////////////
 template <typename ValueT>
-class node_val_data_factory
+class node_val_data_factory 
 {
 public:
-    // This inner class is so that node_val_data_factory can simluate
+    // This inner class is so that node_val_data_factory can simulate
     // a template template parameter
     template <typename IteratorT>
     class factory
@@ -422,11 +453,12 @@ public:
         static node_t group_nodes(ContainerT const& nodes)
         {
             typename node_t::container_t c;
+            typename ContainerT::const_iterator i_end = nodes.end();
             // copy all the nodes text into a new one
             for (typename ContainerT::const_iterator i = nodes.begin();
-                    i != nodes.end(); ++i)
+                 i != i_end; ++i)
             {
-                // See docs: token_node_d or leaf_node_d cannot be used with a
+                // See docs: reduced_node_d cannot be used with a
                 // rule inside the [].
                 assert(i->children.size() == 0);
                 c.insert(c.end(), i->value.begin(), i->value.end());
@@ -436,17 +468,12 @@ public:
     };
 };
 
-
-//////////////////////////////////
-template <typename ValueT = nil_t>
-class node_all_val_data_factory;
-
 //////////////////////////////////
 template <typename ValueT>
 class node_all_val_data_factory
 {
 public:
-    // This inner class is so that node_all_val_data_factory can simluate
+    // This inner class is so that node_all_val_data_factory can simulate
     // a template template parameter
     template <typename IteratorT>
     class factory
@@ -470,12 +497,11 @@ public:
         static node_t group_nodes(ContainerT const& nodes)
         {
             typename node_t::container_t c;
+            typename ContainerT::const_iterator i_end = nodes.end();
             // copy all the nodes text into a new one
             for (typename ContainerT::const_iterator i = nodes.begin();
-                    i != nodes.end(); ++i)
+                    i != i_end; ++i)
             {
-                // See docs: token_node_d or leaf_node_d cannot be used with a
-                // rule inside the [].
                 assert(i->children.size() == 0);
                 c.insert(c.end(), i->value.begin(), i->value.end());
             }
@@ -484,14 +510,6 @@ public:
     };
 };
 
-//  forward declaration
-template <
-    typename IteratorT,
-    typename NodeFactoryT = node_val_data_factory<nil_t>,
-    typename T = nil_t
->
-class tree_match;
-
 namespace impl {
 
     ///////////////////////////////////////////////////////////////////////////
@@ -541,11 +559,31 @@ public:
 
     tree_match(std::size_t length, parse_node_t const& n)
     : match<T>(length), trees()
-    { trees.push_back(node_t(n)); }
+    { 
+        trees.push_back(node_t(n)); 
+    }
 
     tree_match(std::size_t length, param_type val, parse_node_t const& n)
     : match<T>(length, val), trees()
-    { trees.push_back(node_t(n)); }
+    {
+#if !defined(BOOST_SPIRIT_USE_LIST_FOR_TREES)
+        trees.reserve(10); // this is more or less an arbitrary number...
+#endif
+        trees.push_back(node_t(n));
+    }
+
+    // attention, these constructors will change the second parameter!
+    tree_match(std::size_t length, container_t& c)
+    : match<T>(length), trees()
+    { 
+        impl::cp_swap(trees, c);
+    }
+
+    tree_match(std::size_t length, param_type val, container_t& c)
+    : match<T>(length, val), trees()
+    {
+        impl::cp_swap(trees, c);
+    }
 
     template <typename T2>
     tree_match(match<T2> const& other)
@@ -598,7 +636,7 @@ public:
 };
 
 #if defined(BOOST_SPIRIT_DEBUG) && \
-    (BOOST_SPIRIT_DEBUG_FLAGS_NODES & BOOST_SPIRIT_DEBUG_FLAGS_TREES)
+    (BOOST_SPIRIT_DEBUG_FLAGS & BOOST_SPIRIT_DEBUG_FLAGS_NODES)
 template <typename IteratorT, typename NodeFactoryT, typename T>
 inline std::ostream&
 operator<<(std::ostream& o, tree_match<IteratorT, NodeFactoryT, T> const& m)
@@ -607,13 +645,13 @@ operator<<(std::ostream& o, tree_match<IteratorT, NodeFactoryT, T> const& m)
         typename tree_match<IteratorT, NodeFactoryT, T>::container_t::iterator
         iterator;
 
-    o << "(length = " << m.length();
+    o << "(length = " << (int)m.length();
     int c = 0;
     for (iterator i = m.trees.begin(); i != m.trees.end(); ++i)
     {
         o << " trees[" << c++ << "] = " << *i;
     }
-    o << ")";
+    o << "\n)";
     return o;
 }
 #endif
@@ -640,14 +678,25 @@ template <
     typename MatchPolicyT,
     typename IteratorT,
     typename NodeFactoryT,
-    typename TreePolicyT
+    typename TreePolicyT, 
+    typename T
 >
 struct common_tree_match_policy : public match_policy
 {
-    template <typename T>
-    struct result { typedef tree_match<IteratorT, NodeFactoryT, T> type; };
+    common_tree_match_policy()
+    {
+    }
 
-    typedef tree_match<IteratorT, NodeFactoryT> match_t;
+    template <typename PolicyT>
+    common_tree_match_policy(PolicyT const & policies)
+        : match_policy((match_policy const &)policies)
+    {
+    }
+
+    template <typename U>
+    struct result { typedef tree_match<IteratorT, NodeFactoryT, U> type; };
+
+    typedef tree_match<IteratorT, NodeFactoryT, T> match_t;
     typedef IteratorT iterator_t;
     typedef TreePolicyT tree_policy_t;
     typedef NodeFactoryT factory_t;
@@ -664,14 +713,14 @@ struct common_tree_match_policy : public match_policy
         Iterator2T const& last)
     {
 #if defined(BOOST_SPIRIT_DEBUG) && \
-    (BOOST_SPIRIT_DEBUG_FLAGS_NODES & BOOST_SPIRIT_DEBUG_FLAGS_TREES)
+    (BOOST_SPIRIT_DEBUG_FLAGS & BOOST_SPIRIT_DEBUG_FLAGS_NODES)
 
-        BOOST_SPIRIT_DEBUG_OUT << ">>> create_node(begin) <<<\n" 
+        BOOST_SPIRIT_DEBUG_OUT << "\n>>> create_node(begin) <<<\n" 
             "creating node text: \"";
         for (Iterator1T it = first; it != last; ++it)
             impl::token_printer(BOOST_SPIRIT_DEBUG_OUT, *it);
         BOOST_SPIRIT_DEBUG_OUT << "\"\n";
-        BOOST_SPIRIT_DEBUG_OUT << ">>> create_node(end) <<<\n"; 
+        BOOST_SPIRIT_DEBUG_OUT << ">>> create_node(end) <<<\n\n"; 
 #endif
         return tree_match<IteratorT, NodeFactoryT, AttrT>(length, val,
             tree_policy_t::create_node(length, first, last, true));
@@ -681,12 +730,12 @@ struct common_tree_match_policy : public match_policy
     static void concat_match(Match1T& a, Match2T const& b)
     {
 #if defined(BOOST_SPIRIT_DEBUG) && \
-    (BOOST_SPIRIT_DEBUG_FLAGS & BOOST_SPIRIT_DEBUG_FLAGS_TREES)
+    (BOOST_SPIRIT_DEBUG_FLAGS & BOOST_SPIRIT_DEBUG_FLAGS_NODES)
 
-        BOOST_SPIRIT_DEBUG_OUT << ">>> concat_match(begin) <<<\n";
+        BOOST_SPIRIT_DEBUG_OUT << "\n>>> concat_match(begin) <<<\n";
         BOOST_SPIRIT_DEBUG_OUT << "tree a:\n" << a << "\n";
         BOOST_SPIRIT_DEBUG_OUT << "tree b:\n" << b << "\n";
-        BOOST_SPIRIT_DEBUG_OUT << ">>> concat_match(end) <<<\n";
+        BOOST_SPIRIT_DEBUG_OUT << ">>> concat_match(end) <<<\n\n";
 #endif
         BOOST_SPIRIT_ASSERT(a && b);
         if (a.length() == 0)
@@ -694,7 +743,11 @@ struct common_tree_match_policy : public match_policy
             a = b;
             return;
         }
-        else if (b.length() == 0)
+        else if (b.length() == 0
+#ifdef BOOST_SPIRIT_NO_TREE_NODE_COLLAPSING
+            && !b.trees.begin()->value.id().to_long()
+#endif
+            )
         {
             return;
         }
@@ -715,7 +768,7 @@ struct common_tree_match_policy : public match_policy
 #if defined(BOOST_SPIRIT_DEBUG) && \
     (BOOST_SPIRIT_DEBUG_FLAGS & BOOST_SPIRIT_DEBUG_FLAGS_TREES)
 
-        BOOST_SPIRIT_DEBUG_OUT << ">>> group_match(begin) <<<\n"
+        BOOST_SPIRIT_DEBUG_OUT << "\n>>> group_match(begin) <<<\n"
             "new node(" << id << ") \"";
         for (IteratorT2 it = first; it != last; ++it)
             impl::token_printer(BOOST_SPIRIT_DEBUG_OUT, *it);
@@ -725,7 +778,7 @@ struct common_tree_match_policy : public match_policy
         tree_policy_t::group_match(m, id, first, last);
 
         BOOST_SPIRIT_DEBUG_OUT << "new child tree (after grouping):\n" << m << "\n";
-        BOOST_SPIRIT_DEBUG_OUT << ">>> group_match(end) <<<\n";
+        BOOST_SPIRIT_DEBUG_OUT << ">>> group_match(end) <<<\n\n";
 #else
         tree_policy_t::group_match(m, id, first, last);
 #endif
@@ -774,7 +827,6 @@ struct no_tree_gen_node_parser
     typedef no_tree_gen_node_parser<T> self_t;
     typedef no_tree_gen_node_parser_gen parser_generator_t;
     typedef unary_parser_category parser_category_t;
-//    typedef no_tree_gen_node_parser<T> const &embed_t;
 
     no_tree_gen_node_parser(T const& a)
     : unary<T, parser<no_tree_gen_node_parser<T> > >(a) {}
@@ -796,7 +848,6 @@ struct no_tree_gen_node_parser
     }
 };
 
-//////////////////////////////////
 struct no_tree_gen_node_parser_gen
 {
     template <typename T>
@@ -820,20 +871,84 @@ struct no_tree_gen_node_parser_gen
     }
 };
 
-//////////////////////////////////
 const no_tree_gen_node_parser_gen no_node_d = no_tree_gen_node_parser_gen();
 
+//////////////////////////////////
+
+struct leaf_node_parser_gen;
+
+template<typename T>
+struct leaf_node_parser
+:   public unary<T, parser<leaf_node_parser<T> > >
+{
+    typedef leaf_node_parser<T> self_t;
+    typedef leaf_node_parser_gen parser_generator_t;
+    typedef unary_parser_category parser_category_t;
+
+    leaf_node_parser(T const& a)
+    : unary<T, parser<leaf_node_parser<T> > >(a) {}
+
+    template <typename ScannerT>
+    typename parser_result<self_t, ScannerT>::type
+    parse(ScannerT const& scanner) const
+    {
+        typedef scanner_policies< typename ScannerT::iteration_policy_t,
+            match_policy, typename ScannerT::action_policy_t > policies_t;
+
+        typedef typename ScannerT::iterator_t iterator_t;
+        typedef typename parser_result<self_t, ScannerT>::type result_t;
+        typedef typename result_t::node_factory_t factory_t;
+
+        iterator_t from = scanner.first;
+        result_t hit = impl::contiguous_parser_parse<result_t>(this->subject(),
+            scanner.change_policies(policies_t(scanner,match_policy(),scanner)),
+            scanner);
+
+        if (hit)
+            return result_t(hit.length(), 
+                factory_t::create_node(from, scanner.first, true));
+        else
+            return result_t(hit.length());
+    }
+};
+
+struct leaf_node_parser_gen
+{
+    template <typename T>
+    struct result {
+
+        typedef leaf_node_parser<T> type;
+    };
+
+    template <typename T>
+    static leaf_node_parser<T>
+    generate(parser<T> const& s)
+    {
+        return leaf_node_parser<T>(s.derived());
+    }
+
+    template <typename T>
+    leaf_node_parser<T>
+    operator[](parser<T> const& s) const
+    {
+        return leaf_node_parser<T>(s.derived());
+    }
+};
+
+const leaf_node_parser_gen leaf_node_d = leaf_node_parser_gen();
+const leaf_node_parser_gen token_node_d = leaf_node_parser_gen();
 
 //////////////////////////////////
 namespace impl {
 
-template <typename MatchPolicyT>
-struct tree_policy_selector
-{
-    typedef tree_policy type;
-};
+    template <typename MatchPolicyT>
+    struct tree_policy_selector
+    {
+        typedef tree_policy type;
+    };
 
 } // namespace impl
+
 //////////////////////////////////
 template <typename NodeParserT>
 struct node_parser_gen;
@@ -845,11 +960,16 @@ struct node_parser
     typedef node_parser<T, NodeParserT> self_t;
     typedef node_parser_gen<NodeParserT> parser_generator_t;
     typedef unary_parser_category parser_category_t;
-//    typedef node_parser<T, NodeParserT> const &embed_t;
 
     node_parser(T const& a)
     : unary<T, parser<node_parser<T, NodeParserT> > >(a) {}
 
+    template <typename ScannerT>
+    struct result
+    {
+        typedef typename parser_result<T, ScannerT>::type type;
+    };
+
     template <typename ScannerT>
     typename parser_result<self_t, ScannerT>::type
     parse(ScannerT const& scanner) const
@@ -863,7 +983,6 @@ struct node_parser
     }
 };
 
-//////////////////////////////////
 template <typename NodeParserT>
 struct node_parser_gen
 {
@@ -887,20 +1006,8 @@ struct node_parser_gen
         return node_parser<T, NodeParserT>(s.derived());
     }
 };
-
-struct discard_node_op
-{
-    template <typename MatchT>
-    void operator()(MatchT& m) const
-    {
-        m.trees.clear();
-    }
-};
-
-const node_parser_gen<discard_node_op> discard_node_d =
-    node_parser_gen<discard_node_op>();
-
-struct leaf_node_op
+//////////////////////////////////
+struct reduced_node_op
 {
     template <typename MatchT>
     void operator()(MatchT& m) const
@@ -917,10 +1024,21 @@ struct leaf_node_op
     }
 };
 
-const node_parser_gen<leaf_node_op> leaf_node_d =
-    node_parser_gen<leaf_node_op>();
-const node_parser_gen<leaf_node_op> token_node_d =
-    node_parser_gen<leaf_node_op>();
+const node_parser_gen<reduced_node_op> reduced_node_d =
+    node_parser_gen<reduced_node_op>();
+
+
+struct discard_node_op
+{
+    template <typename MatchT>
+    void operator()(MatchT& m) const
+    {
+        m.trees.clear();
+    }
+};
+
+const node_parser_gen<discard_node_op> discard_node_d =
+    node_parser_gen<discard_node_op>();
 
 struct infix_node_op
 {
@@ -938,24 +1056,36 @@ struct infix_node_op
         // copying the tree nodes is expensive, since it may copy a whole
         // tree.  swapping them is cheap, so swap the nodes we want into
         // a new container of children.
-        bool keep = true;
         container_t new_children;
-        for (iter_t i = m.trees.begin(); i != m.trees.end(); ++i)
+        std::size_t length = 0;
+        std::size_t tree_size = m.trees.size();
+
+        // the infix_node_d[] make no sense for nodes with no subnodes
+        BOOST_SPIRIT_ASSERT(tree_size >= 1);
+
+        bool keep = true;
+#if !defined(BOOST_SPIRIT_USE_LIST_FOR_TREES)
+        new_children.reserve((tree_size+1)/2);
+#endif
+        iter_t i_end = m.trees.end();
+        for (iter_t i = m.trees.begin(); i != i_end; ++i)
         {
-            if (keep)
-            {
+            if (keep) {
+                // adjust the length
+                length += std::distance((*i).value.begin(), (*i).value.end());
+
                 // move the child node
                 new_children.push_back(value_t());
                 swap(new_children.back(), *i);
                 keep = false;
             }
-            else
-            {
+            else {
                 // ignore this child node
                 keep = true;
             }
         }
-        swap(m.trees, new_children);
+
+        m = MatchT(length, new_children);
     }
 };
 
@@ -978,17 +1108,48 @@ struct discard_first_node_op
         // copying the tree nodes is expensive, since it may copy a whole
         // tree.  swapping them is cheap, so swap the nodes we want into
         // a new container of children, instead of saying
-        // m.trees.erase(m.trees.begin()) because, on a vector that will cause
-        // all the nodes afterwards to be copied into the previous position.
+        // m.trees.erase(m.trees.begin()) because, on a container_t that will 
+        // cause all the nodes afterwards to be copied into the previous 
+        // position.
         container_t new_children;
-        iter_t i = m.trees.begin();
-        for (++i; i != m.trees.end(); ++i)
-        {
-            // move the child node
-            new_children.push_back(value_t());
-            swap(new_children.back(), *i);
+        std::size_t length = 0;
+        std::size_t tree_size = m.trees.size();
+
+        // the discard_first_node_d[] make no sense for nodes with no subnodes
+        BOOST_SPIRIT_ASSERT(tree_size >= 1);
+
+        if (tree_size > 1) {
+#if !defined(BOOST_SPIRIT_USE_LIST_FOR_TREES)
+            new_children.reserve(tree_size - 1);
+#endif
+            iter_t i = m.trees.begin(), i_end = m.trees.end();
+            for (++i; i != i_end; ++i)
+            {
+                // adjust the length
+                length += std::distance((*i).value.begin(), (*i).value.end());
+
+                // move the child node
+                new_children.push_back(value_t());
+                swap(new_children.back(), *i);
+            }
         }
-        swap(m.trees, new_children);
+        else {
+        // if there was a tree and now there isn't any, insert an empty node
+            iter_t i = m.trees.begin(); 
+
+        // This isn't entirely correct, since the empty node will reference
+        // the end of the discarded node, but I currently don't see any way to 
+        // get at the begin of the node following this subnode.
+        // This should be safe anyway because the it shouldn't get dereferenced
+        // under any circumstances.
+            typedef typename value_t::parse_node_t::iterator_t iterator_type;
+            iterator_type it = (*i).value.end();
+            
+            new_children.push_back(
+                value_t(typename value_t::parse_node_t(it, it)));
+        }
+        
+        m = MatchT(length, new_children);
     }
 };
 
@@ -1000,7 +1161,55 @@ struct discard_last_node_op
     template <typename MatchT>
     void operator()(MatchT& m) const
     {
-        m.trees.pop_back();
+        typedef typename MatchT::container_t container_t;
+        typedef typename MatchT::container_t::iterator iter_t;
+        typedef typename MatchT::container_t::value_type value_t;
+
+        using std::swap;
+        using boost::swap;
+        using boost::spirit::swap;
+
+        // copying the tree nodes is expensive, since it may copy a whole
+        // tree.  swapping them is cheap, so swap the nodes we want into
+        // a new container of children, instead of saying
+        // m.trees.erase(m.trees.begin()) because, on a container_t that will 
+        // cause all the nodes afterwards to be copied into the previous 
+        // position.
+        container_t new_children;
+        std::size_t length = 0;
+        std::size_t tree_size = m.trees.size();
+
+        // the discard_last_node_d[] make no sense for nodes with no subnodes
+        BOOST_SPIRIT_ASSERT(tree_size >= 1);
+
+        if (tree_size > 1) {
+            m.trees.pop_back();
+#if !defined(BOOST_SPIRIT_USE_LIST_FOR_TREES)
+            new_children.reserve(tree_size - 1);
+#endif            
+            iter_t i_end = m.trees.end();
+            for (iter_t i = m.trees.begin(); i != i_end; ++i)
+            {
+                // adjust the length
+                length += std::distance((*i).value.begin(), (*i).value.end());
+
+                // move the child node
+                new_children.push_back(value_t());
+                swap(new_children.back(), *i);
+            }
+        }
+        else {
+        // if there was a tree and now there isn't any, insert an empty node
+            iter_t i = m.trees.begin(); 
+
+            typedef typename value_t::parse_node_t::iterator_t iterator_type;
+            iterator_type it = (*i).value.begin();
+            
+            new_children.push_back(
+                value_t(typename value_t::parse_node_t(it, it)));
+        }
+        
+        m = MatchT(length, new_children);
     }
 };
 
@@ -1023,18 +1232,45 @@ struct inner_node_op
         // copying the tree nodes is expensive, since it may copy a whole
         // tree.  swapping them is cheap, so swap the nodes we want into
         // a new container of children, instead of saying
-        // m.trees.erase(m.trees.begin()) because, on a vector that will cause
-        // all the nodes afterwards to be copied into the previous position.
+        // m.trees.erase(m.trees.begin()) because, on a container_t that will 
+        // cause all the nodes afterwards to be copied into the previous 
+        // position.
         container_t new_children;
-        m.trees.pop_back(); // erase the last element
-        iter_t i = m.trees.begin(); // skip over the first element
-        for (++i; i != m.trees.end(); ++i)
-        {
-            // move the child node
-            new_children.push_back(value_t());
-            swap(new_children.back(), *i);
+        std::size_t length = 0;
+        std::size_t tree_size = m.trees.size();
+        
+        // the inner_node_d[] make no sense for nodes with less then 2 subnodes
+        BOOST_SPIRIT_ASSERT(tree_size >= 2);
+
+        if (tree_size > 2) {
+            m.trees.pop_back(); // erase the last element
+#if !defined(BOOST_SPIRIT_USE_LIST_FOR_TREES)
+            new_children.reserve(tree_size - 1);
+#endif
+            iter_t i = m.trees.begin(); // skip over the first element
+            iter_t i_end = m.trees.end();
+            for (++i; i != i_end; ++i)
+            {
+                // adjust the length
+                length += std::distance((*i).value.begin(), (*i).value.end());
+                
+                // move the child node
+                new_children.push_back(value_t());
+                swap(new_children.back(), *i);
+            }
         }
-        swap(m.trees, new_children);
+        else {
+        // if there was a tree and now there isn't any, insert an empty node
+            iter_t i = m.trees.begin(); // skip over the first element
+
+            typedef typename value_t::parse_node_t::iterator_t iterator_type;
+            iterator_type it = (*++i).value.begin();
+            
+            new_children.push_back(
+                value_t(typename value_t::parse_node_t(it, it)));
+        }
+        
+        m = MatchT(length, new_children);
     }
 };
 
@@ -1059,11 +1295,16 @@ struct action_directive_parser
     typedef action_directive_parser<T, ActionParserT> self_t;
     typedef action_directive_parser_gen<ActionParserT> parser_generator_t;
     typedef unary_parser_category parser_category_t;
-//    typedef action_directive_parser<T, ActionParserT> const &embed_t;
 
     action_directive_parser(T const& a)
         : unary<T, parser<action_directive_parser<T, ActionParserT> > >(a) {}
 
+    template <typename ScannerT>
+    struct result
+    {
+        typedef typename parser_result<T, ScannerT>::type type;
+    };
+
     template <typename ScannerT>
     typename parser_result<self_t, ScannerT>::type
     parse(ScannerT const& scanner) const
@@ -1109,29 +1350,33 @@ struct action_directive_parser_gen
 
 //////////////////////////////////
 // Calls the attached action passing it the match from the parser
-// and the first and last iterators
-struct access_match_action
+// and the first and last iterators.
+// The inner template class is used to simulate template-template parameters
+// (declared in common_fwd.hpp).
+template <typename ParserT, typename ActionT>
+struct access_match_action::action
+:   public unary<ParserT, parser<access_match_action::action<ParserT, ActionT> > >
 {
-    // inner template class to simulate template-template parameters
-    template <typename ParserT, typename ActionT>
-    struct action
-    :   public unary<ParserT, parser<access_match_action::action<ParserT, ActionT> > >
+    typedef action_parser_category parser_category;
+    typedef action<ParserT, ActionT> self_t;
+    
+    template <typename ScannerT>
+    struct result
     {
-        typedef action_parser_category parser_category;
-        typedef action<ParserT, ActionT> self_t;
-
-        action( ParserT const& subject,
-                ActionT const& actor_);
-
-        template <typename ScannerT>
-        typename parser_result<self_t, ScannerT>::type
-        parse(ScannerT const& scanner) const;
-
-        ActionT const &predicate() const;
-
-        private:
-        ActionT actor;
+        typedef typename parser_result<ParserT, ScannerT>::type type;
     };
+
+    action( ParserT const& subject,
+            ActionT const& actor_);
+
+    template <typename ScannerT>
+    typename parser_result<self_t, ScannerT>::type
+    parse(ScannerT const& scanner) const;
+
+    ActionT const &predicate() const;
+
+    private:
+    ActionT actor;
 };
 
 //////////////////////////////////
@@ -1178,28 +1423,32 @@ const action_directive_parser_gen<access_match_action> access_match_d
 //////////////////////////////////
 // Calls the attached action passing it the node from the parser
 // and the first and last iterators
-struct access_node_action
+// The inner template class is used to simulate template-template parameters
+// (declared in common_fwd.hpp).
+template <typename ParserT, typename ActionT>
+struct access_node_action::action
+:   public unary<ParserT, parser<access_node_action::action<ParserT, ActionT> > >
 {
-    // inner template class to simulate template-template parameters
-    template <typename ParserT, typename ActionT>
-    struct action
-    :   public unary<ParserT, parser<access_node_action::action<ParserT, ActionT> > >
+    typedef action_parser_category parser_category;
+    typedef action<ParserT, ActionT> self_t;
+    
+    template <typename ScannerT>
+    struct result
     {
-        typedef action_parser_category parser_category;
-        typedef action<ParserT, ActionT> self_t;
-
-        action( ParserT const& subject,
-                ActionT const& actor_);
-
-        template <typename ScannerT>
-        typename parser_result<self_t, ScannerT>::type
-        parse(ScannerT const& scanner) const;
-
-        ActionT const &predicate() const;
-
-        private:
-        ActionT actor;
+        typedef typename parser_result<ParserT, ScannerT>::type type;
     };
+
+    action( ParserT const& subject,
+            ActionT const& actor_);
+
+    template <typename ScannerT>
+    typename parser_result<self_t, ScannerT>::type
+    parse(ScannerT const& scanner) const;
+
+    ActionT const &predicate() const;
+
+    private:
+    ActionT actor;
 };
 
 //////////////////////////////////
@@ -1271,12 +1520,12 @@ const action_directive_parser_gen<access_node_action> access_node_d
 //
 ///////////////////////////////////////////////////////////////////////////////
 template <
-    typename IteratorT = char const *,
-    typename NodeFactoryT = node_val_data_factory<nil_t>,
-    typename T = nil_t
+    typename IteratorT,
+    typename NodeFactoryT,
+    typename T
 >
-struct tree_parse_info {
-
+struct tree_parse_info 
+{
     IteratorT   stop;
     bool        match;
     bool        full;
