@@ -3,7 +3,7 @@
 
     http://www.boost.org/
 
-    Copyright (c) 2001-2005 Hartmut Kaiser. Distributed under the Boost
+    Copyright (c) 2001-2008 Hartmut Kaiser. Distributed under the Boost
     Software License, Version 1.0. (See accompanying file
     LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 =============================================================================*/
@@ -18,6 +18,7 @@
 #include <boost/spirit/utility/lists.hpp>
 
 #include <boost/wave/wave_config.hpp>
+#include <boost/pool/pool_alloc.hpp>
 
 #if BOOST_WAVE_DUMP_PARSE_TREE != 0
 #include <map>
@@ -30,6 +31,11 @@
 
 #include <boost/wave/cpp_exceptions.hpp>
 
+// this must occur after all of the includes and before any code appears
+#ifdef BOOST_HAS_ABI_HEADERS
+#include BOOST_ABI_PREFIX
+#endif
+
 ///////////////////////////////////////////////////////////////////////////////
 namespace boost {
 namespace wave { 
@@ -37,29 +43,6 @@ namespace grammars {
 
 namespace impl {
 
-///////////////////////////////////////////////////////////////////////////////
-//
-//  store_position 
-//
-//      The store_position functor extracts the actual file position from the 
-//      supplied token.
-//
-///////////////////////////////////////////////////////////////////////////////
-
-    template <typename PositionT>
-    struct store_position {
-
-        store_position(PositionT &pos_) : pos(pos_) {}
-        
-        template <typename TokenT>
-        void operator()(TokenT const &token) const
-        {
-            pos = token.get_position();
-        }
-        
-        PositionT &pos;
-    };
-
 ///////////////////////////////////////////////////////////////////////////////
 //
 //  store_found_eof
@@ -74,7 +57,7 @@ namespace impl {
         store_found_eof(bool &found_eof_) : found_eof(found_eof_) {}
         
         template <typename TokenT>
-        void operator()(TokenT const &token) const
+        void operator()(TokenT const &/*token*/) const
         {
             found_eof = true;
         }
@@ -91,18 +74,43 @@ namespace impl {
 //
 ///////////////////////////////////////////////////////////////////////////////
 
+    template <typename TokenT>
     struct store_found_directive {
 
-        store_found_directive(boost::wave::token_id &found_directive_) 
+        store_found_directive(TokenT &found_directive_) 
         :   found_directive(found_directive_) {}
         
-        template <typename TokenT>
         void operator()(TokenT const &token) const
         {
-            found_directive = boost::wave::token_id(token);
+            found_directive = token;
         }
         
-        boost::wave::token_id &found_directive;
+        TokenT &found_directive;
+    };
+
+///////////////////////////////////////////////////////////////////////////////
+//
+//  store_found_eoltokens
+//
+//      The store_found_eoltokens functor stores the token sequence of the 
+//      line ending for a particular pp directive
+//
+///////////////////////////////////////////////////////////////////////////////
+
+    template <typename ContainerT>
+    struct store_found_eoltokens {
+
+        store_found_eoltokens(ContainerT &found_eoltokens_) 
+        :   found_eoltokens(found_eoltokens_) {}
+        
+        template <typename IteratorT>
+        void operator()(IteratorT const &first, IteratorT const& last) const
+        {
+            std::copy(first, last, 
+                std::inserter(found_eoltokens, found_eoltokens.end()));
+        }
+        
+        ContainerT &found_eoltokens;
     };
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -143,14 +151,15 @@ namespace impl {
 
 ///////////////////////////////////////////////////////////////////////////////
 // Encapsulation of the C++ preprocessor grammar.
-template <typename PositionT>
+template <typename TokenT, typename ContainerT>
 struct cpp_grammar : 
-    public boost::spirit::grammar<cpp_grammar<PositionT> >
+    public boost::spirit::grammar<cpp_grammar<TokenT, ContainerT> >
 {
-    typedef cpp_grammar<PositionT>          grammar_t;
-    typedef impl::store_position<PositionT> store_pos_t;
-    typedef impl::store_found_eof           store_found_eof_t;
-    typedef impl::store_found_directive     store_found_directive_t;
+    typedef typename TokenT::position_type  position_type;
+    typedef cpp_grammar<TokenT, ContainerT> grammar_type;
+    typedef impl::store_found_eof           store_found_eof_type;
+    typedef impl::store_found_directive<TokenT>     store_found_directive_type;
+    typedef impl::store_found_eoltokens<ContainerT> store_found_eoltokens_type;
     
     template <typename ScannerT>
     struct definition
@@ -166,27 +175,31 @@ struct cpp_grammar :
         typedef 
             boost::spirit::scanner<typename ScannerT::iterator_t, policies_t> 
             non_tree_scanner_t;
-        typedef boost::spirit::rule<non_tree_scanner_t> no_tree_rule_t;
+        typedef 
+            boost::spirit::rule<non_tree_scanner_t, boost::spirit::dynamic_parser_tag> 
+            no_tree_rule_type;
 
     // 'normal' (parse_tree generating) rule type
-        typedef boost::spirit::rule<ScannerT> rule_t;
+        typedef 
+            boost::spirit::rule<ScannerT, boost::spirit::dynamic_parser_tag> 
+            rule_type;
 
-        rule_t pp_statement;
-        rule_t include_file, system_include_file, macro_include_file;
-        rule_t plain_define, macro_definition, macro_parameters;
-        rule_t undefine;
-        rule_t ppifdef, ppifndef, ppif, ppelse, ppelif, ppendif;
-        rule_t ppline; 
-        rule_t pperror;
-        rule_t ppwarning;
-        rule_t pppragma;
-        rule_t illformed;
-        rule_t ppqualifiedname;
-        rule_t eol_tokens;
-        no_tree_rule_t ppsp;
+        rule_type pp_statement;
+        rule_type include_file, system_include_file, macro_include_file;
+        rule_type plain_define, macro_definition, macro_parameters;
+        rule_type undefine;
+        rule_type ppifdef, ppifndef, ppif, ppelse, ppelif, ppendif;
+        rule_type ppline; 
+        rule_type pperror;
+        rule_type ppwarning;
+        rule_type pppragma;
+        rule_type illformed;
+        rule_type ppqualifiedname;
+        rule_type eol_tokens;
+        no_tree_rule_type ppsp;
 #if BOOST_WAVE_SUPPORT_MS_EXTENSIONS != 0
-        rule_t ppregion;
-        rule_t ppendregion;
+        rule_type ppregion;
+        rule_type ppendregion;
 #endif
 
         definition(cpp_grammar const &self) 
@@ -196,31 +209,31 @@ struct cpp_grammar :
             using namespace boost::wave;
             using namespace boost::wave::util;
 
-        // save the rule id's for later use
-            self.rule_ids.pp_statement_id = pp_statement.id().to_long();
-            self.rule_ids.include_file_id = include_file.id().to_long();
-            self.rule_ids.sysinclude_file_id = system_include_file.id().to_long();
-            self.rule_ids.macroinclude_file_id = macro_include_file.id().to_long();
-            self.rule_ids.plain_define_id = plain_define.id().to_long();
-            self.rule_ids.macro_parameters_id = macro_parameters.id().to_long();
-            self.rule_ids.macro_definition_id = macro_definition.id().to_long();
-            self.rule_ids.undefine_id = undefine.id().to_long();
-            self.rule_ids.ifdef_id = ppifdef.id().to_long();
-            self.rule_ids.ifndef_id = ppifndef.id().to_long();
-            self.rule_ids.if_id = ppif.id().to_long();
-            self.rule_ids.elif_id = ppelif.id().to_long();
-            self.rule_ids.else_id = ppelse.id().to_long();
-            self.rule_ids.endif_id = ppendif.id().to_long();
-            self.rule_ids.line_id = ppline.id().to_long();
-            self.rule_ids.error_id = pperror.id().to_long();
-            self.rule_ids.warning_id = ppwarning.id().to_long();
-            self.rule_ids.pragma_id = pppragma.id().to_long();
-            self.rule_ids.illformed_id = illformed.id().to_long();
-            self.rule_ids.ppspace_id = ppsp.id().to_long();
-            self.rule_ids.ppqualifiedname_id = ppqualifiedname.id().to_long();
+        // set the rule id's for later use
+            pp_statement.set_id(BOOST_WAVE_PP_STATEMENT_ID);
+            include_file.set_id(BOOST_WAVE_INCLUDE_FILE_ID);
+            system_include_file.set_id(BOOST_WAVE_SYSINCLUDE_FILE_ID);
+            macro_include_file.set_id(BOOST_WAVE_MACROINCLUDE_FILE_ID);
+            plain_define.set_id(BOOST_WAVE_PLAIN_DEFINE_ID);
+            macro_parameters.set_id(BOOST_WAVE_MACRO_PARAMETERS_ID);
+            macro_definition.set_id(BOOST_WAVE_MACRO_DEFINITION_ID);
+            undefine.set_id(BOOST_WAVE_UNDEFINE_ID);
+            ppifdef.set_id(BOOST_WAVE_IFDEF_ID);
+            ppifndef.set_id(BOOST_WAVE_IFNDEF_ID);
+            ppif.set_id(BOOST_WAVE_IF_ID);
+            ppelif.set_id(BOOST_WAVE_ELIF_ID);
+            ppelse.set_id(BOOST_WAVE_ELSE_ID);
+            ppendif.set_id(BOOST_WAVE_ENDIF_ID);
+            ppline.set_id(BOOST_WAVE_LINE_ID);
+            pperror.set_id(BOOST_WAVE_ERROR_ID);
+            ppwarning.set_id(BOOST_WAVE_WARNING_ID);
+            pppragma.set_id(BOOST_WAVE_PRAGMA_ID);
+            illformed.set_id(BOOST_WAVE_ILLFORMED_ID);
+            ppsp.set_id(BOOST_WAVE_PPSPACE_ID);
+            ppqualifiedname.set_id(BOOST_WAVE_PPQUALIFIEDNAME_ID);
 #if BOOST_WAVE_SUPPORT_MS_EXTENSIONS != 0
-            self.rule_ids.region_id = ppregion.id().to_long();
-            self.rule_ids.endregion_id = ppendregion.id().to_long();
+            ppregion.set_id(BOOST_WAVE_REGION_ID);
+            ppendregion.set_id(BOOST_WAVE_ENDREGION_ID);
 #endif
 
 #if BOOST_WAVE_DUMP_PARSE_TREE != 0
@@ -238,28 +251,29 @@ struct cpp_grammar :
         // new-line character that follows the first token in the sequence.
 
             pp_statement
-                =   (   include_file
+                =   (   plain_define
+                    |   include_file
                     |   system_include_file
-                    |   macro_include_file
-                    |   plain_define
-                    |   undefine
-                    |   ppifdef
-                    |   ppifndef
                     |   ppif
-                    |   ppelse
                     |   ppelif
-                    |   ppendif
+                    |   ppifndef
+                    |   ppifdef
+                    |   undefine
+                    |   ppelse
+                    |   macro_include_file
                     |   ppline
+                    |   pppragma
                     |   pperror
                     |   ppwarning
-                    |   pppragma
-                    |   illformed
+                    |   ppendif
 #if BOOST_WAVE_SUPPORT_MS_EXTENSIONS != 0
                     |   ppregion
                     |   ppendregion
 #endif
+                    |   illformed
                     )
                     >> eol_tokens
+                       [ store_found_eoltokens_type(self.found_eoltokens) ]
 //  In parser debug mode it is useful not to flush the underlying stream
 //  to allow its investigation in the debugger and to see the correct
 //  output in the printed debug log..
@@ -267,26 +281,26 @@ struct cpp_grammar :
 #if !(defined(BOOST_SPIRIT_DEBUG) && \
       (BOOST_SPIRIT_DEBUG_FLAGS_CPP & BOOST_SPIRIT_DEBUG_FLAGS_CPP_GRAMMAR) \
      )
-                    >>  impl::flush_underlying_parser_p
+                   >>  impl::flush_underlying_parser_p
 #endif // !(defined(BOOST_SPIRIT_DEBUG) &&
                 ;
 
         // #include ...
             include_file            // include "..."
                 =   ch_p(T_PP_QHEADER) 
-                    [ store_found_directive_t(self.found_directive) ]
+                    [ store_found_directive_type(self.found_directive) ]
 #if BOOST_WAVE_SUPPORT_INCLUDE_NEXT != 0
                 |   ch_p(T_PP_QHEADER_NEXT)
-                    [ store_found_directive_t(self.found_directive) ]
+                    [ store_found_directive_type(self.found_directive) ]
 #endif 
                 ;
 
             system_include_file     // include <...>
                 =   ch_p(T_PP_HHEADER) 
-                    [ store_found_directive_t(self.found_directive) ]
+                    [ store_found_directive_type(self.found_directive) ]
 #if BOOST_WAVE_SUPPORT_INCLUDE_NEXT != 0
                 |   ch_p(T_PP_HHEADER_NEXT)
-                    [ store_found_directive_t(self.found_directive) ]
+                    [ store_found_directive_type(self.found_directive) ]
 #endif 
                 ;
 
@@ -294,10 +308,10 @@ struct cpp_grammar :
                 =   no_node_d
                     [
                         ch_p(T_PP_INCLUDE)
-                        [ store_found_directive_t(self.found_directive) ]
+                        [ store_found_directive_type(self.found_directive) ]
 #if BOOST_WAVE_SUPPORT_INCLUDE_NEXT != 0
                     |   ch_p(T_PP_INCLUDE_NEXT)
-                        [ store_found_directive_t(self.found_directive) ]
+                        [ store_found_directive_type(self.found_directive) ]
 #endif
                     ]
                     >> *(   anychar_p -
@@ -310,13 +324,14 @@ struct cpp_grammar :
                 =   no_node_d
                     [
                         ch_p(T_PP_DEFINE) 
-                        [ store_found_directive_t(self.found_directive) ]
+                        [ store_found_directive_type(self.found_directive) ]
                         >> +ppsp
                     ]
                     >>  (   ch_p(T_IDENTIFIER) 
                         |   pattern_p(KeywordTokenType, TokenTypeMask)
                         |   pattern_p(OperatorTokenType|AltExtTokenType, 
                                 ExtTokenTypeMask)   // and, bit_and etc.
+                        |   pattern_p(BoolLiteralTokenType, TokenTypeMask)  // true/false
                         )
                     >>  (   (   no_node_d[eps_p(ch_p(T_LEFTPAREN))]
                                 >>  macro_parameters
@@ -333,11 +348,12 @@ struct cpp_grammar :
             macro_parameters
                 =   confix_p(
                         no_node_d[ch_p(T_LEFTPAREN) >> *ppsp],
-                        !list_p(
+                       !list_p(
                             (   ch_p(T_IDENTIFIER) 
                             |   pattern_p(KeywordTokenType, TokenTypeMask)
                             |   pattern_p(OperatorTokenType|AltExtTokenType, 
                                     ExtTokenTypeMask)   // and, bit_and etc.
+                            |   pattern_p(BoolLiteralTokenType, TokenTypeMask)  // true/false
 #if BOOST_WAVE_SUPPORT_VARIADICS_PLACEMARKERS != 0
                             |   ch_p(T_ELLIPSIS)
 #endif
@@ -361,13 +377,14 @@ struct cpp_grammar :
                 =   no_node_d
                     [
                         ch_p(T_PP_UNDEF) 
-                        [ store_found_directive_t(self.found_directive) ]
+                        [ store_found_directive_type(self.found_directive) ]
                         >> +ppsp
                     ]
                     >>  (   ch_p(T_IDENTIFIER) 
                         |   pattern_p(KeywordTokenType, TokenTypeMask)
                         |   pattern_p(OperatorTokenType|AltExtTokenType, 
                                 ExtTokenTypeMask)   // and, bit_and etc.
+                        |   pattern_p(BoolLiteralTokenType, TokenTypeMask)  // true/false
                         )
                 ;
 
@@ -376,7 +393,7 @@ struct cpp_grammar :
                 =   no_node_d
                     [
                         ch_p(T_PP_IFDEF) 
-                        [ store_found_directive_t(self.found_directive) ]
+                        [ store_found_directive_type(self.found_directive) ]
                         >> +ppsp
                     ]
                     >>  ppqualifiedname
@@ -386,7 +403,7 @@ struct cpp_grammar :
                 =   no_node_d
                     [
                         ch_p(T_PP_IFNDEF) 
-                        [ store_found_directive_t(self.found_directive) ]
+                        [ store_found_directive_type(self.found_directive) ]
                         >> +ppsp
                     ]
                     >>  ppqualifiedname
@@ -396,7 +413,7 @@ struct cpp_grammar :
                 =   no_node_d
                     [
                         ch_p(T_PP_IF) 
-                        [ store_found_directive_t(self.found_directive) ]
+                        [ store_found_directive_type(self.found_directive) ]
                         >> *ppsp
                     ]
                     >> +(   anychar_p -
@@ -408,7 +425,7 @@ struct cpp_grammar :
                 =   no_node_d
                     [
                         ch_p(T_PP_ELSE)
-                        [ store_found_directive_t(self.found_directive) ]
+                        [ store_found_directive_type(self.found_directive) ]
                     ]
                 ;
 
@@ -416,7 +433,7 @@ struct cpp_grammar :
                 =   no_node_d
                     [
                         ch_p(T_PP_ELIF) 
-                        [ store_found_directive_t(self.found_directive) ]
+                        [ store_found_directive_type(self.found_directive) ]
                         >> *ppsp
                     ]
                     >> +(   anychar_p -
@@ -428,7 +445,7 @@ struct cpp_grammar :
                 =   no_node_d
                     [
                         ch_p(T_PP_ENDIF)
-                        [ store_found_directive_t(self.found_directive) ]
+                        [ store_found_directive_type(self.found_directive) ]
                     ]
                 ;
 
@@ -437,7 +454,7 @@ struct cpp_grammar :
                 =   no_node_d
                     [
                         ch_p(T_PP_LINE) 
-                        [ store_found_directive_t(self.found_directive) ]
+                        [ store_found_directive_type(self.found_directive) ]
                         >> *ppsp
                     ]
                     >> +(   anychar_p -
@@ -451,7 +468,7 @@ struct cpp_grammar :
                 =   no_node_d
                     [
                         ch_p(T_MSEXT_PP_REGION) 
-                        [ store_found_directive_t(self.found_directive) ]
+                        [ store_found_directive_type(self.found_directive) ]
                         >> +ppsp
                     ]
                     >>  ppqualifiedname
@@ -462,7 +479,7 @@ struct cpp_grammar :
                 =   no_node_d
                     [
                         ch_p(T_MSEXT_PP_ENDREGION) 
-                        [ store_found_directive_t(self.found_directive) ]
+                        [ store_found_directive_type(self.found_directive) ]
                     ]
                 ;
 #endif
@@ -490,7 +507,7 @@ struct cpp_grammar :
                 =   no_node_d
                     [
                         ch_p(T_PP_ERROR) 
-                        [ store_found_directive_t(self.found_directive) ]
+                        [ store_found_directive_type(self.found_directive) ]
                         >> *ppsp
                     ]
                     >> *(   anychar_p -
@@ -503,7 +520,7 @@ struct cpp_grammar :
                 =   no_node_d
                     [
                         ch_p(T_PP_WARNING) 
-                        [ store_found_directive_t(self.found_directive) ]
+                        [ store_found_directive_type(self.found_directive) ]
                         >> *ppsp
                     ]
                     >> *(   anychar_p -
@@ -516,7 +533,7 @@ struct cpp_grammar :
                 =   no_node_d
                     [
                         ch_p(T_PP_PRAGMA)
-                        [ store_found_directive_t(self.found_directive) ]
+                        [ store_found_directive_type(self.found_directive) ]
                     ] 
                     >> *(   anychar_p -
                             (ch_p(T_NEWLINE) | ch_p(T_CPPCOMMENT) | ch_p(T_EOF)) 
@@ -529,6 +546,7 @@ struct cpp_grammar :
                         |   pattern_p(KeywordTokenType, TokenTypeMask)
                         |   pattern_p(OperatorTokenType|AltExtTokenType, 
                                 ExtTokenTypeMask)   // and, bit_and etc.
+                        |   pattern_p(BoolLiteralTokenType, TokenTypeMask)  // true/false
                         ) 
                 ;
 
@@ -541,15 +559,14 @@ struct cpp_grammar :
             eol_tokens 
                 =   no_node_d
                     [
-                        *ppsp 
-                        >>  (   ch_p(T_NEWLINE)
-                                [ store_pos_t(self.pos_of_newline) ]
-                            |   ch_p(T_CPPCOMMENT)
-                                [ store_pos_t(self.pos_of_newline) ]
-                            |   ch_p(T_EOF)
-                                [ store_pos_t(self.pos_of_newline) ]
-                                [ store_found_eof_t(self.found_eof) ]
-                            )
+                       *(   ch_p(T_SPACE) 
+                        |   ch_p(T_CCOMMENT)
+                        )
+                    >>  (   ch_p(T_NEWLINE)
+                        |   ch_p(T_CPPCOMMENT)
+                        |   ch_p(T_EOF)
+                            [ store_found_eof_type(self.found_eof) ]
+                        )
                     ]
                 ;
 
@@ -580,19 +597,19 @@ struct cpp_grammar :
         }
 
     // start rule of this grammar
-        rule_t const& start() const
+        rule_type const& start() const
         { return pp_statement; }
     };
 
-    cpp_grammar_rule_ids &rule_ids;
-    PositionT &pos_of_newline;
     bool &found_eof;
-    boost::wave::token_id &found_directive;
+    TokenT &found_directive;
+    ContainerT &found_eoltokens;
     
-    cpp_grammar(cpp_grammar_rule_ids &rule_ids_, PositionT &pos_of_newline_,
-            bool &found_eof_, boost::wave::token_id &found_directive_)
-    :   rule_ids(rule_ids_), pos_of_newline(pos_of_newline_), 
-        found_eof(found_eof_), found_directive(found_directive_)
+    cpp_grammar(bool &found_eof_, TokenT &found_directive_, 
+            ContainerT &found_eoltokens_)
+    :   found_eof(found_eof_), 
+        found_directive(found_directive_),
+        found_eoltokens(found_eoltokens_)
     { 
         BOOST_SPIRIT_DEBUG_TRACE_GRAMMAR_NAME(*this, "cpp_grammar", 
             TRACE_CPP_GRAMMAR); 
@@ -603,7 +620,7 @@ struct cpp_grammar :
     struct map_ruleid_to_name :
         public std::map<boost::spirit::parser_id, std::string> 
     {
-        typedef std::map<boost::spirit::parser_id, std::string> base_t;
+        typedef std::map<boost::spirit::parser_id, std::string> base_type;
 
         void init_rule_id_to_name_map(cpp_grammar const &self)
         {
@@ -612,36 +629,37 @@ struct cpp_grammar :
                 char const *rule_name;
             } 
             init_ruleid_name_map[] = {
-                { self.rule_ids.pp_statement_id, "pp_statement" },
-                { self.rule_ids.include_file_id, "include_file" },
-                { self.rule_ids.sysinclude_file_id, "system_include_file" },
-                { self.rule_ids.macroinclude_file_id, "macro_include_file" },
-                { self.rule_ids.plain_define_id, "plain_define" },
-                { self.rule_ids.macro_parameters_id, "macro_parameters" },
-                { self.rule_ids.macro_definition_id, "macro_definition" },
-                { self.rule_ids.undefine_id, "undefine" },
-                { self.rule_ids.ifdef_id, "ppifdef" },
-                { self.rule_ids.ifndef_id, "ppifndef" },
-                { self.rule_ids.if_id, "ppif" },
-                { self.rule_ids.elif_id, "ppelif" },
-                { self.rule_ids.else_id, "ppelse" },
-                { self.rule_ids.endif_id, "ppendif" },
-                { self.rule_ids.line_id, "ppline" },
-                { self.rule_ids.error_id, "pperror" },
-                { self.rule_ids.warning_id, "ppwarning" },
-                { self.rule_ids.pragma_id, "pppragma" },
-                { self.rule_ids.illformed_id, "illformed" },
-                { self.rule_ids.ppqualifiedname_id, "ppqualifiedname" },
+                { BOOST_WAVE_PP_STATEMENT_ID, "pp_statement" },
+                { BOOST_WAVE_INCLUDE_FILE_ID, "include_file" },
+                { BOOST_WAVE_SYSINCLUDE_FILE_ID, "system_include_file" },
+                { BOOST_WAVE_MACROINCLUDE_FILE_ID, "macro_include_file" },
+                { BOOST_WAVE_PLAIN_DEFINE_ID, "plain_define" },
+                { BOOST_WAVE_MACRO_PARAMETERS_ID, "macro_parameters" },
+                { BOOST_WAVE_MACRO_DEFINITION_ID, "macro_definition" },
+                { BOOST_WAVE_UNDEFINE_ID, "undefine" },
+                { BOOST_WAVE_IFDEF_ID, "ppifdef" },
+                { BOOST_WAVE_IFNDEF_ID, "ppifndef" },
+                { BOOST_WAVE_IF_ID, "ppif" },
+                { BOOST_WAVE_ELIF_ID, "ppelif" },
+                { BOOST_WAVE_ELSE_ID, "ppelse" },
+                { BOOST_WAVE_ENDIF_ID, "ppendif" },
+                { BOOST_WAVE_LINE_ID, "ppline" },
+                { BOOST_WAVE_ERROR_ID, "pperror" },
+                { BOOST_WAVE_WARNING_ID, "ppwarning" },
+                { BOOST_WAVE_PRAGMA_ID, "pppragma" },
+                { BOOST_WAVE_ILLFORMED_ID, "illformed" },
+                { BOOST_WAVE_PPSPACE_ID, "ppspace" },
+                { BOOST_WAVE_PPQUALIFIEDNAME_ID, "ppqualifiedname" },
 #if BOOST_WAVE_SUPPORT_MS_EXTENSIONS != 0
-                { self.rule_ids.region_id, "ppregion" },
-                { self.rule_ids.endregion_id, "ppendregion" },
+                { BOOST_WAVE_REGION_ID, "ppregion" },
+                { BOOST_WAVE_ENDREGION_ID, "ppendregion" },
 #endif
                 { 0 }
             };
 
         // initialize parser_id to rule_name map
             for (int i = 0; 0 != init_ruleid_name_map[i].parser_id; ++i)
-                base_t::insert(base_t::value_type(
+                base_type::insert(base_type::value_type(
                     boost::spirit::parser_id(init_ruleid_name_map[i].parser_id), 
                     std::string(init_ruleid_name_map[i].rule_name))
                 );
@@ -654,6 +672,31 @@ struct cpp_grammar :
 ///////////////////////////////////////////////////////////////////////////////
 #undef TRACE_CPP_GRAMMAR
 
+///////////////////////////////////////////////////////////////////////////////
+//
+//  Special parse function generating a parse tree using a given node_factory.
+//
+///////////////////////////////////////////////////////////////////////////////
+template <typename NodeFactoryT, typename IteratorT, typename ParserT>
+inline boost::spirit::tree_parse_info<IteratorT, NodeFactoryT>
+parsetree_parse(IteratorT const& first_, IteratorT const& last,
+    boost::spirit::parser<ParserT> const& p)
+{
+    using namespace boost::spirit;
+    
+    typedef pt_match_policy<IteratorT, NodeFactoryT> pt_match_policy_type;
+    typedef scanner_policies<iteration_policy, pt_match_policy_type>
+        scanner_policies_type;
+    typedef scanner<IteratorT, scanner_policies_type> scanner_type;
+
+    scanner_policies_type policies;
+    IteratorT first = first_;
+    scanner_type scan(first, last, policies);
+    tree_match<IteratorT, NodeFactoryT> hit = p.derived().parse(scan);
+    return tree_parse_info<IteratorT, NodeFactoryT>(
+        first, hit, hit && (first == last), hit.length(), hit.trees);
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 //  
 //  The following parse function is defined here, to allow the separation of 
@@ -667,51 +710,23 @@ struct cpp_grammar :
 #define BOOST_WAVE_GRAMMAR_GEN_INLINE inline
 #endif 
 
-namespace {
-
-    char const *get_directivename(boost::wave::token_id id)
-    {
-        using namespace boost::wave;
-        switch (static_cast<unsigned int>(id)) {
-        case T_PP_QHEADER:
-        case T_PP_HHEADER:
-        case T_PP_INCLUDE:  return "#include";
-        case T_PP_DEFINE:   return "#define";
-        case T_PP_UNDEF:    return "#undef";
-        case T_PP_IFDEF:    return "#ifdef";
-        case T_PP_IFNDEF:   return "#ifndef";
-        case T_PP_IF:       return "#if";
-        case T_PP_ELSE:     return "#else";
-        case T_PP_ELIF:     return "#elif";
-        case T_PP_ENDIF:    return "#endif";
-        case T_PP_LINE:     return "#line";
-        case T_PP_ERROR:    return "#error";
-        case T_PP_WARNING:  return "#warning";
-        case T_PP_PRAGMA:   return "#pragma";
-        default:
-            return "#unknown directive";
-        }
-    }
-}
-
-template <typename LexIteratorT>
+template <typename LexIteratorT, typename TokenContainerT>
 BOOST_WAVE_GRAMMAR_GEN_INLINE 
-boost::spirit::tree_parse_info<LexIteratorT>
-cpp_grammar_gen<LexIteratorT>::parse_cpp_grammar (
+boost::spirit::tree_parse_info<
+    LexIteratorT, 
+    typename cpp_grammar_gen<LexIteratorT, TokenContainerT>::node_factory_type
+>
+cpp_grammar_gen<LexIteratorT, TokenContainerT>::parse_cpp_grammar (
     LexIteratorT const &first, LexIteratorT const &last,
-    bool &found_eof_, position_type const &act_pos)
+    position_type const &act_pos, bool &found_eof,
+    token_type &found_directive, token_container_type &found_eoltokens)
 {
     using namespace boost::spirit;
     using namespace boost::wave;
     
-    pos_of_newline = position_type();  // reset position
-    found_eof = false;              // reset flag
-    found_directive = T_EOF;        // reset found directive
-    
-    cpp_grammar<position_type> g(
-        rule_ids, pos_of_newline, found_eof, found_directive);
-    
-    tree_parse_info<LexIteratorT> hit = pt_parse (first, last, g);
+    cpp_grammar<token_type, TokenContainerT> g(found_eof, found_directive, found_eoltokens);
+    tree_parse_info<LexIteratorT, node_factory_type> hit = 
+        parsetree_parse<node_factory_type>(first, last, g);
     
 #if BOOST_WAVE_DUMP_PARSE_TREE != 0
     if (hit.match) {
@@ -721,15 +736,6 @@ cpp_grammar_gen<LexIteratorT>::parse_cpp_grammar (
     }
 #endif
 
-    if (!hit.match && found_directive != T_EOF) {
-    // recognized invalid directive
-    std::string directive = get_directivename(found_directive);
-    
-        BOOST_WAVE_THROW(preprocess_exception, ill_formed_directive, 
-            directive.c_str(), act_pos);
-    }
-
-    found_eof_ = found_eof;
     return hit;
 }
 
@@ -740,4 +746,9 @@ cpp_grammar_gen<LexIteratorT>::parse_cpp_grammar (
 }   // namespace wave
 }   // namespace boost 
 
+// the suffix header occurs after all of the code
+#ifdef BOOST_HAS_ABI_HEADERS
+#include BOOST_ABI_SUFFIX
+#endif
+
 #endif // !defined(CPP_GRAMMAR_HPP_FEAEBC2E_2734_428B_A7CA_85E5A415E23E_INCLUDED)
