@@ -10,6 +10,9 @@
 //  LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
 //  See http://www.boost.org/libs/config for most recent version.
+//
+//  Revision $Id: config_info.cpp 41316 2007-11-23 17:03:14Z bemandawes $
+//
 
 #include <boost/config.hpp>
 #include <boost/version.hpp>
@@ -124,6 +127,8 @@ void print_compiler_macros()
   PRINT_MACRO(_Windows);
   PRINT_MACRO(__WIN32__);
   PRINT_MACRO(_WIN32);
+  PRINT_MACRO(_WIN64);
+  PRINT_MACRO(_WIN32_WCE);
   PRINT_MACRO(WIN32);
   PRINT_MACRO(_RTLDLL);
   PRINT_MACRO(__DEBUG);
@@ -157,7 +162,6 @@ void print_compiler_macros()
   PRINT_MACRO(__CHAR_UNSIGNED__);
   PRINT_MACRO(__REGISTER_PREFIX__);
   PRINT_MACRO(__USER_LABEL_PREFIX__);
-  PRINT_MACRO(__INTEL__);
   PRINT_MACRO(__GNUPRO__);
   PRINT_MACRO(__EXCEPTIONS);
   PRINT_MACRO(__FreeBSD__);
@@ -194,6 +198,7 @@ void print_compiler_macros()
   PRINT_MACRO(__hp9000s700);
   PRINT_MACRO(_PA_RISC1_1);
   PRINT_MACRO(__HPUX_SOURCE);
+  PRINT_MACRO(_INCLUDE__STDC_A1_SOURCE);
   // SGI IRIX:
   PRINT_MACRO(__sgi);
   PRINT_MACRO(_COMPILER_VERSION);
@@ -265,12 +270,25 @@ void print_compiler_macros()
   PRINT_MACRO(__RTTI);
   PRINT_MACRO(__PLACEMENT_DELETE);
   PRINT_MACRO(__NO_LONG_LONG);
-  
-  // misc compilers not covered so far:
+
+  // Intel options:
+  PRINT_MACRO(__INTEL__);
   PRINT_MACRO(__ICC);
   PRINT_MACRO(__ICL);
   PRINT_MACRO(__ECC);
   PRINT_MACRO(__INTEL_COMPILER);
+  PRINT_MACRO(__INITIAL_POINTER_SIZE);
+  PRINT_MACRO(_INTEGRAL_MAX_BITS);
+  PRINT_MACRO(__INTEL_COMPILER_BUILD_DATE);
+  PRINT_MACRO(__INTEL_MS_COMPAT_LEVEL);
+  PRINT_MACRO(__LONG_DOUBLE_SIZE__);
+  PRINT_MACRO(_M_X64);
+  PRINT_MACRO(_OPENMP);
+  PRINT_MACRO(_OPENMPT);
+  PRINT_MACRO(_PGO_INSTRUMENT);
+  PRINT_MACRO(__QMSPP_); 
+
+  // misc compilers not covered so far:
   PRINT_MACRO(__USLC__);
   PRINT_MACRO(__DECCXX);
   PRINT_MACRO(__IBMCPP__);
@@ -282,6 +300,9 @@ void print_compiler_macros()
   PRINT_MACRO(__DM__);
   PRINT_MACRO(__osf__);
   PRINT_MACRO(__OSF__);
+  PRINT_MACRO(__QNXNTO__);
+  PRINT_MACRO(__QNX__);
+  PRINT_MACRO(_NTO_VERSION);
 }
 
 void print_stdlib_macros()
@@ -760,7 +781,6 @@ void print_platform_macros()
    PRINT_MACRO(__GLIBC_MINOR__);
    PRINT_MACRO(__GNU_LIBRARY__);
    PRINT_MACRO(_BSD_SOURCE);
-   PRINT_MACRO(_FILE_OFFSET_BITS);
    PRINT_MACRO(_GNU_SOURCE);
    PRINT_MACRO(_ISOC99_SOURCE);
    PRINT_MACRO(_ISOC9X_SOURCE);
@@ -854,6 +874,9 @@ void print_platform_macros()
    PRINT_MACRO(_XOPEN_STREAMS);
    PRINT_MACRO(_XOPEN_UNIX);
    PRINT_MACRO(_XOPEN_VERSION);
+   // Misc:
+   PRINT_MACRO(__USE_BSD);
+   PRINT_MACRO(_FILE_OFFSET_BITS);
 }
 
 void print_boost_macros()
@@ -870,18 +893,23 @@ void print_boost_macros()
    PRINT_MACRO(BOOST_NO_STDLIB_CONFIG);
    PRINT_MACRO(BOOST_NO_PLATFORM_CONFIG);
    // then defect and feature macros:
-   PRINT_MACRO(BOOST_BCB_PARTIAL_SPECIALIZATION_BUG);
-   PRINT_MACRO(BOOST_DEDUCED_TYPENAME);
    PRINT_MACRO(BOOST_DISABLE_THREADS);
    PRINT_MACRO(BOOST_DISABLE_WIN32);
+   PRINT_MACRO(BOOST_HAS_THREADS);
+
+   // BEGIN GENERATED BLOCK DO NOT EDIT THIS!!!!!!
+   PRINT_MACRO(BOOST_BCB_PARTIAL_SPECIALIZATION_BUG);
+   PRINT_MACRO(BOOST_DEDUCED_TYPENAME);
    PRINT_MACRO(BOOST_FUNCTION_SCOPE_USING_DECLARATION_BREAKS_ADL);
    PRINT_MACRO(BOOST_HAS_BETHREADS);
    PRINT_MACRO(BOOST_HAS_CLOCK_GETTIME);
-   PRINT_MACRO(BOOST_HAS_DECLSPEC);
+   PRINT_MACRO(BOOST_HAS_CONCEPTS);
    PRINT_MACRO(BOOST_HAS_DIRENT_H);
+   PRINT_MACRO(BOOST_HAS_EXPM1);
    PRINT_MACRO(BOOST_HAS_FTIME);
    PRINT_MACRO(BOOST_HAS_GETTIMEOFDAY);
    PRINT_MACRO(BOOST_HAS_HASH);
+   PRINT_MACRO(BOOST_HAS_LOG1P);
    PRINT_MACRO(BOOST_HAS_LONG_LONG);
    PRINT_MACRO(BOOST_HAS_MACRO_USE_FACET);
    PRINT_MACRO(BOOST_HAS_MS_INT64);
@@ -889,25 +917,41 @@ void print_boost_macros()
    PRINT_MACRO(BOOST_HAS_NL_TYPES_H);
    PRINT_MACRO(BOOST_HAS_NRVO);
    PRINT_MACRO(BOOST_HAS_PARTIAL_STD_ALLOCATOR);
+   PRINT_MACRO(BOOST_HAS_PTHREADS);
    PRINT_MACRO(BOOST_HAS_PTHREAD_DELAY_NP);
    PRINT_MACRO(BOOST_HAS_PTHREAD_MUTEXATTR_SETTYPE);
    PRINT_MACRO(BOOST_HAS_PTHREAD_YIELD);
-   PRINT_MACRO(BOOST_HAS_PTHREADS);
+   PRINT_MACRO(BOOST_HAS_RVALUE_REFS);
    PRINT_MACRO(BOOST_HAS_SCHED_YIELD);
-   PRINT_MACRO(BOOST_HAS_SIGACTION);
    PRINT_MACRO(BOOST_HAS_SGI_TYPE_TRAITS);
-   PRINT_MACRO(BOOST_HAS_STDINT_H);
+   PRINT_MACRO(BOOST_HAS_SIGACTION);
    PRINT_MACRO(BOOST_HAS_SLIST);
+   PRINT_MACRO(BOOST_HAS_STATIC_ASSERT);
+   PRINT_MACRO(BOOST_HAS_STDINT_H);
    PRINT_MACRO(BOOST_HAS_STLP_USE_FACET);
-   PRINT_MACRO(BOOST_HAS_THREADS);
+   PRINT_MACRO(BOOST_HAS_TR1_ARRAY);
+   PRINT_MACRO(BOOST_HAS_TR1_BIND);
+   PRINT_MACRO(BOOST_HAS_TR1_COMPLEX_INVERSE_TRIG);
+   PRINT_MACRO(BOOST_HAS_TR1_COMPLEX_OVERLOADS);
+   PRINT_MACRO(BOOST_HAS_TR1_FUNCTION);
+   PRINT_MACRO(BOOST_HAS_TR1_HASH);
+   PRINT_MACRO(BOOST_HAS_TR1_MEM_FN);
+   PRINT_MACRO(BOOST_HAS_TR1_RANDOM);
+   PRINT_MACRO(BOOST_HAS_TR1_REFERENCE_WRAPPER);
+   PRINT_MACRO(BOOST_HAS_TR1_REGEX);
+   PRINT_MACRO(BOOST_HAS_TR1_RESULT_OF);
+   PRINT_MACRO(BOOST_HAS_TR1_SHARED_PTR);
+   PRINT_MACRO(BOOST_HAS_TR1_TUPLE);
+   PRINT_MACRO(BOOST_HAS_TR1_TYPE_TRAITS);
+   PRINT_MACRO(BOOST_HAS_TR1_UNORDERED_MAP);
+   PRINT_MACRO(BOOST_HAS_TR1_UNORDERED_SET);
+   PRINT_MACRO(BOOST_HAS_TR1_UTILITY);
    PRINT_MACRO(BOOST_HAS_TWO_ARG_USE_FACET);
    PRINT_MACRO(BOOST_HAS_UNISTD_H);
+   PRINT_MACRO(BOOST_HAS_VARIADIC_TMPL);
    PRINT_MACRO(BOOST_HAS_WINTHREADS);
-   PRINT_MACRO(BOOST_INTEL);
-   PRINT_MACRO(BOOST_MSVC);
-   PRINT_MACRO(BOOST_MSVC_STD_ITERATOR);
    PRINT_MACRO(BOOST_MSVC6_MEMBER_TEMPLATES);
-   PRINT_MACRO(BOOST_NESTED_TEMPLATE);
+   PRINT_MACRO(BOOST_MSVC_STD_ITERATOR);
    PRINT_MACRO(BOOST_NO_ARGUMENT_DEPENDENT_LOOKUP);
    PRINT_MACRO(BOOST_NO_ARRAY_TYPE_SPECIALIZATIONS);
    PRINT_MACRO(BOOST_NO_AUTO_PTR);
@@ -916,31 +960,33 @@ void print_boost_macros()
    PRINT_MACRO(BOOST_NO_CV_VOID_SPECIALIZATIONS);
    PRINT_MACRO(BOOST_NO_CWCHAR);
    PRINT_MACRO(BOOST_NO_CWCTYPE);
-   PRINT_MACRO(BOOST_NO_DEDUCED_TYPENAME);
    PRINT_MACRO(BOOST_NO_DEPENDENT_NESTED_DERIVATIONS);
    PRINT_MACRO(BOOST_NO_DEPENDENT_TYPES_IN_TEMPLATE_VALUE_PARAMETERS);
    PRINT_MACRO(BOOST_NO_EXCEPTIONS);
+   PRINT_MACRO(BOOST_NO_EXCEPTION_STD_NAMESPACE);
    PRINT_MACRO(BOOST_NO_EXPLICIT_FUNCTION_TEMPLATE_ARGUMENTS);
    PRINT_MACRO(BOOST_NO_FUNCTION_TEMPLATE_ORDERING);
-   PRINT_MACRO(BOOST_NO_HASH);
+   PRINT_MACRO(BOOST_NO_FUNCTION_TYPE_SPECIALIZATIONS);
    PRINT_MACRO(BOOST_NO_INCLASS_MEMBER_INITIALIZATION);
-   PRINT_MACRO(BOOST_NO_INT64_T);
    PRINT_MACRO(BOOST_NO_INTEGRAL_INT64_T);
    PRINT_MACRO(BOOST_NO_INTRINSIC_WCHAR_T);
+   PRINT_MACRO(BOOST_NO_IOSFWD);
+   PRINT_MACRO(BOOST_NO_IOSTREAM);
    PRINT_MACRO(BOOST_NO_IS_ABSTRACT);
    PRINT_MACRO(BOOST_NO_LIMITS);
    PRINT_MACRO(BOOST_NO_LIMITS_COMPILE_TIME_CONSTANTS);
    PRINT_MACRO(BOOST_NO_LONG_LONG_NUMERIC_LIMITS);
    PRINT_MACRO(BOOST_NO_MEMBER_FUNCTION_SPECIALIZATIONS);
-   PRINT_MACRO(BOOST_NO_MEMBER_TEMPLATE_KEYWORD);
-   PRINT_MACRO(BOOST_NO_MEMBER_TEMPLATE_FRIENDS);
    PRINT_MACRO(BOOST_NO_MEMBER_TEMPLATES);
+   PRINT_MACRO(BOOST_NO_MEMBER_TEMPLATE_FRIENDS);
+   PRINT_MACRO(BOOST_NO_MEMBER_TEMPLATE_KEYWORD);
    PRINT_MACRO(BOOST_NO_MS_INT64_NUMERIC_LIMITS);
    PRINT_MACRO(BOOST_NO_OPERATORS_IN_NAMESPACE);
    PRINT_MACRO(BOOST_NO_POINTER_TO_MEMBER_CONST);
+   PRINT_MACRO(BOOST_NO_POINTER_TO_MEMBER_TEMPLATE_PARAMETERS);
    PRINT_MACRO(BOOST_NO_PRIVATE_IN_AGGREGATE);
    PRINT_MACRO(BOOST_NO_SFINAE);
-   PRINT_MACRO(BOOST_NO_SLIST);
+   PRINT_MACRO(BOOST_NO_STDC_NAMESPACE);
    PRINT_MACRO(BOOST_NO_STD_ALLOCATOR);
    PRINT_MACRO(BOOST_NO_STD_DISTANCE);
    PRINT_MACRO(BOOST_NO_STD_ITERATOR);
@@ -949,18 +995,34 @@ void print_boost_macros()
    PRINT_MACRO(BOOST_NO_STD_MESSAGES);
    PRINT_MACRO(BOOST_NO_STD_MIN_MAX);
    PRINT_MACRO(BOOST_NO_STD_OUTPUT_ITERATOR_ASSIGN);
+   PRINT_MACRO(BOOST_NO_STD_TYPEINFO);
    PRINT_MACRO(BOOST_NO_STD_USE_FACET);
    PRINT_MACRO(BOOST_NO_STD_WSTREAMBUF);
    PRINT_MACRO(BOOST_NO_STD_WSTRING);
-   PRINT_MACRO(BOOST_NO_STDC_NAMESPACE);
    PRINT_MACRO(BOOST_NO_STRINGSTREAM);
    PRINT_MACRO(BOOST_NO_SWPRINTF);
+   PRINT_MACRO(BOOST_NO_TEMPLATED_ITERATOR_CONSTRUCTORS);
    PRINT_MACRO(BOOST_NO_TEMPLATE_PARTIAL_SPECIALIZATION);
    PRINT_MACRO(BOOST_NO_TEMPLATE_TEMPLATES);
-   PRINT_MACRO(BOOST_NO_TEMPLATED_ITERATOR_CONSTRUCTORS);
+   PRINT_MACRO(BOOST_NO_TWO_PHASE_NAME_LOOKUP);
+   PRINT_MACRO(BOOST_NO_TYPEID);
    PRINT_MACRO(BOOST_NO_UNREACHABLE_RETURN_DETECTION);
+   PRINT_MACRO(BOOST_NO_USING_DECLARATION_OVERLOADS_FROM_TYPENAME_BASE);
    PRINT_MACRO(BOOST_NO_USING_TEMPLATE);
    PRINT_MACRO(BOOST_NO_VOID_RETURNS);
+
+
+
+
+
+
+
+
+
+   // END GENERATED BLOCK
+
+   PRINT_MACRO(BOOST_INTEL);
+   PRINT_MACRO(BOOST_MSVC);
    PRINT_MACRO(BOOST_STD_EXTENSION_NAMESPACE);
    PRINT_MACRO(BOOST_UNREACHABLE_RETURN(0));
 }
@@ -994,3 +1056,5 @@ int main()
 
 
 
+
+
