@@ -52,6 +52,7 @@
 
 #include <locale>
 #include <cstddef> // size_t
+#include <cwchar>  // mbstate_t
 #include <boost/integer_traits.hpp>
 #include <boost/iostreams/detail/config/wide_streams.hpp>
 #include <boost/iostreams/detail/codecvt_helper.hpp>
@@ -110,7 +111,7 @@ protected:
     virtual std::codecvt_base::result do_unshift(
         std::mbstate_t&,
         char * from,
-        char * to,
+        char * /* to */,
         char * & next
     ) const{
         next = from;
