@@ -1,6 +1,6 @@
 /* Used in Boost.MultiIndex tests.
  *
- * Copyright 2003-2005 Joaqun M Lpez Muoz.
+ * Copyright 2003-2007 Joaqun M Lpez Muoz.
  * Distributed under the Boost Software License, Version 1.0.
  * (See accompanying file LICENSE_1_0.txt or copy at
  * http://www.boost.org/LICENSE_1_0.txt)
@@ -12,15 +12,18 @@
 #define BOOST_MULTI_INDEX_TEST_EMPLOYEE_HPP
 
 #include <boost/config.hpp> /* keep it first to prevent nasty warns in MSVC */
+#include <boost/mpl/vector.hpp>
 #include <boost/multi_index_container.hpp>
 #include <boost/multi_index/hashed_index.hpp>
 #include <boost/multi_index/identity.hpp>
 #include <boost/multi_index/member.hpp>
 #include <boost/multi_index/ordered_index.hpp>
+#include <boost/multi_index/random_access_index.hpp>
 #include <boost/multi_index/sequenced_index.hpp>
 #include <cstddef>
 #include <ostream>
 #include <string>
+#include "non_std_allocator.hpp"
 
 struct employee
 {
@@ -66,25 +69,32 @@ struct by_name{};
 struct age{};
 struct as_inserted{};
 struct ssn{};
+struct randomly{};
+
+struct employee_set_indices:
+  boost::mpl::vector<
+    boost::multi_index::ordered_unique<
+      boost::multi_index::identity<employee> >,
+    boost::multi_index::hashed_non_unique<
+      boost::multi_index::tag<name,by_name>,
+      BOOST_MULTI_INDEX_MEMBER(employee,std::string,name)>,
+    boost::multi_index::ordered_non_unique<
+      boost::multi_index::tag<age>,
+      BOOST_MULTI_INDEX_MEMBER(employee,int,age)>,
+    boost::multi_index::sequenced<
+      boost::multi_index::tag<as_inserted> >,
+    boost::multi_index::hashed_unique<
+      boost::multi_index::tag<ssn>,
+      BOOST_MULTI_INDEX_MEMBER(employee,int,ssn)>,
+    boost::multi_index::random_access<
+      boost::multi_index::tag<randomly> > >
+{};
 
 typedef
   boost::multi_index::multi_index_container<
     employee,
-    boost::multi_index::indexed_by<
-      boost::multi_index::ordered_unique<
-        boost::multi_index::identity<employee> >,
-      boost::multi_index::hashed_non_unique<
-        boost::multi_index::tag<name,by_name>,
-        BOOST_MULTI_INDEX_MEMBER(employee,std::string,name)>,
-      boost::multi_index::ordered_non_unique<
-        boost::multi_index::tag<age>,
-        BOOST_MULTI_INDEX_MEMBER(employee,int,age)>,
-      boost::multi_index::sequenced<
-        boost::multi_index::tag<as_inserted> >,
-      boost::multi_index::hashed_unique<
-        boost::multi_index::tag<ssn>,
-        BOOST_MULTI_INDEX_MEMBER(employee,int,ssn)> > >
-  employee_set;
+    employee_set_indices,
+    non_std_allocator<employee> >        employee_set;
 
 #if defined(BOOST_NO_MEMBER_TEMPLATES)
 typedef boost::multi_index::nth_index<
@@ -100,4 +110,12 @@ typedef boost::multi_index::index<
 typedef boost::multi_index::index<
          employee_set,ssn>::type         employee_set_by_ssn;
 
+#if defined(BOOST_NO_MEMBER_TEMPLATES)
+typedef boost::multi_index::index<
+         employee_set,randomly>::type    employee_set_randomly;
+#else
+typedef employee_set::index<
+          randomly>::type                employee_set_randomly;
+#endif
+
 #endif
