@@ -1,6 +1,6 @@
 /* Boost.MultiIndex test for serialization.
  *
- * Copyright 2003-2006 Joaqun M Lpez Muoz.
+ * Copyright 2003-2007 Joaqun M Lpez Muoz.
  * Distributed under the Boost Software License, Version 1.0.
  * (See accompanying file LICENSE_1_0.txt or copy at
  * http://www.boost.org/LICENSE_1_0.txt)
@@ -11,9 +11,11 @@
 #include "test_serialization.hpp"
 #include "test_serialization1.hpp"
 #include "test_serialization2.hpp"
+#include "test_serialization3.hpp"
 
 void test_serialization()
 {
   test_serialization1();
   test_serialization2();
+  test_serialization3();
 }
