@@ -1,7 +1,7 @@
 /*
  *
  * Copyright (c) 1998-2002
- * Dr John Maddock
+ * John Maddock
  *
  * Use, modification and distribution are subject to the 
  * Boost Software License, Version 1.0. (See accompanying file 
@@ -33,9 +33,14 @@ using std::getline;
 
 #include <boost/config.hpp>
 #include <boost/regex.hpp>
+#include <boost/cregex.hpp>
 #include <boost/timer.hpp> 
 #include <boost/smart_ptr.hpp>
 
+#if defined(_WIN32) && defined(BOOST_REGEX_USE_WIN32_LOCALE)
+#include <windows.h>
+#endif
+
 #if (defined(_MSC_VER) && (_MSC_VER <= 1300)) || defined(__sgi)
 // maybe no Koenig lookup, use using declaration instead:
 using namespace boost;
@@ -138,7 +143,7 @@ int main(int argc, char**argv)
    boost::match_results<std::deque<char>::iterator> dm;
    std::string s1, s2, ts;
    std::deque<char> ds;
-   boost::regex_t r;
+   boost::regex_tA r;
    boost::scoped_array<boost::regmatch_t> matches;
    std::size_t nsubs;
    boost::timer t;
@@ -170,12 +175,12 @@ int main(int argc, char**argv)
          cout << "Error in expression: \"" << e.what() << "\"" << endl;
          continue;
       }
-      int code = regcomp(&r, s1.c_str(), boost::REG_PERL);
+      int code = regcompA(&r, s1.c_str(), boost::REG_PERL);
       if(code != 0)
       {
          char buf[256];
-         regerror(code, &r, buf, 256);
-         cout << "regcomp error: \"" << buf << "\"" << endl;
+         regerrorA(code, &r, buf, 256);
+         cout << "regcompA error: \"" << buf << "\"" << endl;
          continue;
       }
       nsubs = r.re_nsub + 1;
@@ -210,7 +215,7 @@ int main(int argc, char**argv)
          // cache load:
          regex_search(s2, sm, ex);
 
-         // measure time interval for reg_expression<char>
+         // measure time interval for basic_regex<char>
          do{
             iters *= (tim > 0.001) ? (1.1/tim) : 100;
             t.restart();
@@ -278,7 +283,7 @@ int main(int argc, char**argv)
          }
 #endif
         
-         // measure time interval for reg_expression<char> using a deque
+         // measure time interval for basic_regex<char> using a deque
          iters = 10;
          tim = 1.1;
          // cache load:
@@ -319,17 +324,17 @@ int main(int argc, char**argv)
          iters = 10;
          tim = 1.1;
          // cache load:
-         regexec(&r, s2.c_str(), nsubs, matches.get(), 0);
+         regexecA(&r, s2.c_str(), nsubs, matches.get(), 0);
          do{
             iters *= (tim > 0.001) ? (1.1/tim) : 100;
             t.restart();
             for(i = 0; i < iters; ++i)
             {
-               result = regexec(&r, s2.c_str(), nsubs, matches.get(), 0);
+               result = regexecA(&r, s2.c_str(), nsubs, matches.get(), 0);
             }
             tim = t.elapsed();
          }while(tim < wait_time);
-         cout << "POSIX regexec time: " << (tim * 1000000 / iters) << "us" << endl;
+         cout << "POSIX regexecA time: " << (tim * 1000000 / iters) << "us" << endl;
 
          if(result == 0)
          {
@@ -355,7 +360,7 @@ int main(int argc, char**argv)
             cout << "\" (matched=" << (matches[0].rm_eo != s2.size()) << ")" << endl << endl;
          }
       }
-      regfree(&r);
+      regfreeA(&r);
    }
 
    if(pbuf)
@@ -367,6 +372,9 @@ int main(int argc, char**argv)
    return 0;
 }
 
+#if defined(_WIN32) && defined(BOOST_REGEX_USE_WIN32_LOCALE) && !defined(UNDER_CE)
+#pragma comment(lib, "user32.lib")
+#endif
 
 
 
