@@ -16,7 +16,7 @@
 
 //  See http://www.boost.org for updates, documentation, and revision history.
 
-#include <boost/test/test_tools.hpp>
+#include "test_tools.hpp"
 #include <boost/detail/no_exceptions_support.hpp>
 #include <boost/throw_exception.hpp>
 #include <boost/serialization/split_member.hpp>
