@@ -6,8 +6,6 @@
 // License, Version 1.0. (See accompanying file LICENSE_1_0.txt or copy at
 // http://www.boost.org/LICENSE_1_0.txt)
 
-#include <boost/test/test_tools.hpp>
-
+#include "test_tools.hpp"
 #define main test_main
-
 #include "../example/demo_polymorphic.cpp"
