@@ -38,6 +38,6 @@ private:
 };
 
 int
-test_main(int argc, char * argv[]){
+main(int argc, char * argv[]){
     return 0;
 }
