@@ -1,6 +1,6 @@
 /*=============================================================================
     Copyright (c) 2001-2003 Joel de Guzman
-    Copyright (c) 2001-2003 Hartmut Kaiser
+    Copyright (c) 2001-2008 Hartmut Kaiser
     http://spirit.sourceforge.net/
 
     Use, modification and distribution is subject to the Boost Software
@@ -9,8 +9,9 @@
 =============================================================================*/
 #include <boost/spirit/core.hpp>
 #include <boost/spirit/actor/assign_actor.hpp>
+#include <climits>
 #include <iostream>
-#include <assert.h>
+#include <boost/detail/lightweight_test.hpp>
 
 using namespace std;
 using namespace boost::spirit;
@@ -111,7 +112,7 @@ main()
     char const* int_underflow = "-2147483649";
     char const* max_binary = "11111111111111111111111111111111";
     char const* binary_overflow = "100000000000000000000000000000000";
-    char const* max_octal = "77777777777";
+    char const* max_octal = "37777777777";
     char const* octal_overflow = "100000000000";
     char const* max_hex = "FFFFFFFF";
     char const* hex_overflow = "100000000";
@@ -132,76 +133,76 @@ main()
 //  unsigned integer
 
     parse("123456", uint_p[assign_a(u)]);
-    assert(u == 123456);
+    BOOST_TEST(u == 123456);
 
     parse(max_unsigned, uint_p[assign_a(u)]);
-    assert(u == UINT_MAX);
+    BOOST_TEST(u == UINT_MAX);
 
-    assert(!parse(unsigned_overflow, uint_p).full);
+    BOOST_TEST(!parse(unsigned_overflow, uint_p).full);
 
 //  signed integer
 
     int i;
 
     parse("123456", int_p[assign_a(i)]);
-    assert(i == 123456);
+    BOOST_TEST(i == 123456);
 
     parse("-123456", int_p[assign_a(i)]);
-    assert(i == -123456);
+    BOOST_TEST(i == -123456);
 
     parse(max_int, int_p[assign_a(i)]);
-    assert(i == INT_MAX);
+    BOOST_TEST(i == INT_MAX);
 
     parse(min_int, int_p[assign_a(i)]);
-    assert(i == INT_MIN);
+    BOOST_TEST(i == INT_MIN);
 
-    assert(!parse(int_overflow, int_p).full);
-    assert(!parse(int_underflow, int_p).full);
+    BOOST_TEST(!parse(int_overflow, int_p).full);
+    BOOST_TEST(!parse(int_underflow, int_p).full);
 
-    assert(!parse("-", int_p).hit);
+    BOOST_TEST(!parse("-", int_p).hit);
 
 //  binary
 
     parse("11111110", bin_p[assign_a(u)]);
-    assert(u == 0xFE);
+    BOOST_TEST(u == 0xFE);
 
     parse(max_binary, bin_p[assign_a(u)]);
-    assert(u == UINT_MAX);
+    BOOST_TEST(u == UINT_MAX);
 
-    assert(!parse(binary_overflow, bin_p).full);
+    BOOST_TEST(!parse(binary_overflow, bin_p).full);
 
 //  octal
 
     parse("12545674515", oct_p[assign_a(u)]);
-    assert(u == 012545674515);
+    BOOST_TEST(u == 012545674515);
 
     parse(max_octal, oct_p[assign_a(u)]);
-    assert(u == UINT_MAX);
+    BOOST_TEST(u == UINT_MAX);
 
-    assert(!parse(octal_overflow, oct_p).full);
+    BOOST_TEST(!parse(octal_overflow, oct_p).full);
 
 //  hex
 
     parse("95BC8DF", hex_p[assign_a(u)]);
-    assert(u == 0x95BC8DF);
+    BOOST_TEST(u == 0x95BC8DF);
 
     parse("abcdef12", hex_p[assign_a(u)]);
-    assert(u == 0xabcdef12);
+    BOOST_TEST(u == 0xabcdef12);
 
     parse(max_hex, hex_p[assign_a(u)]);
-    assert(u == UINT_MAX);
+    BOOST_TEST(u == UINT_MAX);
 
-    assert(!parse(hex_overflow, hex_p).full);
+    BOOST_TEST(!parse(hex_overflow, hex_p).full);
 
 //  limited fieldwidth
 
     uint_parser<unsigned, 10, 1, 3> uint3_p;
     parse("123456", uint3_p[assign_a(u)]);
-    assert(u == 123);
+    BOOST_TEST(u == 123);
 
     uint_parser<unsigned, 10, 1, 4> uint4_p;
     parse("123456", uint4_p[assign_a(u)]);
-    assert(u == 1234);
+    BOOST_TEST(u == 1234);
 
     uint_parser<unsigned, 10, 3, 3> uint3_3_p;
 
@@ -209,12 +210,12 @@ main()
 
 #define r (uint3_p >> *(',' >> uint3_3_p))
 
-    assert(parse("1,234,567,890", r).full);     //  OK
-    assert(parse("12,345,678,900", r).full);    //  OK
-    assert(parse("123,456,789,000", r).full);   //  OK
-    assert(!parse("1000,234,567,890", r).full); //  Bad
-    assert(!parse("1,234,56,890", r).full);     //  Bad
-    assert(!parse("1,66", r).full);             //  Bad
+    BOOST_TEST(parse("1,234,567,890", r).full);     //  OK
+    BOOST_TEST(parse("12,345,678,900", r).full);    //  OK
+    BOOST_TEST(parse("123,456,789,000", r).full);   //  OK
+    BOOST_TEST(!parse("1000,234,567,890", r).full); //  Bad
+    BOOST_TEST(!parse("1,234,56,890", r).full);     //  Bad
+    BOOST_TEST(!parse("1,66", r).full);             //  Bad
 
 //  long long
 
@@ -234,27 +235,27 @@ main()
     int_parser< ::boost::long_long_type> long_long_p;
 
     parse("1234567890123456789", long_long_p[assign_a(ll)]);
-    assert(ll == 1234567890123456789LL);
+    BOOST_TEST(ll == 1234567890123456789LL);
 
     parse("-1234567890123456789", long_long_p[assign_a(ll)]);
-    assert(ll == -1234567890123456789LL);
+    BOOST_TEST(ll == -1234567890123456789LL);
 
     parse(max_long_long, long_long_p[assign_a(ll)]);
-    assert(ll == LONG_LONG_MAX);
+    BOOST_TEST(ll == LONG_LONG_MAX);
 
     parse(min_long_long, long_long_p[assign_a(ll)]);
-    assert(ll == LONG_LONG_MIN);
+    BOOST_TEST(ll == LONG_LONG_MIN);
 
 #if defined(__GNUG__) && (__GNUG__ == 3) && (__GNUC_MINOR__ < 3) \
     && !defined(__EDG__)
     // gcc 3.2.3 crashes on parse(long_long_overflow, long_long_p)
     // wrapping long_long_p into a rule avoids the crash
     rule<> gcc_3_2_3_long_long_r = long_long_p;
-    assert(!parse(long_long_overflow, gcc_3_2_3_long_long_r).full);
-    assert(!parse(long_long_underflow, gcc_3_2_3_long_long_r).full);
+    BOOST_TEST(!parse(long_long_overflow, gcc_3_2_3_long_long_r).full);
+    BOOST_TEST(!parse(long_long_underflow, gcc_3_2_3_long_long_r).full);
 #else
-    assert(!parse(long_long_overflow, long_long_p).full);
-    assert(!parse(long_long_underflow, long_long_p).full);
+    BOOST_TEST(!parse(long_long_overflow, long_long_p).full);
+    BOOST_TEST(!parse(long_long_underflow, long_long_p).full);
 #endif
 
 #endif
@@ -263,51 +264,50 @@ main()
 
     double  d;
 
-    assert(parse("1234", ureal_p[assign_a(d)]).full && d == 1234);      //  Good.
-    assert(parse("1.2e3", ureal_p[assign_a(d)]).full && d == 1.2e3);    //  Good.
-    assert(parse("1.2e-3", ureal_p[assign_a(d)]).full && d == 1.2e-3);  //  Good.
-    assert(parse("1.e2", ureal_p[assign_a(d)]).full && d == 1.e2);      //  Good.
-    assert(parse(".2e3", ureal_p[assign_a(d)]).full && d == .2e3);      //  Good.
-    assert(parse("2e3", ureal_p[assign_a(d)]).full && d == 2e3);        //  Good. No fraction
-    assert(!parse("e3", ureal_p).full);                                 //  Bad! No number
-    assert(!parse("-1.2e3", ureal_p).full);                             //  Bad! Negative number
-    assert(!parse("+1.2e3", ureal_p).full);                             //  Bad! Positive sign
-    assert(!parse("1.2e", ureal_p).full);                               //  Bad! No exponent
-    assert(!parse("-.3", ureal_p).full);                                //  Bad! Negative
+    BOOST_TEST(parse("1234", ureal_p[assign_a(d)]).full && d == 1234);      //  Good.
+    BOOST_TEST(parse("1.2e3", ureal_p[assign_a(d)]).full && d == 1.2e3);    //  Good.
+    BOOST_TEST(parse("1.2e-3", ureal_p[assign_a(d)]).full && d == 1.2e-3);  //  Good.
+    BOOST_TEST(parse("1.e2", ureal_p[assign_a(d)]).full && d == 1.e2);      //  Good.
+    BOOST_TEST(parse(".2e3", ureal_p[assign_a(d)]).full && d == .2e3);      //  Good.
+    BOOST_TEST(parse("2e3", ureal_p[assign_a(d)]).full && d == 2e3);        //  Good. No fraction
+    BOOST_TEST(!parse("e3", ureal_p).full);                                 //  Bad! No number
+    BOOST_TEST(!parse("-1.2e3", ureal_p).full);                             //  Bad! Negative number
+    BOOST_TEST(!parse("+1.2e3", ureal_p).full);                             //  Bad! Positive sign
+    BOOST_TEST(!parse("1.2e", ureal_p).full);                               //  Bad! No exponent
+    BOOST_TEST(!parse("-.3", ureal_p).full);                                //  Bad! Negative
 
-    assert(parse("-1234", real_p[assign_a(d)]).full && d == -1234);     //  Good.
-    assert(parse("-1.2e3", real_p[assign_a(d)]).full && d == -1.2e3);   //  Good.
-    assert(parse("+1.2e3", real_p[assign_a(d)]).full && d == 1.2e3);    //  Good.
-    assert(parse("-0.1", real_p[assign_a(d)]).full && d == -0.1);       //  Good.
-    assert(parse("-1.2e-3", real_p[assign_a(d)]).full && d == -1.2e-3); //  Good.
-    assert(parse("-1.e2", real_p[assign_a(d)]).full && d == -1.e2);     //  Good.
-    assert(parse("-.2e3", real_p[assign_a(d)]).full && d == -.2e3);     //  Good.
-    assert(parse("-2e3", real_p[assign_a(d)]).full && d == -2e3);       //  Good. No fraction
-    assert(!parse("-e3", real_p).full);                                 //  Bad! No number
-    assert(!parse("-1.2e", real_p).full);                               //  Bad! No exponent
+    BOOST_TEST(parse("-1234", real_p[assign_a(d)]).full && d == -1234);     //  Good.
+    BOOST_TEST(parse("-1.2e3", real_p[assign_a(d)]).full && d == -1.2e3);   //  Good.
+    BOOST_TEST(parse("+1.2e3", real_p[assign_a(d)]).full && d == 1.2e3);    //  Good.
+    BOOST_TEST(parse("-0.1", real_p[assign_a(d)]).full && d == -0.1);       //  Good.
+    BOOST_TEST(parse("-1.2e-3", real_p[assign_a(d)]).full && d == -1.2e-3); //  Good.
+    BOOST_TEST(parse("-1.e2", real_p[assign_a(d)]).full && d == -1.e2);     //  Good.
+    BOOST_TEST(parse("-.2e3", real_p[assign_a(d)]).full && d == -.2e3);     //  Good.
+    BOOST_TEST(parse("-2e3", real_p[assign_a(d)]).full && d == -2e3);       //  Good. No fraction
+    BOOST_TEST(!parse("-e3", real_p).full);                                 //  Bad! No number
+    BOOST_TEST(!parse("-1.2e", real_p).full);                               //  Bad! No exponent
 
-    assert(!parse("1234", strict_ureal_p[assign_a(d)]).full);           //  Bad. Strict real
-    assert(parse("1.2", strict_ureal_p[assign_a(d)]).full && d == 1.2); //  Good.
-    assert(!parse("-1234", strict_real_p[assign_a(d)]).full);           //  Bad. Strict real
-    assert(parse("123.", strict_real_p[assign_a(d)]).full && d == 123); //  Good.
-    assert(parse("3.E6", strict_real_p[assign_a(d)]).full && d == 3e6); //  Good.
+    BOOST_TEST(!parse("1234", strict_ureal_p[assign_a(d)]).full);           //  Bad. Strict real
+    BOOST_TEST(parse("1.2", strict_ureal_p[assign_a(d)]).full && d == 1.2); //  Good.
+    BOOST_TEST(!parse("-1234", strict_real_p[assign_a(d)]).full);           //  Bad. Strict real
+    BOOST_TEST(parse("123.", strict_real_p[assign_a(d)]).full && d == 123); //  Good.
+    BOOST_TEST(parse("3.E6", strict_real_p[assign_a(d)]).full && d == 3e6); //  Good.
 
-    assert(!parse("1234.", notrdot_real_p[assign_a(d)]).full);          //  Bad trailing dot
-    assert(!parse(".1234", nolddot_real_p[assign_a(d)]).full);          //  Bad leading dot
+    BOOST_TEST(!parse("1234.", notrdot_real_p[assign_a(d)]).full);          //  Bad trailing dot
+    BOOST_TEST(!parse(".1234", nolddot_real_p[assign_a(d)]).full);          //  Bad leading dot
 
 //  Special thousands separated numbers
 
-    assert(parse("123,456,789.01", ts_real_p[assign_a(d)]).full && d == 123456789.01);  //  Good.
-    assert(parse("12,345,678.90", ts_real_p[assign_a(d)]).full && d == 12345678.90);    //  Good.
-    assert(parse("1,234,567.89", ts_real_p[assign_a(d)]).full && d == 1234567.89);      //  Good.
-    assert(!parse("1234,567,890", ts_real_p).full);     //  Bad.
-    assert(!parse("1,234,5678,9", ts_real_p).full);     //  Bad.
-    assert(!parse("1,234,567.89e6", ts_real_p).full);   //  Bad.
-    assert(!parse("1,66", ts_real_p).full);             //  Bad.
+    BOOST_TEST(parse("123,456,789.01", ts_real_p[assign_a(d)]).full && d == 123456789.01);  //  Good.
+    BOOST_TEST(parse("12,345,678.90", ts_real_p[assign_a(d)]).full && d == 12345678.90);    //  Good.
+    BOOST_TEST(parse("1,234,567.89", ts_real_p[assign_a(d)]).full && d == 1234567.89);      //  Good.
+    BOOST_TEST(!parse("1234,567,890", ts_real_p).full);     //  Bad.
+    BOOST_TEST(!parse("1,234,5678,9", ts_real_p).full);     //  Bad.
+    BOOST_TEST(!parse("1,234,567.89e6", ts_real_p).full);   //  Bad.
+    BOOST_TEST(!parse("1,66", ts_real_p).full);             //  Bad.
 
 //  END TESTS.
 
 /////////////////////////////////////////////////////////////////
-    cout << "success!\n";
-    return 0;
+    return boost::report_errors();
 }
