@@ -48,7 +48,7 @@ static const unsigned long maximum_test_size = 1000000UL;
 #include <boost/thread/xtime.hpp>
 #include <vector>
 #include <algorithm>
-#include <cassert>
+#include <boost/detail/lightweight_test.hpp>
 
 using boost::spirit::impl::object_with_id;
 
@@ -109,6 +109,24 @@ private:
     unsigned int         progress;
 };
 
+template <typename T>
+class callable_reference_wrapper
+    : public boost::reference_wrapper<T>
+{
+public:
+    explicit callable_reference_wrapper(T& t)
+        : boost::reference_wrapper<T>(t)
+    {}
+    inline void operator()() { this->get().operator()(); }
+};
+
+template <typename T>
+callable_reference_wrapper<T>
+callable_ref(T &t)
+{
+    return callable_reference_wrapper<T>(t);
+}
+
 test_task<class1> test1;
 test_task<class1> test2;
 test_task<class1> test3;
@@ -132,7 +150,7 @@ check_ascending(test_task<ClassT> const &t)
         }
         p = n;
     }
-};
+}
 
 struct less1
 {
@@ -164,15 +182,15 @@ check_not_contained_in(
         }
         ++p;
     }
-};
+}
 
 void concurrent_creation_of_objects()
 {
     {
         boost::xtime_get(&start_time, boost::TIME_UTC);
-        boost::thread thread1(boost::ref(test1));
-        boost::thread thread2(boost::ref(test2));
-        boost::thread thread3(boost::ref(test3));
+        boost::thread thread1(callable_ref(test1));
+        boost::thread thread2(callable_ref(test2));
+        boost::thread thread3(callable_ref(test3));
 
         thread1.join();
         thread2.join();
@@ -184,9 +202,9 @@ void local_uniqueness()
 {
 
 
-    assert(test1.data().size()==test_size);
-    assert(test2.data().size()==test_size);
-    assert(test3.data().size()==test_size);
+    BOOST_TEST(test1.data().size()==test_size);
+    BOOST_TEST(test2.data().size()==test_size);
+    BOOST_TEST(test3.data().size()==test_size);
 }
 
 void local_ordering_and_uniqueness()
