@@ -1,10 +1,11 @@
-//  (C) Copyright Gennadiy Rozental 2003-2005.
+//  (C) Copyright Gennadiy Rozental 2003-2006.
 //  Distributed under the Boost Software License, Version 1.0.
 //  (See accompanying file LICENSE_1_0.txt or copy at 
 //  http://www.boost.org/LICENSE_1_0.txt)
 
 //  See http://www.boost.org/libs/test for the library home page.
 
+#include <boost/test/prg_exec_monitor.hpp>
 #include <boost/test/execution_monitor.hpp>
 #include <boost/test/utils/basic_cstring/io.hpp>
 
@@ -61,7 +62,7 @@ void translate_my_exception2( my_exception2 const& ex )
 } // local_namespace
 
 int
-main( int argc , char *[] )
+cpp_main( int argc , char *[] )
 { 
     ::boost::execution_monitor ex_mon;
 
