@@ -1,4 +1,4 @@
-//  (C) Copyright Gennadiy Rozental 2001-2004.
+//  (C) Copyright Gennadiy Rozental 2001-2007.
 //  (C) Copyright Beman Dawes 2001. 
 //  Distributed under the Boost Software License, Version 1.0.
 //  (See accompanying file LICENSE_1_0.txt or copy at 
@@ -6,9 +6,9 @@
 
 //  See http://www.boost.org/libs/test for the library home page.
 //
-//  File        : $RCSfile: prg_exec_fail1.cpp,v $
+//  File        : $RCSfile$
 //
-//  Version     : $Revision: 1.13 $
+//  Version     : $Revision: 41370 $
 //
 //  Description : tests an ability of Program Execution Monitor to catch 
 //  uncatched exceptions. Should fail during run.
@@ -26,6 +26,8 @@
 #include <cstddef> //Metrowerks linker needs at least one standard library
 #endif
 
+#include <boost/test/included/prg_exec_monitor.hpp>
+
 int
 cpp_main( int argc, char *[] )  // note the name
 {
@@ -37,20 +39,4 @@ cpp_main( int argc, char *[] )  // note the name
 
 //____________________________________________________________________________//
 
-// ***************************************************************************
-//  Revision History :
-//  
-//  $Log: prg_exec_fail1.cpp,v $
-//  Revision 1.13  2004/05/21 06:26:10  rogeeff
-//  licence update
-//
-//  Revision 1.12  2004/01/07 22:06:44  beman_dawes
-//  Fix Metrowerks link failures for some compiler configurations. See comment in code. (fix from Ed Swartz of Metrowerks)
-//
-//  Revision 1.11  2003/12/01 00:42:38  rogeeff
-//  prerelease cleaning
-//
-
-// ***************************************************************************
-
 // EOF
