@@ -1,5 +1,6 @@
 // Copyright (C) 2001-2003
 // William E. Kempf
+// Copyright (C) 2007 Anthony Williams
 //
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying 
 //  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -62,10 +63,10 @@ void test_tss_thread()
     }
 }
 
-#if defined(BOOST_HAS_WINTHREADS)
+#if defined(BOOST_THREAD_PLATFORM_WIN32)
     typedef HANDLE native_thread_t;
 
-    DWORD WINAPI test_tss_thread_native(LPVOID lpParameter)
+    DWORD WINAPI test_tss_thread_native(LPVOID /*lpParameter*/)
     {
         test_tss_thread();
         return 0;
@@ -73,7 +74,7 @@ void test_tss_thread()
 
     native_thread_t create_native_thread(void)
     {
-        return CreateThread(
+        native_thread_t const res=CreateThread(
             0, //security attributes (0 = not inheritable)
             0, //stack size (0 = default) 
             &test_tss_thread_native, //function to execute
@@ -81,6 +82,8 @@ void test_tss_thread()
             0, //creation flags (0 = run immediately)
             0  //thread id (0 = thread id not returned)
             );
+        BOOST_CHECK(res!=0);
+        return res;
     }
 
     void join_native_thread(native_thread_t thread)
@@ -91,6 +94,33 @@ void test_tss_thread()
         res = CloseHandle(thread);
         BOOST_CHECK(SUCCEEDED(res));
     }
+#elif defined(BOOST_THREAD_PLATFORM_PTHREAD)
+    typedef pthread_t native_thread_t;
+
+extern "C"
+{
+    void* test_tss_thread_native(void* lpParameter)
+    {
+        test_tss_thread();
+        return 0;
+    }
+}
+
+    native_thread_t create_native_thread()
+    {
+        native_thread_t thread_handle;
+        
+        int const res = pthread_create(&thread_handle, 0, &test_tss_thread_native, 0);
+        BOOST_CHECK(!res);
+        return thread_handle;
+    }
+
+    void join_native_thread(native_thread_t thread)
+    {
+        void* result=0;
+        int const res=pthread_join(thread,&result);
+        BOOST_CHECK(!res);
+    }
 #endif
 
 void do_test_tss()
@@ -100,9 +130,19 @@ void do_test_tss()
 
     const int NUMTHREADS=5;
     boost::thread_group threads;
-    for (int i=0; i<NUMTHREADS; ++i)
-        threads.create_thread(&test_tss_thread);
-    threads.join_all();
+    try
+    {
+        for (int i=0; i<NUMTHREADS; ++i)
+            threads.create_thread(&test_tss_thread);
+        threads.join_all();
+    }
+    catch(...)
+    {
+        threads.interrupt_all();
+        threads.join_all();
+        throw;
+    }
+
 
     std::cout
         << "tss_instances = " << tss_instances
@@ -113,44 +153,33 @@ void do_test_tss()
     BOOST_CHECK_EQUAL(tss_instances, 0);
     BOOST_CHECK_EQUAL(tss_total, 5);
 
-    #if defined(BOOST_HAS_WINTHREADS)
-        tss_instances = 0;
-        tss_total = 0;
+    tss_instances = 0;
+    tss_total = 0;
 
-        native_thread_t thread1 = create_native_thread();
-        BOOST_CHECK(thread1 != 0);
+    native_thread_t thread1 = create_native_thread();
+    native_thread_t thread2 = create_native_thread();
+    native_thread_t thread3 = create_native_thread();
+    native_thread_t thread4 = create_native_thread();
+    native_thread_t thread5 = create_native_thread();
 
-        native_thread_t thread2 = create_native_thread();
-        BOOST_CHECK(thread2 != 0);
+    join_native_thread(thread5);
+    join_native_thread(thread4);
+    join_native_thread(thread3);
+    join_native_thread(thread2);
+    join_native_thread(thread1);
 
-        native_thread_t thread3 = create_native_thread();
-        BOOST_CHECK(thread3 != 0);
+    std::cout
+        << "tss_instances = " << tss_instances
+        << "; tss_total = " << tss_total
+        << "\n";
+    std::cout.flush();
 
-        native_thread_t thread4 = create_native_thread();
-        BOOST_CHECK(thread3 != 0);
-
-        native_thread_t thread5 = create_native_thread();
-        BOOST_CHECK(thread3 != 0);
-
-        join_native_thread(thread5);
-        join_native_thread(thread4);
-        join_native_thread(thread3);
-        join_native_thread(thread2);
-        join_native_thread(thread1);
-
-        std::cout
-            << "tss_instances = " << tss_instances
-            << "; tss_total = " << tss_total
-            << "\n";
-        std::cout.flush();
-
-        // The following is not really an error. TSS cleanup support still is available for boost threads.
-        // Also this usually will be triggered only when bound to the static version of thread lib.
-        // 2006-10-02 Roland Schwarz
-        //BOOST_CHECK_EQUAL(tss_instances, 0);
-        BOOST_CHECK_MESSAGE(tss_instances == 0, "Support of automatic tss cleanup for native threading API not available");
-        BOOST_CHECK_EQUAL(tss_total, 5);
-    #endif
+    // The following is not really an error. TSS cleanup support still is available for boost threads.
+    // Also this usually will be triggered only when bound to the static version of thread lib.
+    // 2006-10-02 Roland Schwarz
+    //BOOST_CHECK_EQUAL(tss_instances, 0);
+    BOOST_CHECK_MESSAGE(tss_instances == 0, "Support of automatic tss cleanup for native threading API not available");
+    BOOST_CHECK_EQUAL(tss_total, 5);
 }
 
 void test_tss()
@@ -158,12 +187,90 @@ void test_tss()
     timed_test(&do_test_tss, 2);
 }
 
+bool tss_cleanup_called=false;
+
+struct Dummy
+{};
+
+void tss_custom_cleanup(Dummy* d)
+{
+    delete d;
+    tss_cleanup_called=true;
+}
+
+boost::thread_specific_ptr<Dummy> tss_with_cleanup(tss_custom_cleanup);
+
+void tss_thread_with_custom_cleanup()
+{
+    tss_with_cleanup.reset(new Dummy);
+}
+
+void do_test_tss_with_custom_cleanup()
+{
+    boost::thread t(tss_thread_with_custom_cleanup);
+    try
+    {
+        t.join();
+    }
+    catch(...)
+    {
+        t.interrupt();
+        t.join();
+        throw;
+    }
+
+    BOOST_CHECK(tss_cleanup_called);
+}
+
+
+void test_tss_with_custom_cleanup()
+{
+    timed_test(&do_test_tss_with_custom_cleanup, 2);
+}
+
+Dummy* tss_object=new Dummy;
+
+void tss_thread_with_custom_cleanup_and_release()
+{
+    tss_with_cleanup.reset(tss_object);
+    tss_with_cleanup.release();
+}
+
+void do_test_tss_does_no_cleanup_after_release()
+{
+    tss_cleanup_called=false;
+    boost::thread t(tss_thread_with_custom_cleanup_and_release);
+    try
+    {
+        t.join();
+    }
+    catch(...)
+    {
+        t.interrupt();
+        t.join();
+        throw;
+    }
+
+    BOOST_CHECK(!tss_cleanup_called);
+    if(!tss_cleanup_called)
+    {
+        delete tss_object;
+    }
+}
+
+void test_tss_does_no_cleanup_after_release()
+{
+    timed_test(&do_test_tss_does_no_cleanup_after_release, 2);
+}
+
 boost::unit_test_framework::test_suite* init_unit_test_suite(int, char*[])
 {
     boost::unit_test_framework::test_suite* test =
         BOOST_TEST_SUITE("Boost.Threads: tss test suite");
 
     test->add(BOOST_TEST_CASE(test_tss));
+    test->add(BOOST_TEST_CASE(test_tss_with_custom_cleanup));
+    test->add(BOOST_TEST_CASE(test_tss_does_no_cleanup_after_release));
 
     return test;
 }
