@@ -5,7 +5,7 @@
     
     http://www.boost.org/
 
-    Copyright (c) 2001-2005 Hartmut Kaiser. Distributed under the Boost 
+    Copyright (c) 2001-2008 Hartmut Kaiser. Distributed under the Boost 
     Software License, Version 1.0. (See accompanying file 
     LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 =============================================================================*/
@@ -29,6 +29,10 @@
 #include <boost/wave/cpplexer/validate_universal_char.hpp>
 #include <boost/wave/cpplexer/convert_trigraphs.hpp>
 #include <boost/wave/cpplexer/cpplexer_exceptions.hpp>
+#if BOOST_WAVE_SUPPORT_PRAGMA_ONCE != 0
+#include <boost/wave/cpplexer/detect_include_guards.hpp>
+#endif
+#include <boost/wave/cpplexer/cpp_lex_interface.hpp>
 
 #include "../slex_interface.hpp"
 #include "../slex_token.hpp"
@@ -44,15 +48,15 @@ namespace slex {
 namespace lexer {
 
 ///////////////////////////////////////////////////////////////////////////////
-//  The following numbers are the arraysizes of the token regex's which we
+//  The following numbers are the array sizes of the token regex's which we
 //  need to specify to make the CW compiler happy (at least up to V9.5).
 #if BOOST_WAVE_SUPPORT_MS_EXTENSIONS != 0
-#define INIT_DATA_SIZE        176
-#define INIT_DATA_CPP_SIZE    15
+#define INIT_DATA_SIZE              175
 #else
-#define INIT_DATA_SIZE        159
-#define INIT_DATA_CPP_SIZE    15
+#define INIT_DATA_SIZE              158
 #endif
+#define INIT_DATA_CPP_SIZE          15
+#define INIT_DATA_PP_NUMBER_SIZE    2
 
 ///////////////////////////////////////////////////////////////////////////////
 // 
@@ -107,15 +111,16 @@ private:
 
     static typename base_type::lexer_data const init_data[INIT_DATA_SIZE];          // common patterns
     static typename base_type::lexer_data const init_data_cpp[INIT_DATA_CPP_SIZE];  // C++ only patterns
+    static typename base_type::lexer_data const init_data_pp_number[INIT_DATA_PP_NUMBER_SIZE];  // pp-number only patterns
 };
 
 ///////////////////////////////////////////////////////////////////////////////
 //  data required for initialization of the lexer (token definitions)
-#define OR      "|"
-#define Q(c)    "\\" c
-#define TRI(c)  Q("?") Q("?") c
+#define OR                  "|"
+#define Q(c)                "\\" c
+#define TRI(c)              Q("?") Q("?") c
 
-// definition of some subtoken regexps to simplify the regex definitions
+// definition of some sub-token regexps to simplify the regex definitions
 #define BLANK               "[ \\t]"
 #define CCOMMENT            \
     Q("/") Q("*") "[^*]*" Q("*") "+" "(" "[^/*][^*]*" Q("*") "+" ")*" Q("/")
@@ -125,33 +130,42 @@ private:
 #define OCTALDIGIT          "[0-7]"
 #define DIGIT               "[0-9]"
 #define HEXDIGIT            "[0-9a-fA-F]"
-#define SIGN                "[-+]?"
-#define EXPONENT            "(" "[eE]" SIGN "[0-9]+" ")"
+#define OPTSIGN             "[-+]?"
+#define EXPSTART            "[eE]" "[-+]"
+#define EXPONENT            "(" "[eE]" OPTSIGN "[0-9]+" ")"
+#define NONDIGIT            "[a-zA-Z_]"
 
 #define INTEGER             \
     "(" "(0x|0X)" HEXDIGIT "+" OR "0" OCTALDIGIT "*" OR "[1-9]" DIGIT "*" ")"
             
 #define INTEGER_SUFFIX      "(" "[uU][lL]?|[lL][uU]?" ")"
-#define LONGINTEGER_SUFFIX  "(" "[uU]" "(" "ll" OR "LL" ")" OR \
-                            "(" "ll" OR "LL" ")" "[uU]" "?" ")"
-#define FLOAT_SUFFIX        "(" "[fF][lL]?|[lL][fF]?" ")"
+#if BOOST_WAVE_SUPPORT_MS_EXTENSIONS != 0
+#define LONGINTEGER_SUFFIX  "(" "[uU]" "(" "[lL][lL]" ")" OR \
+                                "(" "[lL][lL]" ")" "[uU]" "?" OR \
+                                "i64" \
+                            ")" 
+#else
+#define LONGINTEGER_SUFFIX  "(" "[uU]" "(" "[lL][lL]" ")" OR \
+                            "(" "[lL][lL]" ")" "[uU]" "?" ")"
+#endif
+#define FLOAT_SUFFIX        "(" "[fF][lL]?" OR "[lL][fF]?" ")"
 #define CHAR_SPEC           "L?"
 
 #define BACKSLASH           "(" Q("\\") OR TRI(Q("/")) ")"
-#define ESCAPESEQ           BACKSLASH "(" \
+#define ESCAPESEQ           "(" BACKSLASH "(" \
                                 "[abfnrtv?'\"]" OR \
                                 BACKSLASH OR \
                                 "x" HEXDIGIT "+" OR \
                                 OCTALDIGIT OCTALDIGIT "?" OCTALDIGIT "?" \
-                            ")"
-#define HEXQUAD             HEXDIGIT HEXDIGIT HEXDIGIT HEXDIGIT 
-#define UNIVERSALCHAR       BACKSLASH "(" \
+                            "))"
+#define HEXQUAD             "(" HEXDIGIT HEXDIGIT HEXDIGIT HEXDIGIT ")"
+#define UNIVERSALCHAR       "(" BACKSLASH "(" \
                                 "u" HEXQUAD OR \
                                 "U" HEXQUAD HEXQUAD \
-                            ")" 
+                            "))" 
 
 #define POUNDDEF            "(" "#" OR TRI("=") OR Q("%:") ")"
-#define NEWLINEDEF          "(" "\\n" OR "\\r" OR "\\r\\n" ")"
+#define NEWLINEDEF          "(" "\n" OR "\r" OR "\r\n" ")"
 
 #if BOOST_WAVE_SUPPORT_INCLUDE_NEXT != 0
 #define INCLUDEDEF          "(include|include_next)"
@@ -159,8 +173,10 @@ private:
 #define INCLUDEDEF          "include"
 #endif
 
+#define PP_NUMBERDEF        Q(".") "?" DIGIT "(" DIGIT OR NONDIGIT OR EXPSTART OR Q(".") ")*"
+
 ///////////////////////////////////////////////////////////////////////////////
-//  sexer state constants
+//  lexer state constants
 #define LEXER_STATE_NORMAL  0
 #define LEXER_STATE_PP      1
 
@@ -255,7 +271,6 @@ lexer<IteratorT, PositionT>::init_data[INIT_DATA_SIZE] =
     TOKEN_DATA(CONSTCAST, "const_cast"),
     TOKEN_DATA(CONTINUE, "continue"),
     TOKEN_DATA(DEFAULT, "default"),
-//    TOKEN_DATA(DEFINED, "defined"),
     TOKEN_DATA(DELETE, "delete"),
     TOKEN_DATA(DO, "do"),
     TOKEN_DATA(DOUBLE, "double"),
@@ -342,25 +357,29 @@ lexer<IteratorT, PositionT>::init_data[INIT_DATA_SIZE] =
     TOKEN_DATA(MSEXT_PP_REGION, POUNDDEF PPSPACE "region"),
     TOKEN_DATA(MSEXT_PP_ENDREGION, POUNDDEF PPSPACE "endregion"),
 #endif // BOOST_WAVE_SUPPORT_MS_EXTENSIONS != 0
-    TOKEN_DATA(IDENTIFIER, "([a-zA-Z_]" OR UNIVERSALCHAR ")([a-zA-Z0-9_]" OR UNIVERSALCHAR ")*"),
 //  TOKEN_DATA(OCTALINT, "0" OCTALDIGIT "*" INTEGER_SUFFIX "?"),
 //  TOKEN_DATA(DECIMALINT, "[1-9]" DIGIT "*" INTEGER_SUFFIX "?"),
 //  TOKEN_DATA(HEXAINT, "(0x|0X)" HEXDIGIT "+" INTEGER_SUFFIX "?"),
-    TOKEN_DATA(INTLIT, INTEGER INTEGER_SUFFIX "?"),
     TOKEN_DATA(LONGINTLIT, INTEGER LONGINTEGER_SUFFIX),
+    TOKEN_DATA(INTLIT, INTEGER INTEGER_SUFFIX "?"),
     TOKEN_DATA(FLOATLIT, 
         "(" DIGIT "*" Q(".") DIGIT "+" OR DIGIT "+" Q(".") ")" 
         EXPONENT "?" FLOAT_SUFFIX "?" OR
         DIGIT "+" EXPONENT FLOAT_SUFFIX "?"),
+#if BOOST_WAVE_USE_STRICT_LEXER != 0
+    TOKEN_DATA(IDENTIFIER, "([a-zA-Z_]" OR UNIVERSALCHAR ")([a-zA-Z0-9_]" OR UNIVERSALCHAR ")*"),
+#else
+    TOKEN_DATA(IDENTIFIER, "([a-zA-Z_$]" OR UNIVERSALCHAR ")([a-zA-Z0-9_$]" OR UNIVERSALCHAR ")*"),
+#endif
     TOKEN_DATA(CCOMMENT, CCOMMENT),
     TOKEN_DATA(CPPCOMMENT, Q("/") Q("/[^\\n\\r]*") NEWLINEDEF ),
     TOKEN_DATA(CHARLIT, CHAR_SPEC "'" 
                 "(" ESCAPESEQ OR "[^\\n\\r']" OR UNIVERSALCHAR ")+" "'"),
     TOKEN_DATA(STRINGLIT, CHAR_SPEC Q("\"") 
                 "(" ESCAPESEQ OR "[^\\n\\r\"]" OR UNIVERSALCHAR ")*" Q("\"")),
-    TOKEN_DATA(SPACE, BLANK "+"),
-    TOKEN_DATA(SPACE2, "[\\v\\f]+"),
-    TOKEN_DATA(CONTLINE, Q("\\") "\\n"), 
+    TOKEN_DATA(SPACE, "[ \t\v\f]+"),
+//    TOKEN_DATA(SPACE2, "[\\v\\f]+"),
+    TOKEN_DATA(CONTLINE, "\\" "\\n"), 
     TOKEN_DATA(NEWLINE, NEWLINEDEF),
     TOKEN_DATA(POUND_POUND, "##"),
     TOKEN_DATA(POUND_POUND_ALT, Q("%:") Q("%:")),
@@ -368,9 +387,9 @@ lexer<IteratorT, PositionT>::init_data[INIT_DATA_SIZE] =
     TOKEN_DATA(POUND, "#"),
     TOKEN_DATA(POUND_ALT, Q("%:")),
     TOKEN_DATA(POUND_TRIGRAPH, TRI("=")),
-    TOKEN_DATA(ANY, "."),
     TOKEN_DATA(ANY_TRIGRAPH, TRI(Q("/"))),
-    { token_id(0) }       // this should be the last entry
+    TOKEN_DATA(ANY, "."),     // this should be the last recognized token
+    { token_id(0) }           // this should be the last entry
 };
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -390,12 +409,25 @@ lexer<IteratorT, PositionT>::init_data_cpp[INIT_DATA_CPP_SIZE] =
     TOKEN_DATA(NOTEQUAL_ALT, "not_eq"),
     TOKEN_DATA(NOT_ALT, "not"),
     TOKEN_DATA(COMPL_ALT, "compl"),
+#if BOOST_WAVE_SUPPORT_IMPORT_KEYWORD != 0
+    TOKEN_DATA(IMPORT, "import"),
+#endif
     TOKEN_DATA(ARROWSTAR, Q("->") Q("*")),
     TOKEN_DATA(DOTSTAR, Q(".") Q("*")),
     TOKEN_DATA(COLON_COLON, "::"),
     { token_id(0) }       // this should be the last entry
 };
 
+///////////////////////////////////////////////////////////////////////////////
+// C++ only token definitions
+template <typename IteratorT, typename PositionT>
+typename lexer_base<IteratorT, PositionT>::lexer_data const 
+lexer<IteratorT, PositionT>::init_data_pp_number[INIT_DATA_PP_NUMBER_SIZE] = 
+{
+    TOKEN_DATA(PP_NUMBER, PP_NUMBERDEF),
+    { token_id(0) }       // this should be the last entry
+};
+
 ///////////////////////////////////////////////////////////////////////////////
 //  undefine macros, required for regular expression definitions
 #undef INCLUDEDEF
@@ -405,7 +437,9 @@ lexer<IteratorT, PositionT>::init_data_cpp[INIT_DATA_CPP_SIZE] =
 #undef DIGIT
 #undef OCTALDIGIT
 #undef HEXDIGIT
-#undef SIGN
+#undef NONDIGIT
+#undef OPTSIGN
+#undef EXPSTART
 #undef EXPONENT
 #undef LONGINTEGER_SUFFIX
 #undef INTEGER_SUFFIX
@@ -416,6 +450,7 @@ lexer<IteratorT, PositionT>::init_data_cpp[INIT_DATA_CPP_SIZE] =
 #undef ESCAPESEQ    
 #undef HEXQUAD      
 #undef UNIVERSALCHAR
+#undef PP_NUMBERDEF
 
 #undef Q
 #undef TRI
@@ -439,6 +474,15 @@ lexer<IteratorT, PositionT>::init_dfa(boost::wave::language_support lang)
 {
     if (this->has_compiled_dfa())
         return;
+
+// if pp-numbers should be preferred, insert the corresponding rule first
+    if (boost::wave::need_prefer_pp_numbers(lang)) {
+        for (int j = 0; 0 != init_data_pp_number[j].tokenid; ++j) {
+            this->register_regex(init_data_pp_number[j].tokenregex, 
+                init_data_pp_number[j].tokenid, init_data_pp_number[j].tokencb, 
+                init_data_pp_number[j].lexerstate);
+        }
+    }
         
 // if in C99 mode, some of the keywords are not valid    
     if (!boost::wave::need_c99(lang)) {
@@ -525,11 +569,11 @@ public:
     typedef typename lexer::lexer<IteratorT, PositionT>::token_type token_type;
 
     slex_functor(IteratorT const &first_, IteratorT const &last_, 
-            PositionT const &pos_, boost::wave::language_support language)
-    :   first(first_, last_, pos_), language(language), at_eof(false)
+            PositionT const &pos_, boost::wave::language_support language_)
+    :   first(first_, last_, pos_), language(language_), at_eof(false)
     {
         // initialize lexer dfa tables
-        init_lexer(lexer, language);  
+        init_lexer(lexer, language_);  
     }
     virtual ~slex_functor() {}
 
@@ -557,7 +601,7 @@ public:
                     case T_IDENTIFIER:
                     // test identifier characters for validity (throws if 
                     // invalid chars found)
-                        if (!(language & support_option_no_character_validation)) {
+                        if (!boost::wave::need_no_character_validation(language)) {
                             using boost::wave::cpplexer::impl::validate_identifier_name;
                             validate_identifier_name(token_val, 
                                 pos.get_line(), pos.get_column(), pos.get_file()); 
@@ -568,12 +612,11 @@ public:
                     case T_CHARLIT:
                     // test literal characters for validity (throws if invalid 
                     // chars found)
-                        if (language & support_option_convert_trigraphs) {
+                        if (boost::wave::need_convert_trigraphs(language)) {
                             using boost::wave::cpplexer::impl::convert_trigraphs;
-                            token_val = convert_trigraphs(token_val, 
-                                pos.get_line(), pos.get_column(), pos.get_file()); 
+                            token_val = convert_trigraphs(token_val); 
                         }
-                        if (!(language & support_option_no_character_validation)) {
+                        if (!boost::wave::need_no_character_validation(language)) {
                             using boost::wave::cpplexer::impl::validate_literal;
                             validate_literal(token_val, 
                                 pos.get_line(), pos.get_column(), pos.get_file()); 
@@ -622,30 +665,39 @@ public:
                     case T_COMPL_TRIGRAPH:
                     case T_POUND_TRIGRAPH:
                     case T_ANY_TRIGRAPH:
-                        if (language & support_option_convert_trigraphs)
+                        if (boost::wave::need_convert_trigraphs(language))
                         {
                             using boost::wave::cpplexer::impl::convert_trigraph;
-                            token_val = convert_trigraph(
-                                token_val, pos.get_line(), pos.get_column(), 
-                                pos.get_file());
+                            token_val = convert_trigraph(token_val);
                         }
                         break;
                     }
+                    
+#if BOOST_WAVE_SUPPORT_PRAGMA_ONCE != 0
+                    return guards.detect_guard(token_type(id, token_val, pos));
+#else
                     return token_type(id, token_val, pos);
+#endif
                 }
             
             // skip the T_CONTLINE token
             } while (true);
         }
-        return token;       // return T_EOI
+        return token;   // return T_EOI
     }
+    
     void set_position(PositionT const &pos) 
     { 
         // set position has to change the file name and line number only
         first.get_position().set_file(pos.get_file()); 
         first.get_position().set_line(pos.get_line()); 
     }
-    
+
+#if BOOST_WAVE_SUPPORT_PRAGMA_ONCE != 0
+    bool has_include_guards(std::string& guard_name) const 
+        { return guards.detected(guard_name); }
+#endif
+
 private:
     iterator_type first;
     iterator_type last;
@@ -653,6 +705,10 @@ private:
     static lexer::lexer<IteratorT, PositionT> lexer;   // needed only once
     
     bool at_eof;
+
+#if BOOST_WAVE_SUPPORT_PRAGMA_ONCE != 0
+    include_guards<token_type> guards;
+#endif
 };
 
 template <typename IteratorT, typename PositionT>
@@ -693,7 +749,7 @@ lexer::lexer<IteratorT, PositionT> slex_functor<IteratorT, PositionT>::lexer;
 
 template <typename IteratorT, typename PositionT>
 BOOST_WAVE_SLEX_NEW_LEXER_INLINE
-slex_input_interface<slex_token<PositionT> > *
+lex_input_interface<slex_token<PositionT> > *
 new_lexer_gen<IteratorT, PositionT>::new_lexer(IteratorT const &first,
     IteratorT const &last, PositionT const &pos, 
     boost::wave::language_support language)
