@@ -5,7 +5,7 @@
     
     http://www.boost.org/
 
-    Copyright (c) 2001-2007 Hartmut Kaiser. Distributed under the Boost 
+    Copyright (c) 2001-2008 Hartmut Kaiser. Distributed under the Boost 
     Software License, Version 1.0. (See accompanying file 
     LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 =============================================================================*/
@@ -55,7 +55,7 @@ public:
             boost::wave::language_support language)
     :   functor_ptr(slex_input_interface<TokenT>
             ::new_lexer(first, last, pos, language)) 
-#if 0 != __DECCXX_VER
+#if 0 != __DECCXX_VER || BOOST_INTEL_CXX_VERSION > 900 || defined(__PGI)
       , eof()
 #endif // 0 != __DECCXX_VER
     {}
@@ -84,7 +84,7 @@ public:
 #endif    
 
 private:
-    boost::shared_ptr<slex_input_interface<TokenT> > functor_ptr;
+    boost::shared_ptr<lex_input_interface<TokenT> > functor_ptr;
 };
 
 #if 0 != __COMO_VERSION__
