@@ -11,14 +11,21 @@
 ///////////////////////////////////////////////////////////////////////////////
 // test_main
 //   read the tests from the input file and execute them
-int test_main( int, char*[] )
+void test_main()
 {
 #ifndef BOOST_XPRESSIVE_NO_WREGEX
     typedef std::wstring::const_iterator iterator_type;
-    boost::iterator_range<test_case<iterator_type> const *> rng = get_test_cases<iterator_type>();
+    boost::iterator_range<xpr_test_case<iterator_type> const *> rng = get_test_cases<iterator_type>();
     std::for_each(rng.begin(), rng.end(), test_runner<iterator_type>());
 #endif
-
-    return 0;
 }
 
+///////////////////////////////////////////////////////////////////////////////
+// init_unit_test_suite
+//
+test_suite* init_unit_test_suite( int argc, char* argv[] )
+{
+    test_suite *test = BOOST_TEST_SUITE("test3u");
+    test->add(BOOST_TEST_CASE(&test_main));
+    return test;
+}
