@@ -1,7 +1,9 @@
 //  inspect program  ---------------------------------------------------------//
 
 //  Copyright Beman Dawes 2002.
-//  Copyright Rene Rivera 2004.
+//  Copyright Rene Rivera 2004-2006.
+//  Copyright Gennaro Prota 2006.
+
 //  Distributed under the Boost Software License, Version 1.0.
 //  (See accompanying file LICENSE_1_0.txt or copy at
 //  http://www.boost.org/LICENSE_1_0.txt)
@@ -11,24 +13,23 @@
 //  and the code that identifies library names assumes the Boost directory
 //  structure.
 
-//  See http://www.boost.org/tools/inspect for more information.
+//  See http://www.boost.org/tools/inspect/ for more information.
 
-#include <boost/shared_ptr.hpp>
-#include <boost/filesystem/exception.hpp>
-#include <boost/filesystem/operations.hpp>
-#include <boost/filesystem/fstream.hpp>
 
-#include <iostream>
-#include <cassert>
 #include <vector>
 #include <list>
-#include <utility>
 #include <algorithm>
 #include <cstring>
 
-#include "inspector.hpp" // includes <string>, <boost/filesystem/path.hpp>,
-                         // <iostream>, <set>
-                         // and gives using for string and path.
+#include "boost/shared_ptr.hpp"
+#include "boost/filesystem/operations.hpp"
+#include "boost/filesystem/fstream.hpp"
+
+#include "time_string.hpp"
+
+#include "inspector.hpp"
+
+// the inspectors
 #include "copyright_check.hpp"
 #include "crlf_check.hpp"
 #include "license_check.hpp"
@@ -36,26 +37,31 @@
 #include "long_name_check.hpp"
 #include "tab_check.hpp"
 #include "minmax_check.hpp"
+#include "unnamed_namespace_check.hpp"
+
 #include "cvs_iterator.hpp"
 
+#include "boost/test/included/prg_exec_monitor.hpp"
+
 namespace fs = boost::filesystem;
 
 namespace
 {
-  typedef boost::shared_ptr< boost::inspect::inspector > inspector_ptr;
-
-  struct inspector_element
+  class inspector_element
   {
-    inspector_ptr  inspector;
+    typedef boost::shared_ptr< boost::inspect::inspector > inspector_ptr;
 
+  public:
+    inspector_ptr  inspector;
+    explicit
     inspector_element( boost::inspect::inspector * p ) : inspector(p) {}
   };
 
   typedef std::list< inspector_element > inspector_list;
 
-  long file_count;
-  long directory_count;
-  long error_count;
+  long file_count = 0;
+  long directory_count = 0;
+  long error_count = 0;
 
   boost::inspect::string_set content_signatures;
 
@@ -78,18 +84,50 @@ namespace
   typedef std::vector< error_msg > error_msg_vector;
   error_msg_vector msgs;
 
+//  get info (as a string) if inspect_root is svn working copy  --------------//
+
+  string info( const fs::path & inspect_root )
+  {
+    string rev;
+    string repos;
+    fs::path entries( inspect_root / ".svn" / "entries" );
+    fs::ifstream entries_file( entries );
+    if ( entries_file )
+    {
+      std::getline( entries_file, rev );
+      std::getline( entries_file, rev );
+      std::getline( entries_file, rev );
+      std::getline( entries_file, rev );    // revision number as a string
+      std::getline( entries_file, repos );  // repository as a string
+    }
+    return repos + " at revision " + rev;
+  }
+
 //  visit_predicate (determines which directories are visited)  --------------//
 
   typedef bool(*pred_type)(const path&);
 
   bool visit_predicate( const path & pth )
   {
+    string local( boost::inspect::relative_to( pth, fs::initial_path() ) );
     string leaf( pth.leaf() );
     return
+      // so we can inspect a checkout
       leaf != "CVS"
+      // don't look at binaries
       && leaf != "bin"
-      && leaf != "jam_src" // this really out of our hands
-      && leaf != "status"  // too many issues with generated HTML files
+      && leaf != "bin.v2"
+      // this really out of our hands
+      && leaf != "jam_src"
+      && local.find("tools/jam/src") != 0
+      // too many issues with generated HTML files
+      && leaf != "status"
+      // no point in checking doxygen xml output
+      && local.find("doc/xml") != 0
+      // ignore some web files
+      && leaf != ".htaccess"
+      // ignore svn files:
+      && leaf != ".svn"
       ;
   }
 
@@ -97,17 +135,25 @@ namespace
 
   string library_from_content( const string & content )
   {
-    string::size_type pos( content.find( "www.boost.org/libs/" ) );
+    const string unknown_library ( "unknown" );
+    const string lib_root ( "www.boost.org/libs/" );
+    string::size_type pos( content.find( lib_root ) );
 
-    if ( pos == string::npos ) return "unknown";
+    string lib = unknown_library;
+
+    if ( pos != string::npos ) {
+
+        pos += lib_root.length();
+
+        const char delims[] = " " // space and...
+                              "/\n\r\t";
+
+        string::size_type n = content.find_first_of( string(delims), pos );
+        if (n != string::npos)
+            lib = string(content, pos, n - pos);
+        
+    }
 
-    string lib;
-    pos += 19;
-    while ( content[pos] != ' '
-      && content[pos] != '/'
-      && content[pos] != '\n'
-      && content[pos] != '\r'
-      && content[pos] != '\t' ) lib += content[pos++];
     return lib;
   }
 
@@ -184,36 +230,68 @@ namespace
         ++file_count;
         string content;
         load_content( *itr, content );
-        check( lib == "unknown"
-          ? library_from_content( content ) : lib, *itr, content, insps );
+        check( lib.empty()
+                ? library_from_content( content ) : lib
+               , *itr, content, insps );
       }
     }
   }
 
+//  display  -----------------------------------------------------------------//
+
+  enum display_format_type
+  {
+    display_html, display_text
+  }
+  display_format = display_html;
+
+  enum display_mode_type
+  {
+    display_full, display_brief
+  }
+  display_mode = display_full;
+
 //  display_summary_helper  --------------------------------------------------//
 
   void display_summary_helper( const string & current_library, int err_count )
   {
-    std::cout << "  <tr><td><a href=\"#" 
-              << current_library
-              << "\">" << current_library
-              << "</a></td><td align=\"center\">"
-              << err_count << "</td></tr>\n";
+    if (display_text == display_format)
+    {
+        std::cout << "  " << current_library << " (" << err_count << ")\n";
+    }
+    else
+    {
+      std::cout
+        << "  <tr><td><a href=\"#"
+        << current_library          // what about malformed for URI refs? [gps]
+        << "\">" << current_library
+        << "</a></td><td align=\"center\">"
+        << err_count << "</td></tr>\n";
+    }
   }
-  
+
 //  display_summary  ---------------------------------------------------------//
 
   void display_summary()
   {
-    std::cout << "</pre>\n"
-            "<h2>Summary</h2>\n"
-            "<table border=\"1\" cellpadding=\"5\" cellspacing=\"0\">\n"
-            "  <tr>\n"
-            "    <td><b>Library</b></td>\n"
-            "    <td><b>Problems</b></td>\n"
-            "  </tr>\n"
-            ;
-    string current_library( msgs.begin()->library ); 
+    if (display_text == display_format)
+    {
+      std::cout << "Summary:\n";
+    }
+    else
+    {
+      std::cout
+        << "</pre>\n"
+        "<h2>Summary</h2>\n"
+        "<table border=\"1\" cellpadding=\"5\" cellspacing=\"0\">\n"
+        "  <tr>\n"
+        "    <td><b>Library</b></td>\n"
+        "    <td><b>Problems</b></td>\n"
+        "  </tr>\n"
+        ;
+    }
+
+    string current_library( msgs.begin()->library );
     int err_count = 0;
     for ( error_msg_vector::iterator itr ( msgs.begin() );
       itr != msgs.end(); ++itr )
@@ -228,7 +306,14 @@ namespace
     }
     display_summary_helper( current_library, err_count );
 
-    std::cout << "</table>\n";
+    if (display_text == display_format)
+    {
+      std::cout << "\n";
+    }
+    else
+    {
+      std::cout << "</table>\n";
+    }
   }
 
 
@@ -236,41 +321,95 @@ namespace
 
   void display_details()
   {
-    std::cout << "<h2>Details</h2>\n";
+    // gps - review this
 
-    // display error messages with group indication
-    error_msg current;
-    string sep;
-    bool first = true;
-    for ( error_msg_vector::iterator itr ( msgs.begin() );
-      itr != msgs.end(); ++itr )
+    if (display_text == display_format)
     {
-      if ( current.library != itr->library )
+      // display error messages with group indication
+      error_msg current;
+      string sep;
+      for ( error_msg_vector::iterator itr ( msgs.begin() );
+        itr != msgs.end(); ++itr )
       {
-        if ( !first ) std::cout << "</pre>\n";
-        std::cout << "\n<h3><a name=\"" << itr->library
-                  << "\">" << itr->library << "</a></h3>\n<pre>";
+        if ( current.library != itr->library )
+        {
+          if ( display_full == display_mode )
+              std::cout << "\n|" << itr->library << "|\n";
+          else
+              std::cout << "\n\n|" << itr->library << '|';
+        }
+
+        if ( current.library != itr->library
+          || current.rel_path != itr->rel_path )
+        {
+          if ( display_full == display_mode )
+          {
+            std::cout << "  " << itr->rel_path << ":\n";
+          }
+          else
+          {
+            path current_rel_path(current.rel_path);
+            path this_rel_path(itr->rel_path);
+            if (current_rel_path.branch_path() != this_rel_path.branch_path())
+            {
+              std::cout << "\n  " << this_rel_path.branch_path().string() << '/';
+            }
+            std::cout << "\n    " << this_rel_path.leaf() << ':';
+          }
+        }
+        if ( current.library != itr->library
+          || current.rel_path != itr->rel_path
+          || current.msg != itr->msg )
+        {
+          const string m = itr->msg;
+
+          if ( display_full == display_mode )
+              std::cout << "    " << m << '\n';
+          else
+              std::cout << ' ' << m;
+        }
+        current.library = itr->library;
+        current.rel_path = itr->rel_path;
+        current.msg = itr->msg;
       }
-      if ( current.library != itr->library
-        || current.rel_path != itr->rel_path )
+      std::cout << "\n";
+    }
+    else
+    {
+      // display error messages with group indication
+      error_msg current;
+      string sep;
+      bool first = true;
+      for ( error_msg_vector::iterator itr ( msgs.begin() );
+        itr != msgs.end(); ++itr )
       {
-        std::cout << "\n";
-        std::cout << itr->rel_path;
-        sep = ": ";
+        if ( current.library != itr->library )
+        {
+          if ( !first ) std::cout << "</pre>\n";
+          std::cout << "\n<h3><a name=\"" << itr->library
+                    << "\">" << itr->library << "</a></h3>\n<pre>";
+        }
+        if ( current.library != itr->library
+          || current.rel_path != itr->rel_path )
+        {
+          std::cout << "\n";
+          std::cout << itr->rel_path;
+          sep = ": ";
+        }
+        if ( current.library != itr->library
+          || current.rel_path != itr->rel_path
+          || current.msg != itr->msg )
+        {
+          std::cout << sep << itr->msg;
+          sep = ", ";
+        }
+        current.library = itr->library;
+        current.rel_path = itr->rel_path;
+        current.msg = itr->msg;
+        first = false;
       }
-      if ( current.library != itr->library
-        || current.rel_path != itr->rel_path
-        || current.msg != itr->msg )
-      {
-        std::cout << sep << itr->msg;
-        sep = ", ";
-      }
-      current.library = itr->library;
-      current.rel_path = itr->rel_path;
-      current.msg = itr->msg;
-      first = false;
-   }
-   std::cout << "</pre>\n";
+      std::cout << "</pre>\n";
+    }
   }
 
   const char * options()
@@ -283,7 +422,27 @@ namespace
          "  -long_name\n"
          "  -tab\n"
          "  -minmax\n"
-         "default is all checks on; otherwise options specify desired checks\n";
+         "  -unnamed\n"
+         " default is all checks on; otherwise options specify desired checks"
+         "\n";
+  }
+
+  const char * doctype_declaration()
+  {
+    return
+         "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\"\n"
+         "\"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">"
+         ;
+  }
+
+  std::string validator_link(const std::string & text)
+  {
+    return
+        // with link to validation service
+        "<a href=\"http://validator.w3.org/check?uri=referer\">"
+        + text
+        + "</a>"
+        ;
   }
 
 } // unnamed namespace
@@ -318,7 +477,7 @@ namespace boost
 //        << msg << '\n';
 
     }
-    
+
     source_inspector::source_inspector()
     {
       // C/C++ source code...
@@ -331,12 +490,12 @@ namespace boost
       register_signature( ".hxx" );
       register_signature( ".inc" );
       register_signature( ".ipp" );
-      
+
       // Boost.Build BJam source code...
       register_signature( "Jamfile" );
       register_signature( ".jam" );
       register_signature( ".v2" );
-      
+
       // Other scripts; Python, shell, autoconfig, etc.
       register_signature( "configure.in" );
       register_signature( "GNUmakefile" );
@@ -346,7 +505,7 @@ namespace boost
       register_signature( ".pl" );
       register_signature( ".py" );
       register_signature( ".sh" );
-      
+
       // Hypertext, Boost.Book, and other text...
       register_signature( "news" );
       register_signature( "readme" );
@@ -364,8 +523,9 @@ namespace boost
       register_signature( ".xml" );
       register_signature( ".xsd" );
       register_signature( ".xsl" );
+      register_signature( ".qbk" );
     }
-    
+
     hypertext_inspector::hypertext_inspector()
     {
       register_signature( ".htm" );
@@ -375,18 +535,19 @@ namespace boost
 
 //  impute_library  ----------------------------------------------------------//
 
+    // may return an empty string [gps]
     string impute_library( const path & full_dir_path )
     {
       path relative( relative_to( full_dir_path, fs::initial_path() ),
         fs::no_check );
       if ( relative.empty() ) return "boost-root";
       string first( *relative.begin() );
-      string second =  // borland 5.61 requires op=  
+      string second =  // borland 5.61 requires op=
         ++relative.begin() == relative.end()
           ? string() : *++relative.begin();
 
       if ( first == "boost" )
-        return second.empty() ? string( "unknown" ) : second;
+        return second;
 
       return (( first == "libs" || first == "tools" ) && !second.empty())
         ? second : first;
@@ -397,19 +558,21 @@ namespace boost
 
 //  cpp_main()  --------------------------------------------------------------//
 
-#include <boost/test/included/prg_exec_monitor.hpp>
-
-int cpp_main( int argc, char * argv[] )
+int cpp_main( int argc_param, char * argv_param[] )
 {
-  fs::initial_path();
+  // <hack> for the moment, let's be on the safe side
+  // and ensure we don't modify anything being pointed to;
+  // then we'll do some cleanup here
+  int argc = argc_param;
+  const char* const * argv = &argv_param[0];
 
   if ( argc > 1 && (std::strcmp( argv[1], "-help" ) == 0
     || std::strcmp( argv[1], "--help" ) == 0 ) )
   {
-    std::clog << "Usage: inspect [-cvs] [options...]\n"
-      "options:\n"
-      << options();
-    return 1;
+    std::clog << "Usage: inspect [-cvs] [-text] [-brief] [options...]\n\n"
+      " Options:\n"
+      << options() << '\n';
+    return 0;
   }
 
   bool license_ck = true;
@@ -419,6 +582,7 @@ int cpp_main( int argc, char * argv[] )
   bool long_name_ck = true;
   bool tab_ck = true;
   bool minmax_ck = true;
+  bool unnamed_ck = true;
   bool cvs = false;
 
   if ( argc > 1 && std::strcmp( argv[1], "-cvs" ) == 0 )
@@ -427,6 +591,18 @@ int cpp_main( int argc, char * argv[] )
     --argc; ++argv;
   }
 
+  if ( argc > 1 && std::strcmp( argv[1], "-text" ) == 0 )
+  {
+    display_format = display_text;
+    --argc; ++argv;
+  }
+
+  if ( argc > 1 && std::strcmp( argv[1], "-brief" ) == 0 )
+  {
+    display_mode = display_brief;
+    --argc; ++argv;
+  }
+
   if ( argc > 1 && *argv[1] == '-' )
   {
     license_ck = false;
@@ -436,8 +612,10 @@ int cpp_main( int argc, char * argv[] )
     long_name_ck = false;
     tab_ck = false;
     minmax_ck = false;
+    unnamed_ck = false;
   }
 
+  bool invalid_options = false;
   for(; argc > 1; --argc, ++argv )
   {
     if ( std::strcmp( argv[1], "-license" ) == 0 )
@@ -450,19 +628,35 @@ int cpp_main( int argc, char * argv[] )
       link_ck = true;
     else if ( std::strcmp( argv[1], "-long_name" ) == 0 )
       long_name_ck = true;
+    else if ( std::strcmp( argv[1], "-long-name" ) == 0 )
+      long_name_ck = true;
     else if ( std::strcmp( argv[1], "-tab" ) == 0 )
       tab_ck = true;
     else if ( std::strcmp( argv[1], "-minmax" ) == 0 )
         minmax_ck = true;
+    else if ( std::strcmp( argv[1], "-unnamed" ) == 0 )
+        unnamed_ck = true;
     else
     {
-      std::cerr << "unknown option: " << argv[1]
-      << "\nvalid options are:\n"
-      << options();
-      return 1;
+      std::cerr << "unknown option: " << argv[1] << '\n';
+      invalid_options = true;
     }
   }
+  if ( invalid_options ) {
+      std::cerr << "\nvalid options are:\n"
+                << options();
+      return 1;
+  }
 
+  string inspector_keys;
+  fs::initial_path();
+  
+
+  {
+
+  // note how this is in its own block; reporting will happen
+  // automatically, from each registered inspector, when
+  // leaving, due to destruction of the inspector_list object
   inspector_list inspectors;
 
   if ( license_ck )
@@ -474,11 +668,13 @@ int cpp_main( int argc, char * argv[] )
   if ( link_ck )
     inspectors.push_back( inspector_element( new boost::inspect::link_check ) );
   if ( long_name_ck )
-    inspectors.push_back( inspector_element( new boost::inspect::long_name_check ) );
+    inspectors.push_back( inspector_element( new boost::inspect::file_name_check ) );
   if ( tab_ck )
       inspectors.push_back( inspector_element( new boost::inspect::tab_check ) );
   if ( minmax_ck )
       inspectors.push_back( inspector_element( new boost::inspect::minmax_check ) );
+  if ( unnamed_ck )
+      inspectors.push_back( inspector_element( new boost::inspect::unnamed_namespace_check ) );
 
   // perform the actual inspection, using the requested type of iteration
   if ( cvs )
@@ -495,43 +691,93 @@ int cpp_main( int argc, char * argv[] )
     itr->inspector->close();
   }
 
-  char run_date[128];
-  std::time_t tod;
-  std::time( &tod );
-  std::strftime( run_date, sizeof(run_date),
-    "%X UTC, %A %d %B %Y", std::gmtime( &tod ) );
+  string run_date ( "n/a" );
+  boost::time_string( run_date );
 
-  std::cout << "<html>\n"
-          "<head>\n"
-          "<title>Boost Inspection Report</title>\n"
-          "</head>\n"
-          "<body bgcolor=\"#ffffff\" text=\"#000000\">\n"
-          "<table border=\"0\">\n"
-          "<tr>\n"
-          "<td><img border=\"0\" src=\"../boost.png\" width=\"277\" "
-          "height=\"86\"></td>\n"
-          "<td align=\"center\">\n"
-          "<h1>Boost Inspection Report</h1>\n"
-          "<b>Run Date:</b> " << run_date  << "\n"
-          "</td>\n"
-          "</table>\n"
-          "<p>An <a href=\"http://www.boost.org/tools/inspect/index.html\">inspection\n" 
-          "program</a> checks each file in the current Boost CVS for various problems,\n" 
-          "generating this web page as output. Problems detected include tabs in files,\n" 
-          "missing copyrights, broken URL's, and similar misdemeanors.</p>\n"
-          ;
+  if (display_text == display_format)
+  {
+    std::cout
+      <<
+        "Boost Inspection Report\n"
+        "Run Date: " << run_date  << "\n"
+        "\n"
+        "An inspection program <http://www.boost.org/tools/inspect/index.html>\n"
+        "checks each file in the current Boost CVS for various problems,\n"
+        "generating an HTML page as output.\n"
+        "\n"
+      ;
 
-  std::cout << "<h2>Totals</h2>\n<pre>"
-            << file_count << " files scanned\n"
-            << directory_count << " directories scanned\n"
-            << error_count << " problems reported\n";
+    std::cout
+      << "Totals:\n"
+      << "  " << file_count << " files scanned\n"
+      << "  " << directory_count << " directories scanned (including root)\n"
+      << "  " << error_count << " problems reported\n"
+      << '\n'
+      ;
+  }
+  else
+  {
+    //
+    std::cout << doctype_declaration() << '\n';
 
-  std::cout << "\nproblem counts:\n";
+    std::cout
+      << "<html>\n"
+      "<head>\n"
+      "<title>Boost Inspection Report</title>\n"
+      "</head>\n"
+
+      "<body>\n"
+      // we should not use a table, of course [gps]
+      "<table>\n"
+      "<tr>\n"
+      "<td><img src=\"http://www.boost.org/boost.png\" alt=\"Boost logo\" />"
+      "</td>\n"
+      "<td>\n"
+      "<h1>Boost Inspection Report</h1>\n"
+      "<b>Run Date:</b> " << run_date  << "\n"
+      //"&nbsp;&nbsp;/ " << validator_link( "validate me" ) << " /\n"
+      "</td>\n"
+      "</tr>\n"
+      "</table>\n"
+
+      "<p>This report is generated by an <a href=\"http://www.boost.org/tools/inspect/index.html\">inspection\n"
+      "program</a> that checks files for the problems noted below.</p>\n"
+      ;
+    std::cout
+      << "<p>The files checked were from "
+      << info( fs::initial_path() )
+      << ".</p>\n";
+
+
+    std::cout
+      << "<h2>Totals</h2>\n<pre>"
+      << file_count << " files scanned\n"
+      << directory_count << " directories scanned (including root)\n"
+      << error_count << " problems reported\n";
+  }
 
   for ( inspector_list::iterator itr = inspectors.begin();
         itr != inspectors.end(); ++itr )
   {
-    itr->inspector.reset();
+    const string line_break (
+        display_text == display_format? "\n" : "<br>\n"); // gps
+
+    inspector_keys += static_cast<string>("  ")
+        + itr->inspector->name()
+        + ' ' + itr->inspector->desc()
+        + line_break
+        ;
+  }
+
+  
+  std::cout
+      << "\nProblem counts:\n";
+
+  } // end of block: starts reporting
+
+  if (display_text == display_format)
+  {
+    std::cout << "\n" ;
   }
 
   std::sort( msgs.begin(), msgs.end() );
@@ -539,10 +785,28 @@ int cpp_main( int argc, char * argv[] )
   if ( !msgs.empty() )
   {
     display_summary();
+
+    if (display_text == display_format)
+    {
+      std::cout << "Details:\n" << inspector_keys;
+    }
+    else
+    {
+      std::cout << "<h2>Details</h2>\n" << inspector_keys;
+    }
     display_details();
   }
 
-  std::cout << "</body>\n"
-               "</html>\n";
+  if (display_text == display_format)
+  {
+    std::cout << "\n\n" ;
+  }
+  else
+  {
+    std::cout
+      << "</body>\n"
+      "</html>\n";
+  }
+
   return 0;
 }
